@@ -1,44 +1,10 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Hkan Waara <hwaara@chello.se>
- *   Andrew Sutherland <asutherland@asutherland.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource:///modules/MailUtils.js");
 Components.utils.import("resource://gre/modules/PluralForm.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
 
 var gCurrentFolder;
 
@@ -283,7 +249,7 @@ function searchOnLoad()
   });
 
   if (window.arguments && window.arguments[0])
-      selectFolder(window.arguments[0].folder);
+      updateSearchFolderPicker(window.arguments[0].folder);
 
   // trigger searchTermOverlay.js to create the first criterion
   onMore(null);
@@ -331,27 +297,10 @@ function onResetSearch(event) {
   gStatusFeedback.showStatusString("");
 }
 
-function selectFolder(folder)
+function updateSearchFolderPicker(folder)
 {
-    var folderURI;
-
-    // if we can't search messages on this folder, just select the first one
-    if (!folder || !folder.server.canSearchMessages ||
-        (folder.flags & Components.interfaces.nsMsgFolderFlags.Virtual)) {
-        // find first item in our folder picker menu list
-        folderURI = gFolderPicker.firstChild.tree.builderView.getResourceAtIndex(0).Value;
-    } else {
-        folderURI = folder.URI;
-    }
-    updateSearchFolderPicker(folderURI);
-}
-
-function updateSearchFolderPicker(folderURI)
-{
-  SetFolderPicker(folderURI, gFolderPicker.id);
-
-  // use the URI to get the real folder
-  gCurrentFolder = GetMsgFolderFromUri(folderURI);
+  gCurrentFolder = folder;
+  gFolderPicker.menupopup.selectFolder(folder);
 
   var searchOnline = document.getElementById("checkSearchOnline");
   // We will hide and disable the search online checkbox if we are offline, or
@@ -359,9 +308,7 @@ function updateSearchFolderPicker(folderURI)
 
   // Any offlineSupportLevel > 0 is an online server like IMAP or news.
   if (gCurrentFolder.server.offlineSupportLevel &&
-      !Components.classes["@mozilla.org/network/io-service;1"]
-                         .getService(Components.interfaces.nsIIOService)
-                         .offline)
+      !Services.io.offline)
   {
     searchOnline.hidden = false;
     searchOnline.disabled = false;
@@ -384,13 +331,6 @@ function UpdateAfterCustomHeaderChange()
   updateSearchAttributes();
 }
 
-function onChooseFolder(event) {
-    var folderURI = event.id;
-    if (folderURI) {
-        updateSearchFolderPicker(folderURI);
-    }
-}
-
 function onEnterInSearchTerm()
 {
   // on enter
@@ -630,8 +570,8 @@ function MoveMessageInSearch(destFolder)
   if (destUri.length == 0)
     destUri = destFolder.getAttribute('file-uri');
 
-  let destMsgFolder = GetMsgFolderFromUri(destUri).QueryInterface(
-                        Components.interfaces.nsIMsgFolder);
+  let destMsgFolder = MailUtils.getFolderForURI(destUri)
+    .QueryInterface(Components.interfaces.nsIMsgFolder);
 
   gFolderDisplay.hintAboutToDeleteMessages();
   gFolderDisplay.doCommandWithFolder(nsMsgViewCommandType.moveMessages,
@@ -661,9 +601,7 @@ function saveAsVirtualFolder()
   var dialog = window.openDialog("chrome://messenger/content/virtualFolderProperties.xul", "",
                                  "chrome,titlebar,modal,centerscreen",
                                  {folder: window.arguments[0].folder,
-                                  searchTerms: toXPCOMArray(getSearchTerms(),
-                                                            Components.interfaces.nsISupportsArray),
+                                  searchTerms: getSearchTerms(),
                                   searchFolderURIs: searchFolderURIs,
                                   searchOnline: doOnlineSearch});
 }
-
