@@ -1,54 +1,26 @@
-/* ***** BEGIN LICENSE BLOCK *****
- *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Thunderbird Mail Client.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Blake Winton <bwinton@latte.ca>
- *   Dan Mosedale <dmose@mozillamessaging.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/*
- * Test that we can add a tag to a message without messing up the header.
+/**
+ * Test functionality in the message header, e.g. tagging, contact editing,
+ * the more button ...
  */
+
+// make SOLO_TEST=message-header/test-message-header.js mozmill-one
+
 var MODULE_NAME = 'test-message-header';
 
 var RELATIVE_ROOT = '../shared-modules';
 var MODULE_REQUIRES = ['folder-display-helpers', 'window-helpers',
-                       'address-book-helpers'];
+                       'address-book-helpers', 'dom-helpers'];
 
 var elib = {};
 Cu.import('resource://mozmill/modules/elementslib.js', elib);
+Cu.import("resource:///modules/mailServices.js");
+Cu.import("resource://gre/modules/Services.jsm");
 
-var folder;
+var folder, folderMore;
 var gInterestingMessage;
 
 function setupModule(module) {
@@ -58,8 +30,11 @@ function setupModule(module) {
   wh.installInto(module);
   let abh = collector.getModule('address-book-helpers');
   abh.installInto(module);
+  let dh = collector.getModule('dom-helpers');
+  dh.installInto(module);
 
   folder = create_folder("MessageWindowA");
+  folderMore = create_folder("MesageHeaderMoreButton");
 
   // create a message that has the interesting headers that commonly
   // show up in the message header pane for testing
@@ -74,6 +49,18 @@ function setupModule(module) {
 
   add_message_to_folder(folder, gInterestingMessage);
 
+  // create a message that has more to and cc addresses than visible in the
+  // tooltip text of the more button
+  let msgMore1 = create_message({to: msgGen.makeNamesAndAddresses(40),
+                                 cc: msgGen.makeNamesAndAddresses(40)});
+  add_message_to_folder(folderMore, msgMore1);
+
+  // create a message that has more to and cc addresses than visible in the
+  // header
+  let msgMore2 = create_message({to: msgGen.makeNamesAndAddresses(20),
+                                 cc: msgGen.makeNamesAndAddresses(20)});
+  add_message_to_folder(folderMore, msgMore2);
+
   // create a message that has boring headers to be able to switch to and
   // back from, to force the more button to collapse again.
   let msg = create_message();
@@ -218,7 +205,9 @@ function test_clicking_star_opens_inline_contact_editor()
   // Click on the star, and ensure that the inline contact
   // editing panel opens
   mc.click(mc.aid(lastAddr, {class: 'emailStar'}));
-  assert_equals(contactPanel.state, "open");
+  mc.waitFor(function() contactPanel.state == "open",
+             "Timeout waiting for contactPanel to open; state=" +
+             contactPanel.state);
   contactPanel.hidePopup();
 }
 
@@ -238,9 +227,7 @@ function assert_shown(id, visible) {
  * Test that clicking references context menu works properly.
  */
 function test_msg_id_context_menu() {
-  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
-    .getService(Ci.nsIPrefService).getBranch(null);
-  prefBranch.setBoolPref("mailnews.headers.showReferences", true);
+  Services.prefs.setBoolPref("mailnews.headers.showReferences", true);
 
   // Add a new message
   let msg = create_message({
@@ -264,7 +251,7 @@ function test_msg_id_context_menu() {
 
   close_popup(mc, mc.eid("messageIdContext"));
 
-  prefBranch.setBoolPref("mailnews.headers.showReferences", false);
+  Services.prefs.setBoolPref("mailnews.headers.showReferences", false);
 }
 
 /**
@@ -417,8 +404,8 @@ function test_add_contact_from_context_menu() {
 function test_that_msg_without_date_clears_previous_headers() {
   be_in_folder(folder);
 
-  // create a message
-  let msg = create_message();
+  // create a message: with descritive subject
+  let msg = create_message({subject: "this is without date" });
 
   // ensure that this message doesn't have a Date header
   delete msg.headers.Date;
@@ -426,8 +413,9 @@ function test_that_msg_without_date_clears_previous_headers() {
   // this will add the message to the end of the folder
   add_message_to_folder(folder, msg);
 
-  // select and open the first message
-  let curMessage = select_click_row(0);
+  // Not the first anymore. The timestamp is that of "NOW".
+  // select and open the LAST message
+  let curMessage = select_click_row(-1);
 
   // make sure it loads
   wait_for_message_display_completion(mc);
@@ -439,7 +427,7 @@ function test_that_msg_without_date_clears_previous_headers() {
   // certain bugs in the display of this header could cause the collapse
   // never to have happened.
   if (mc.e("expandednewsgroupsRow").collapsed != true) {
-    throw new Error("Expected <row> elemnent for Newsgroups header to be " +
+    throw new Error("Expected <row> element for Newsgroups header to be " +
                     "collapsed, but it wasn't\n!");
   }
 }
@@ -450,13 +438,15 @@ function test_that_msg_without_date_clears_previous_headers() {
 function test_more_widget() {
   // generate message with 35 recips (effectively guarantees overflow for n=3)
   be_in_folder(folder);
-  let msg = create_message({toCount: 35});
+  let msg = create_message({toCount: 35,
+                            subject: "Many To addresses to test_more_widget" });
 
   // add the message to the end of the folder
   add_message_to_folder(folder, msg);
 
-  // select and open the last message
-  let curMessage = select_click_row(-1);
+ // select and open the injected message;
+ // It is at the second last message in the display list.
+ let curMessage = select_click_row(-2);
 
   // make sure it loads
   wait_for_message_display_completion(mc);
@@ -468,6 +458,26 @@ function test_more_widget() {
   subtest_more_widget_display(toDescription);
   subtest_more_widget_click(toDescription);
   subtest_more_widget_star_click(toDescription);
+
+  let showNLinesPref = Services.prefs.getIntPref("mailnews.headers.show_n_lines_before_more");
+  Services.prefs.clearUserPref("mailnews.headers.show_n_lines_before_more");
+  change_to_header_normal_mode();
+  be_in_folder(folderMore);
+
+  // first test a message with so many addresses that they don't fit in the
+  // more widget's tooltip text
+  let msg = select_click_row(0);
+  wait_for_message_display_completion(mc);
+  assert_selected_and_displayed(mc, msg);
+  subtest_more_button_tooltip(msg);
+
+  // then test a message with so many addresses that they do fit in the
+  // more widget's tooltip text
+  msg = select_click_row(1);
+  wait_for_message_display_completion(mc);
+  assert_selected_and_displayed(mc, msg);
+  subtest_more_button_tooltip(msg);
+  Services.prefs.setIntPref("mailnews.headers.show_n_lines_before_more", showNLinesPref);
 }
 
 /**
@@ -476,13 +486,15 @@ function test_more_widget() {
 function test_show_all_header_mode() {
   // generate message with 35 recips (effectively guarantees overflow for n=3)
   be_in_folder(folder);
-  let msg = create_message({toCount: 35});
+  let msg = create_message({toCount: 35,
+			    subject: "many To addresses for test_show_all_header_mode" });
 
   // add the message to the end of the folder
   add_message_to_folder(folder, msg);
 
-  // select and open the last message
-  let curMessage = select_click_row(-1);
+  // select and open the added message.
+  // It is at the second last position in the display list.
+  let curMessage = select_click_row(-2);
 
   // make sure it loads
   wait_for_message_display_completion(mc);
@@ -533,9 +545,7 @@ function subtest_more_widget_display(toDescription) {
   let numLines = help_get_num_lines(toDescription);
 
   // get maxline pref
-  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
-    .getService(Ci.nsIPrefService).getBranch(null);
-  let maxLines = prefBranch.getIntPref(
+  let maxLines = Services.prefs.getIntPref(
     "mailnews.headers.show_n_lines_before_more");
 
   // allow for a 15% tolerance for any padding that may be applied
@@ -626,12 +636,11 @@ function subtest_more_widget_star_click(toDescription) {
 function test_more_widget_with_maxlines_of_3(){
 
   // set maxLines to 3
-  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
-    .getService(Ci.nsIPrefService).getBranch(null);
-  let maxLines = prefBranch.setIntPref(
+  let maxLines = Services.prefs.setIntPref(
     "mailnews.headers.show_n_lines_before_more", 3);
 
   // call test_more_widget again
+  // We need to look at the second last article in the display list.
   test_more_widget();
 }
 
@@ -642,9 +651,7 @@ function test_more_widget_with_maxlines_of_3(){
 function test_more_widget_with_disabled_more(){
 
   // set maxLines to 0
-  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
-    .getService(Ci.nsIPrefService).getBranch(null);
-  let maxLines = prefBranch.setIntPref(
+  let maxLines = Services.prefs.setIntPref(
     "mailnews.headers.show_n_lines_before_more", 0);
 
   // generate message with 35 recips (effectively guarantees overflow for n=3)
@@ -749,6 +756,128 @@ function test_toolbar_collapse_and_expand() {
     mc.window.resizeTo(1024, 768);
   }
 }
+
+/**
+ * Test if the tooltip text of the more widget contains the correct addresses
+ * not shown in the header and the number of addresses also hidden in the
+ * tooltip text.
+ * @param aMsg the message for which the subtest should be performed
+ */
+function subtest_more_button_tooltip(aMsg) {
+  // check for more indicator number of the more widget
+  let addresses = {};
+  let fullNames = {};
+  let names = {};
+  let numAddrsCC = MailServices.headerParser.parseHeadersWithArray(
+    aMsg.ccList, addresses, names, fullNames);
+  let numAddrsTo = MailServices.headerParser.parseHeadersWithArray(
+    aMsg.recipients, addresses, names, fullNames);
+
+  let shownToAddrNum = get_number_of_addresses_in_header("expandedtoBox");
+  let shownCCAddrNum = get_number_of_addresses_in_header("expandedccBox");
+
+  // first check the number of addresses in the more widget
+  let hiddenCCAddrsNum = numAddrsCC - shownCCAddrNum;
+  let hiddenToAddrsNum = numAddrsTo - shownToAddrNum;
+
+  let moreNumberTo = get_number_of_more_button("expandedtoBox");
+  assert_not_equals(NaN, moreNumberTo);
+  assert_equals(hiddenToAddrsNum, moreNumberTo);
+
+  let moreNumberCC = get_number_of_more_button("expandedccBox");
+  assert_not_equals(NaN, moreNumberCC);
+  assert_equals(hiddenCCAddrsNum, moreNumberCC);
+
+  subtest_addresses_in_tooltip_text(aMsg.recipients, "expandedtoBox",
+                                    shownToAddrNum, hiddenToAddrsNum);
+  subtest_addresses_in_tooltip_text(aMsg.ccList, "expandedccBox",
+                                    shownCCAddrNum, hiddenCCAddrsNum);
+}
+
+/**
+ * Return the number of addresses visible in headerBox.
+ * @param aHeaderBox the id of the header box element for which to look for
+ *                   visible addresses
+ * @return           the number of visible addresses in the header box
+ */
+function get_number_of_addresses_in_header(aHeaderBox) {
+  let headerBoxElement = mc.a(aHeaderBox, {class: "headerValue"});
+  let addrs = headerBoxElement.getElementsByTagName('mail-emailaddress');
+  let addrNum = 0;
+  for (let i = 0; i < addrs.length; i++) {
+    // check that the address is really visible and not just a cached
+    // element
+    if (element_visible_recursive(addrs[i]))
+      addrNum += 1;
+  }
+  return addrNum;
+}
+
+/**
+ * Return the number shown in the more widget.
+ * @param aHeaderBox the id of the header box element for which to look for
+ *                   the number in the more widget
+ * @return           the number shown in the more widget
+ */
+function get_number_of_more_button(aHeaderBox) {
+  let moreNumber = 0;
+  let headerBoxElement = mc.e(aHeaderBox);
+  let moreIndicator = headerBoxElement.more;
+  if (element_visible_recursive(moreIndicator)) {
+    let moreText = moreIndicator.getAttribute("value");
+    let moreSplit = moreText.split(" ");
+    moreNumber = parseInt(moreSplit[0])
+  }
+  return moreNumber;
+}
+
+/**
+ * Check if hidden addresses are part of more tooltip text.
+ * @param aRecipients     an array containing the addresses to look for in the
+ *                        header or the tooltip text
+ * @param aHeaderBox      the id of the header box element for which to look
+ *                        for hidden addresses
+ * @param aShownAddrsNum  the number of addresses shown in the header
+ * @param aHiddenAddrsNum the number of addresses not shown in the header
+ */
+function subtest_addresses_in_tooltip_text(aRecipients, aHeaderBox,
+                                           aShownAddrsNum, aHiddenAddrsNum) {
+  // check for more indicator number of the more widget
+  let addresses = {};
+  let fullNames = {};
+  let names = {};
+  let numAddresses = MailServices.headerParser.parseHeadersWithArray(
+      aRecipients, addresses, names, fullNames);
+
+  let headerBoxElement = mc.e(aHeaderBox);
+  let moreIndicator = headerBoxElement.more;
+  let tooltipText = moreIndicator.getAttribute("tooltiptext");
+  let maxTooltipAddrsNum = headerBoxElement.maxAddressesInMoreTooltipValue;
+  let addrsNumInTooltip = 0;
+
+  for (let i = aShownAddrsNum; (i < numAddresses) &&
+                               (i < maxTooltipAddrsNum + aShownAddrsNum); i++) {
+    assert_true(tooltipText.contains(fullNames.value[i]), fullNames.value[i]);
+    addrsNumInTooltip += 1;
+  }
+
+  if (aHiddenAddrsNum < maxTooltipAddrsNum) {
+    assert_equals(aHiddenAddrsNum, addrsNumInTooltip);
+  }
+  else {
+    assert_equals(maxTooltipAddrsNum, addrsNumInTooltip);
+    // check if ", and X more" shows the correct number
+    let moreTooltipSplit = tooltipText.split(", ");
+    let words = mc.window.document
+                         .getElementById("bundle_messenger")
+                         .getString("headerMoreAddrsTooltip");
+    let remainingAddresses = numAddresses - aShownAddrsNum - maxTooltipAddrsNum;
+    let moreForm = mc.window.PluralForm.get(remainingAddresses, words)
+                            .replace("#1", remainingAddresses);
+    assert_equals(moreForm, ", " + moreTooltipSplit[moreTooltipSplit.length - 1]);
+  }
+}
+
 // Some platforms (notably Mac) don't have a11y, so disable these tests there.
 if ("nsIAccessibleRole" in Ci) {
   /**
@@ -883,7 +1012,7 @@ if ("nsIAccessibleRole" in Ci) {
     be_in_folder(folder);
 
     // select and open the interesting message
-    
+
     let curMessage = select_click_row(mc.dbView.findIndexOfMsgHdr(
                                         gInterestingMessage, false));
 
