@@ -500,7 +500,7 @@ NS_IMETHODIMP nsAbCardProperty::TranslateTo(const nsACString &type, nsACString &
   return NS_ERROR_ILLEGAL_VALUE;
 }
 //
-static VObject* myAddPropValue(VObject *o, const char *propName, const PRUnichar *propValue, bool *aCardHasData)
+static VObject* myAddPropValue(VObject *o, const char *propName, const char16_t *propValue, bool *aCardHasData)
 {
     if (aCardHasData)
         *aCardHasData = true;
@@ -685,10 +685,10 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString &aResult)
     uint32_t format;
     rv = GetPropertyAsUint32(kPreferMailFormatProperty, &format);
     if (NS_SUCCEEDED(rv) && format == nsIAbPreferMailFormat::html) {
-        myAddPropValue(vObj, VCUseHTML, NS_LITERAL_STRING("TRUE").get(), &vCardHasData);
+        myAddPropValue(vObj, VCUseHTML, MOZ_UTF16("TRUE"), &vCardHasData);
     }
     else if (NS_SUCCEEDED(rv) && format == nsIAbPreferMailFormat::plaintext) {
-        myAddPropValue(vObj, VCUseHTML, NS_LITERAL_STRING("FALSE").get(), &vCardHasData);
+        myAddPropValue(vObj, VCUseHTML, MOZ_UTF16("FALSE"), &vCardHasData);
     }
 
     rv = GetPropertyAsAString(kWorkWebPageProperty, str);
@@ -697,7 +697,7 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString &aResult)
         myAddPropValue(vObj, VCURLProp, str.get(), &vCardHasData);
     }
 
-    myAddPropValue(vObj, VCVersionProp, NS_LITERAL_STRING("2.1").get(), nullptr);
+    myAddPropValue(vObj, VCVersionProp, MOZ_UTF16("2.1"), nullptr);
 
     if (!vCardHasData) {
         aResult.Truncate();
@@ -732,7 +732,7 @@ nsresult nsAbCardProperty::ConvertToBase64EncodedXML(nsACString &result)
     rv = stringBundleService->CreateBundle(sAddrbookProperties, getter_AddRefs(bundle));
     if (NS_SUCCEEDED(rv)) {
       nsString addrBook;
-      rv = bundle->GetStringFromName(NS_LITERAL_STRING("addressBook").get(), getter_Copies(addrBook));
+      rv = bundle->GetStringFromName(MOZ_UTF16("addressBook"), getter_Copies(addrBook));
       if (NS_SUCCEEDED(rv)) {
         xmlStr.AppendLiteral("<title xmlns=\"http://www.w3.org/1999/xhtml\">");
         xmlStr.Append(addrBook);
@@ -824,7 +824,7 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString &aXMLSubstr)
     xmlStr.AppendLiteral("<section><sectiontitle>");
 
     nsString headingAddresses;
-    rv = bundle->GetStringFromName(NS_LITERAL_STRING("headingAddresses").get(), getter_Copies(headingAddresses));
+    rv = bundle->GetStringFromName(MOZ_UTF16("headingAddresses"), getter_Copies(headingAddresses));
     NS_ENSURE_SUCCESS(rv, rv);
 
     xmlStr.Append(headingAddresses);
@@ -958,9 +958,9 @@ nsresult nsAbCardProperty::AppendLine(const AppendItem &aItem,
   if (NS_FAILED(rv) || attrValue.IsEmpty())
     return NS_OK;
 
-  aResult.Append(PRUnichar('<'));
+  aResult.Append(char16_t('<'));
   aResult.Append(NS_ConvertUTF8toUTF16(aItem.mColumn));
-  aResult.Append(PRUnichar('>'));
+  aResult.Append(char16_t('>'));
 
   // use ScanTXT to convert < > & to safe values.
   nsString safeText;
@@ -970,7 +970,7 @@ nsresult nsAbCardProperty::AppendLine(const AppendItem &aItem,
 
   aResult.AppendLiteral("</");
   aResult.Append(NS_ConvertUTF8toUTF16(aItem.mColumn));
-  aResult.Append(PRUnichar('>'));
+  aResult.Append(char16_t('>'));
 
   return NS_OK;
 }
@@ -1045,19 +1045,19 @@ nsresult nsAbCardProperty::AppendCityStateZip(const AppendItem &aItem,
   nsString formattedString;
 
   if (!cityResult.IsEmpty() && !stateResult.IsEmpty() && !zipResult.IsEmpty()) {
-    const PRUnichar *formatStrings[] = { cityResult.get(), stateResult.get(), zipResult.get() };
-    rv = aBundle->FormatStringFromName(NS_LITERAL_STRING("cityAndStateAndZip").get(), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    const char16_t *formatStrings[] = { cityResult.get(), stateResult.get(), zipResult.get() };
+    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityAndStateAndZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else if (!cityResult.IsEmpty() && !stateResult.IsEmpty() && zipResult.IsEmpty()) {
-    const PRUnichar *formatStrings[] = { cityResult.get(), stateResult.get() };
-    rv = aBundle->FormatStringFromName(NS_LITERAL_STRING("cityAndStateNoZip").get(), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    const char16_t *formatStrings[] = { cityResult.get(), stateResult.get() };
+    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityAndStateNoZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else if ((!cityResult.IsEmpty() && stateResult.IsEmpty() && !zipResult.IsEmpty()) ||
           (cityResult.IsEmpty() && !stateResult.IsEmpty() && !zipResult.IsEmpty())) {
-    const PRUnichar *formatStrings[] = { cityResult.IsEmpty() ? stateResult.get() : cityResult.get(), zipResult.get() };
-    rv = aBundle->FormatStringFromName(NS_LITERAL_STRING("cityOrStateAndZip").get(), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    const char16_t *formatStrings[] = { cityResult.IsEmpty() ? stateResult.get() : cityResult.get(), zipResult.get() };
+    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityOrStateAndZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else {
@@ -1109,15 +1109,15 @@ NS_IMETHODIMP nsAbCardProperty::GenerateName(int32_t aGenerateFormat,
     nsString result;
 
     if (aGenerateFormat == GENERATE_LAST_FIRST_ORDER) {
-      const PRUnichar *stringParams[2] = {lastName.get(), firstName.get()};
+      const char16_t *stringParams[2] = {lastName.get(), firstName.get()};
 
-      rv = bundle->FormatStringFromName(NS_LITERAL_STRING("lastFirstFormat").get(),
+      rv = bundle->FormatStringFromName(MOZ_UTF16("lastFirstFormat"),
                                         stringParams, 2, getter_Copies(result));
     }
     else {
-      const PRUnichar *stringParams[2] = {firstName.get(), lastName.get()};
-        
-      rv = bundle->FormatStringFromName(NS_LITERAL_STRING("firstLastFormat").get(),
+      const char16_t *stringParams[2] = {firstName.get(), lastName.get()};
+
+      rv = bundle->FormatStringFromName(MOZ_UTF16("firstLastFormat"),
                                         stringParams, 2, getter_Copies(result));
     }
     NS_ENSURE_SUCCESS(rv, rv); 
