@@ -24,8 +24,9 @@
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIMsgFilterService.h"
-#include "nsMemory.h"
+#include "nsIMutableArray.h"
 #include "prmem.h"
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/Services.h"
 
 static const char *kImapPrefix = "//imap:";
@@ -173,7 +174,6 @@ nsMsgFilter::nsMsgFilter():
     m_expressionTree(nullptr)
 {
   NS_NewISupportsArray(getter_AddRefs(m_termList));
-  NS_NewISupportsArray(getter_AddRefs(m_actionList));
 
   m_type = nsMsgFilterType::InboxRule | nsMsgFilterType::Manual;
 }
@@ -289,11 +289,15 @@ nsMsgFilter::CreateAction(nsIMsgRuleAction **aAction)
 //    m+1     MoveToFolder or Delete
 //    m+2     StopExecution
 NS_IMETHODIMP
-nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
+nsMsgFilter::GetSortedActionList(nsIArray **aActionList)
 {
-  NS_ENSURE_ARG_POINTER(actionList);
+  NS_ENSURE_ARG_POINTER(aActionList);
+
   uint32_t numActions;
-  nsresult rv = m_actionList->Count(&numActions);
+  nsresult rv = GetActionCount(&numActions);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMutableArray> orderedActions(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // hold separate pointers into the action list
@@ -301,9 +305,8 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
   for (uint32_t index = 0; index < numActions; ++index)
   {
     nsCOMPtr<nsIMsgRuleAction> action;
-    rv = m_actionList->QueryElementAt(index, NS_GET_IID(nsIMsgRuleAction),
-                                      (void **)getter_AddRefs(action));
-    if (!action)
+    rv = GetActionAt(index, getter_AddRefs(action));
+    if (NS_FAILED(rv) || !action)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -313,8 +316,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::FetchBodyFromPop3Server:
       {
         // always insert in front
-        // XXX Cast from bool to nsresult
-        rv = static_cast<nsresult>(actionList->InsertElementAt(action, 0));
+        rv = orderedActions->InsertElementAt(action, 0, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForNormal;
         ++nextIndexForCopy;
@@ -325,8 +327,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::CopyToFolder:
       {
         // insert into copy actions block, in order of appearance
-        // XXX Cast from bool to nsresult
-        rv = static_cast<nsresult>(actionList->InsertElementAt(action, nextIndexForCopy));
+        rv = orderedActions->InsertElementAt(action, nextIndexForCopy, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForCopy;
         ++nextIndexForMove;
@@ -337,8 +338,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::Delete:
       {
         // insert into move/delete action block
-        // XXX Cast from bool to nsresult
-        rv = static_cast<nsresult>(actionList->InsertElementAt(action, nextIndexForMove));
+        rv = orderedActions->InsertElementAt(action, nextIndexForMove, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForMove;
         break;
@@ -347,8 +347,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       case nsMsgFilterAction::StopExecution:
       {
         // insert into stop action block
-        // XXX Cast from bool to nsresult
-        rv = static_cast<nsresult>(actionList->AppendElement(action));
+        rv = orderedActions->AppendElement(action, false);
         NS_ENSURE_SUCCESS(rv, rv);
         break;
       }
@@ -356,8 +355,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       default:
       {
         // insert into normal action block, in order of appearance
-        // XXX Cast from bool to nsresult
-        rv = static_cast<nsresult>(actionList->InsertElementAt(action, nextIndexForNormal));
+        rv = orderedActions->InsertElementAt(action, nextIndexForNormal, false);
         NS_ENSURE_SUCCESS(rv, rv);
         ++nextIndexForNormal;
         ++nextIndexForCopy;
@@ -366,37 +364,54 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
       }
     }
   }
-  return rv;
+
+  orderedActions.forget(aActionList);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgFilter::AppendAction(nsIMsgRuleAction *aAction)
 {
   NS_ENSURE_ARG_POINTER(aAction);
-  return m_actionList->AppendElement(static_cast<nsISupports*>(aAction));
+
+  m_actionList.AppendElement(aAction);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgFilter::GetActionAt(int32_t aIndex, nsIMsgRuleAction **aAction)
+nsMsgFilter::GetActionAt(uint32_t aIndex, nsIMsgRuleAction **aAction)
 {
   NS_ENSURE_ARG_POINTER(aAction);
-  return m_actionList->QueryElementAt(aIndex, NS_GET_IID(nsIMsgRuleAction),
-                                      (void **) aAction);
+  NS_ENSURE_ARG(aIndex < m_actionList.Length());
+
+  NS_ENSURE_TRUE(*aAction = m_actionList[aIndex], NS_ERROR_ILLEGAL_VALUE);
+  NS_IF_ADDREF(*aAction);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgFilter::GetActionList(nsISupportsArray **actionList)
+nsMsgFilter::GetActionIndex(nsIMsgRuleAction *aAction, int32_t *aIndex)
 {
-  NS_ENSURE_ARG_POINTER(actionList);
+  NS_ENSURE_ARG_POINTER(aIndex);
 
-  NS_IF_ADDREF(*actionList = m_actionList);
+  *aIndex = m_actionList.IndexOf(aAction);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgFilter::GetActionCount(uint32_t *aCount)
+{
+  NS_ENSURE_ARG_POINTER(aCount);
+
+  *aCount = m_actionList.Length();
   return NS_OK;
 }
 
 NS_IMETHODIMP  //for editing a filter
 nsMsgFilter::ClearActionList()
 {
-  return m_actionList->Clear();
+  m_actionList.Clear();
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgFilter::GetTerm(int32_t termIndex,
@@ -518,10 +533,10 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
       getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    const PRUnichar *filterLogDetectFormatStrings[4] = { filterName.get(), authorValue.get(), subjectValue.get(), dateValue.get() };
+    const char16_t *filterLogDetectFormatStrings[4] = { filterName.get(), authorValue.get(), subjectValue.get(), dateValue.get() };
     nsString filterLogDetectStr;
     rv = bundle->FormatStringFromName(
-      NS_LITERAL_STRING("filterLogDetectStr").get(),
+      MOZ_UTF16("filterLogDetectStr"),
       filterLogDetectFormatStrings, 4,
       getter_Copies(filterLogDetectStr));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -540,12 +555,11 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
       aMsgHdr->GetMessageId(getter_Copies(msgId));
       NS_ConvertASCIItoUTF16 msgIdValue(msgId);
 
-      const PRUnichar *logMoveFormatStrings[2] = { msgIdValue.get(), actionFolderUriValue.get() };
+      const char16_t *logMoveFormatStrings[2] = { msgIdValue.get(), actionFolderUriValue.get() };
       nsString logMoveStr;
       rv = bundle->FormatStringFromName(
         (actionType == nsMsgFilterAction::MoveToFolder) ?
-          NS_LITERAL_STRING("logMoveStr").get() :
-          NS_LITERAL_STRING("logCopyStr").get(),
+          MOZ_UTF16("logMoveStr") : MOZ_UTF16("logCopyStr"),
         logMoveFormatStrings, 2,
         getter_Copies(logMoveStr));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -561,7 +575,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
         customAction->GetName(filterActionName);
       if (filterActionName.IsEmpty())
         bundle->GetStringFromName(
-                  NS_LITERAL_STRING("filterMissingCustomAction").get(),
+                  MOZ_UTF16("filterMissingCustomAction"),
                   getter_Copies(filterActionName));
       buffer += NS_ConvertUTF16toUTF8(filterActionName);
     }
@@ -777,15 +791,14 @@ nsresult nsMsgFilter::SaveRule(nsIOutputStream *aStream)
   nsAutoCString  actionFilingStr;
 
   uint32_t numActions;
-  err = m_actionList->Count(&numActions);
+  err = GetActionCount(&numActions);
   NS_ENSURE_SUCCESS(err, err);
 
-  for (uint32_t index =0; index < numActions; index++)
+  for (uint32_t index = 0; index < numActions; index++)
   {
     nsCOMPtr<nsIMsgRuleAction> action;
-    err = m_actionList->QueryElementAt(index, NS_GET_IID(nsIMsgRuleAction),
-                                       (void **)getter_AddRefs(action));
-    if (!action)
+    err = GetActionAt(index, getter_AddRefs(action));
+    if (NS_FAILED(err) || !action)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -896,7 +909,7 @@ static struct RuleActionsTableEntry ruleActionsTable[] =
   { nsMsgFilterAction::Custom,                  "Custom"},
 };
 
-static const unsigned int sNumActions = NS_ARRAY_LENGTH(ruleActionsTable);
+static const unsigned int sNumActions = MOZ_ARRAY_LENGTH(ruleActionsTable);
 
 const char *nsMsgFilter::GetActionStr(nsMsgRuleActionType action)
 {
