@@ -51,7 +51,6 @@
 #include "nsIDocShell.h"
 #include "nsIDocShellLoadInfo.h"
 #include "nsIDocShellTreeItem.h"
-#include "nsIDocShellTreeNode.h"
 #include "nsIWebNavigation.h"
 
 // mail
@@ -252,20 +251,16 @@ NS_IMETHODIMP nsMessenger::SetWindow(nsIDOMWindow *aWin, nsIMsgWindow *aMsgWindo
     nsCOMPtr<nsIDocShellTreeItem> rootDocShellAsItem;
     docShellAsItem->GetSameTypeRootTreeItem(getter_AddRefs(rootDocShellAsItem));
 
-    nsCOMPtr<nsIDocShellTreeNode> rootDocShellAsNode(do_QueryInterface(rootDocShellAsItem));
-    if (rootDocShellAsNode)
-    {
-      nsCOMPtr<nsIDocShellTreeItem> childAsItem;
-      rv = rootDocShellAsNode->FindChildWithName(NS_LITERAL_STRING("messagepane").get(),
-                                                 true, false, nullptr, nullptr, getter_AddRefs(childAsItem));
+    nsCOMPtr<nsIDocShellTreeItem> childAsItem;
+    rv = rootDocShellAsItem->FindChildWithName(MOZ_UTF16("messagepane"), true, false,
+                                               nullptr, nullptr, getter_AddRefs(childAsItem));
 
-      mDocShell = do_QueryInterface(childAsItem);
-      if (NS_SUCCEEDED(rv) && mDocShell) {
-        mCurrentDisplayCharset = ""; // Important! Clear out mCurrentDisplayCharset so we reset a default charset on mDocshell the next time we try to load something into it.
+    mDocShell = do_QueryInterface(childAsItem);
+    if (NS_SUCCEEDED(rv) && mDocShell) {
+      mCurrentDisplayCharset = ""; // Important! Clear out mCurrentDisplayCharset so we reset a default charset on mDocshell the next time we try to load something into it.
 
-        if (aMsgWindow)
-          aMsgWindow->GetTransactionManager(getter_AddRefs(mTxnMgr));
-      }
+      if (aMsgWindow)
+        aMsgWindow->GetTransactionManager(getter_AddRefs(mTxnMgr));
     }
 
     // we don't always have a message pane, like in the addressbook
@@ -327,14 +322,14 @@ nsMessenger::PromptIfFileExists(nsIFile *file)
     nsString errorMessage;
 
     file->GetPath(path);
-    const PRUnichar *pathFormatStrings[] = { path.get() };
+    const char16_t *pathFormatStrings[] = { path.get() };
 
     if (!mStringBundle)
     {
       rv = InitStringBundle();
       NS_ENSURE_SUCCESS(rv, rv);
     }
-    rv = mStringBundle->FormatStringFromName(NS_LITERAL_STRING("fileExists").get(),
+    rv = mStringBundle->FormatStringFromName(MOZ_UTF16("fileExists"),
                                              pathFormatStrings, 1,
                                              getter_Copies(errorMessage));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1265,7 +1260,7 @@ nsMessenger::GetSaveToDir(nsIFile **aSaveDir)
 
 NS_IMETHODIMP
 nsMessenger::SaveMessages(uint32_t aCount,
-                          const PRUnichar **aFilenameArray,
+                          const char16_t **aFilenameArray,
                           const char **aMessageUriArray)
 {
   NS_ENSURE_ARG_MIN(aCount, 1);
@@ -2179,7 +2174,7 @@ NS_IMETHODIMP nsMessenger::OnItemBoolPropertyChanged(nsIMsgFolder *item, nsIAtom
 }
 
 /* void OnItemUnicharPropertyChanged (in nsIMsgFolder item, in nsIAtom property, in wstring oldValue, in wstring newValue); */
-NS_IMETHODIMP nsMessenger::OnItemUnicharPropertyChanged(nsIMsgFolder *item, nsIAtom *property, const PRUnichar *oldValue, const PRUnichar *newValue)
+NS_IMETHODIMP nsMessenger::OnItemUnicharPropertyChanged(nsIMsgFolder *item, nsIAtom *property, const char16_t *oldValue, const char16_t *newValue)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -2959,14 +2954,14 @@ nsMessenger::PromptIfDeleteAttachments(bool aSaveFirst,
   {
     ConvertAndSanitizeFileName(aDisplayNameArray[u], displayString);
     attachmentList.Append(displayString);
-    attachmentList.Append(PRUnichar('\n'));
+    attachmentList.Append(char16_t('\n'));
   }
-  const PRUnichar *formatStrings[] = { attachmentList.get() };
+  const char16_t *formatStrings[] = { attachmentList.get() };
 
   // format the message and display
   nsString promptMessage;
-  const PRUnichar * propertyName = aSaveFirst ?
-    NS_LITERAL_STRING("detachAttachments").get() : NS_LITERAL_STRING("deleteAttachments").get();
+  const char16_t * propertyName = aSaveFirst ?
+    MOZ_UTF16("detachAttachments") : MOZ_UTF16("deleteAttachments");
   rv = mStringBundle->FormatStringFromName(propertyName, formatStrings, 1,getter_Copies(promptMessage));
   NS_ENSURE_SUCCESS(rv, rv);
 
