@@ -35,7 +35,6 @@
 #include "nsToolkitCompsCID.h"
 #include "nsIMsgDatabase.h"
 #include "nsIMsgHdr.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIWindowWatcher.h"
 #include "nsMsgLocalCID.h"
@@ -47,6 +46,7 @@
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "mozINewMailNotificationService.h"
+#include "mozilla/mailnews/MimeHeaderParser.h"
 
 #include <Carbon/Carbon.h>
 #import <Cocoa/Cocoa.h>
@@ -59,6 +59,8 @@
 #define kNewChatMessageTopic "new-directed-incoming-message"
 #define kUnreadImCountChangedTopic "unread-im-count-changed"
 
+using namespace mozilla::mailnews;
+
 // HACK: Limitations in Focus/SetFocus on Mac (see bug 465446)
 nsresult FocusAppNative()
 {
@@ -196,7 +198,7 @@ nsMessengerOSXIntegration::OnItemPropertyChanged(nsIMsgFolder *, nsIAtom *, char
 }
 
 NS_IMETHODIMP
-nsMessengerOSXIntegration::OnItemUnicharPropertyChanged(nsIMsgFolder *, nsIAtom *, const PRUnichar *, const PRUnichar *)
+nsMessengerOSXIntegration::OnItemUnicharPropertyChanged(nsIMsgFolder *, nsIAtom *, const char16_t *, const char16_t *)
 {
   return NS_OK;
 }
@@ -208,7 +210,7 @@ nsMessengerOSXIntegration::OnItemRemoved(nsIMsgFolder *, nsISupports *)
 }
 
 NS_IMETHODIMP
-nsMessengerOSXIntegration::Observe(nsISupports* aSubject, const char* aTopic, const PRUnichar* aData)
+nsMessengerOSXIntegration::Observe(nsISupports* aSubject, const char* aTopic, const char16_t* aData)
 {
   if (!strcmp(aTopic, "alertfinished"))
     return OnAlertFinished();
@@ -216,6 +218,13 @@ nsMessengerOSXIntegration::Observe(nsISupports* aSubject, const char* aTopic, co
   if (!strcmp(aTopic, "alertclickcallback"))
     return OnAlertClicked(aData);
 
+#ifdef MOZ_SUITE
+  // SeaMonkey does most of the GUI work in JS code when clicking on a mail
+  // notification, so it needs an extra function here 
+  if (!strcmp(aTopic, "alertclicksimplecallback"))
+    return OnAlertClickedSimple();
+#endif
+
   if (!strcmp(aTopic, "mail-startup-done")) {
     nsresult rv;
     nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
@@ -323,19 +332,19 @@ nsMessengerOSXIntegration::FillToolTipInfo(nsIMsgFolder *aFolder, int32_t aNewCo
       {
         nsAutoString numNotDisplayedText;
         numNotDisplayedText.AppendInt(numNotDisplayed);
-        const PRUnichar *formatStrings[3] = { numNewMsgsText.get(), authors.get(), numNotDisplayedText.get() };
-        bundle->FormatStringFromName(NS_LITERAL_STRING("macBiffNotification_messages_extra").get(),
+        const char16_t *formatStrings[3] = { numNewMsgsText.get(), authors.get(), numNotDisplayedText.get() };
+        bundle->FormatStringFromName(MOZ_UTF16("macBiffNotification_messages_extra"),
                                      formatStrings,
                                      3,
                                      getter_Copies(finalText));
       }
       else
       {
-        const PRUnichar *formatStrings[2] = { numNewMsgsText.get(), authors.get() };
+        const char16_t *formatStrings[2] = { numNewMsgsText.get(), authors.get() };
 
         if (aNewCount == 1)
         {
-          bundle->FormatStringFromName(NS_LITERAL_STRING("macBiffNotification_message").get(),
+          bundle->FormatStringFromName(MOZ_UTF16("macBiffNotification_message"),
                                        formatStrings,
                                        2,
                                        getter_Copies(finalText));
@@ -359,7 +368,7 @@ nsMessengerOSXIntegration::FillToolTipInfo(nsIMsgFolder *aFolder, int32_t aNewCo
           }
         }
         else
-          bundle->FormatStringFromName(NS_LITERAL_STRING("macBiffNotification_messages").get(),
+          bundle->FormatStringFromName(MOZ_UTF16("macBiffNotification_messages"),
                                        formatStrings,
                                        2,
                                        getter_Copies(finalText));
@@ -392,7 +401,7 @@ nsMessengerOSXIntegration::ShowAlertMessage(const nsAString& aAlertTitle,
                                            NS_ConvertASCIItoUTF16(aFolderURI),
                                            this, EmptyString(),
                                            NS_LITERAL_STRING("auto"),
-                                           EmptyString());
+                                           EmptyString(), nullptr);
     }
 
     BounceDockIcon();
@@ -442,12 +451,23 @@ nsMessengerOSXIntegration::OnItemIntPropertyChanged(nsIMsgFolder *aFolder,
 }
 
 nsresult
-nsMessengerOSXIntegration::OnAlertClicked(const PRUnichar* aAlertCookie)
+nsMessengerOSXIntegration::OnAlertClicked(const char16_t* aAlertCookie)
 {
   openMailWindow(NS_ConvertUTF16toUTF8(aAlertCookie));
   return NS_OK;
 }
 
+#ifdef MOZ_SUITE
+nsresult
+nsMessengerOSXIntegration::OnAlertClickedSimple()
+{
+  // SeaMonkey only function; only focus the app here, rest of the work will
+  // be done in suite/mailnews/mailWidgets.xml
+  FocusAppNative();
+  return NS_OK;
+}
+#endif
+
 nsresult
 nsMessengerOSXIntegration::OnAlertFinished()
 {
@@ -472,7 +492,7 @@ nsMessengerOSXIntegration::BounceDockIcon()
   if (mediator)
   {
     nsCOMPtr<nsIDOMWindow> domWindow;
-    mediator->GetMostRecentWindow(NS_LITERAL_STRING("mail:3pane").get(), getter_AddRefs(domWindow));
+    mediator->GetMostRecentWindow(MOZ_UTF16("mail:3pane"), getter_AddRefs(domWindow));
     if (domWindow)
     {
       nsCOMPtr<nsIDOMChromeWindow> chromeWindow(do_QueryInterface(domWindow));
@@ -596,10 +616,6 @@ nsMessengerOSXIntegration::GetNewMailAuthors(nsIMsgFolder* aFolder,
   uint32_t numNewKeys = 0;
   if (NS_SUCCEEDED(rv) && db)
   {
-    nsCOMPtr<nsIMsgHeaderParser> parser =
-      do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-
     nsCOMPtr<nsIObserverService> os =
       do_GetService("@mozilla.org/observer-service;1", &rv);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -615,7 +631,7 @@ nsMessengerOSXIntegration::GetNewMailAuthors(nsIMsgFolder* aFolder,
     if (NS_SUCCEEDED(rv))
     {
       nsString listSeparator;
-      bundle->GetStringFromName(NS_LITERAL_STRING("macBiffNotification_separator").get(), getter_Copies(listSeparator));
+      bundle->GetStringFromName(MOZ_UTF16("macBiffNotification_separator"), getter_Copies(listSeparator));
 
       int32_t displayed = 0;
       for (int32_t i = numNewKeys - 1; i >= 0; i--, aNewCount--)
@@ -633,11 +649,8 @@ nsMessengerOSXIntegration::GetNewMailAuthors(nsIMsgFolder* aFolder,
           if (NS_FAILED(rv))
             continue;
 
-          nsCString name;
-          rv = parser->ExtractHeaderAddressName(NS_ConvertUTF16toUTF8(author),
-                                                name);
-          if (NS_FAILED(rv))
-            continue;
+          nsString name;
+          ExtractName(DecodedHeader(author), name);
 
           // Give extensions a chance to suppress notifications for this author
           nsCOMPtr<nsISupportsPRBool> notify =
@@ -651,12 +664,11 @@ nsMessengerOSXIntegration::GetNewMailAuthors(nsIMsgFolder* aFolder,
           notify->GetData(&includeSender);
 
           // Don't add unwanted or duplicate names
-          if (includeSender &&
-              aAuthors.Find(name.get(), true) == -1)
+          if (includeSender && aAuthors.Find(name, true) == -1)
           {
             if (displayed > 0)
               aAuthors.Append(listSeparator);
-            aAuthors.Append(NS_ConvertUTF8toUTF16(name));
+            aAuthors.Append(name);
             displayed++;
           }
         }
