@@ -2,8 +2,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-// nsMsgPrintEngine.cpp: provides a DocShell container for use 
-// in printing
+/**
+ * nsMsgPrintEngine.cpp provides a DocShell container for use in printing.
+ */
 
 #include "nscore.h"
 #include "nsCOMPtr.h"
@@ -13,21 +14,16 @@
 #include "nsISupports.h"
 
 #include "nsIURI.h"
-#include "nsIDocShell.h"
+
 #include "nsPIDOMWindow.h"
 #include "nsIContentViewer.h"
-#include "nsIPresShell.h"
-#include "nsIDocument.h"
 #include "nsIMsgMessageService.h"
 #include "nsMsgUtils.h"
 #include "nsIWebProgress.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsMsgPrintEngine.h"
-#include "nsMsgBaseCID.h"
 #include "nsIDocumentLoader.h"
-#include "nsIWidget.h"
-#include "nsIXULWindow.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 #include "nsThreadUtils.h"
@@ -38,11 +34,9 @@
 #include "nsIBaseWindow.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIDocShellTreeItem.h"
-#include "nsIDocShellTreeNode.h"
 #include "nsIWebNavigation.h"
 #include "nsIChannel.h"
 #include "nsIContentViewerFile.h"
-#include "nsIPrintingPromptService.h"
 #include "nsServiceManagerUtils.h"
 
 static const char* kPrintingPromptService = "@mozilla.org/embedcomp/printingprompt-service;1";
@@ -84,7 +78,7 @@ nsMsgPrintEngine::OnStateChange(nsIWebProgress* aWebProgress,
     if (progressStateFlags & nsIWebProgressListener::STATE_START) {
       // Tell the user we are loading...
       nsString msg;
-      GetString(NS_LITERAL_STRING("LoadingMessageToPrint").get(), msg);
+      GetString(MOZ_UTF16("LoadingMessageToPrint"), msg);
       SetStatusMessage(msg);
     }
 
@@ -127,7 +121,7 @@ nsMsgPrintEngine::OnStateChange(nsIWebProgress* aWebProgress,
 
           // Tell the user the message is loaded...
           nsString msg;
-          GetString(NS_LITERAL_STRING("MessageLoaded").get(), msg);
+          GetString(MOZ_UTF16("MessageLoaded"), msg);
           SetStatusMessage(msg);
 
           NS_ASSERTION(mDocShell,"can't print, there is no docshell");
@@ -201,7 +195,7 @@ NS_IMETHODIMP
 nsMsgPrintEngine::OnStatusChange(nsIWebProgress* aWebProgress,
                     nsIRequest* aRequest,
                     nsresult aStatus,
-                    const PRUnichar* aMessage)
+                    const char16_t* aMessage)
 {
     return NS_OK;
 }
@@ -240,11 +234,8 @@ nsMsgPrintEngine::SetWindow(nsIDOMWindow *aWin)
   nsCOMPtr<nsIDocShellTreeItem> rootAsItem;
   docShellAsItem->GetSameTypeRootTreeItem(getter_AddRefs(rootAsItem));
 
-  nsCOMPtr<nsIDocShellTreeNode> rootAsNode(do_QueryInterface(rootAsItem));
-  NS_ENSURE_TRUE(rootAsNode, NS_ERROR_FAILURE);
-
   nsCOMPtr<nsIDocShellTreeItem> childItem;
-  rootAsNode->FindChildWithName(NS_LITERAL_STRING("content").get(), true,
+  rootAsItem->FindChildWithName(MOZ_UTF16("content"), true,
 				false, nullptr, nullptr,
 				getter_AddRefs(childItem));
 
@@ -299,7 +290,7 @@ nsMsgPrintEngine::ShowWindow(bool aShow)
 }
 
 NS_IMETHODIMP
-nsMsgPrintEngine::AddPrintURI(const PRUnichar *aMsgURI)
+nsMsgPrintEngine::AddPrintURI(const char16_t *aMsgURI)
 {
   NS_ENSURE_ARG_POINTER(aMsgURI);
 
@@ -321,7 +312,7 @@ nsMsgPrintEngine::StartPrintOperation(nsIPrintSettings* aPS)
   mPrintSettings = aPS;
 
   // Load the about:blank on the tail end...
-  nsresult rv = AddPrintURI(NS_LITERAL_STRING("about:blank").get()); 
+  nsresult rv = AddPrintURI(MOZ_UTF16("about:blank")); 
   if (NS_FAILED(rv)) return rv; 
   return StartNextPrintOperation();
 }
@@ -387,9 +378,9 @@ nsMsgPrintEngine::ShowProgressDialog(bool aIsForPrinting, bool& aDoNotify)
           NS_ADDREF(wpl);
           nsString msg;
           if (mIsDoingPrintPreview) {
-            GetString(NS_LITERAL_STRING("LoadingMailMsgForPrintPreview").get(), msg);
+            GetString(MOZ_UTF16("LoadingMailMsgForPrintPreview"), msg);
           } else {
-            GetString(NS_LITERAL_STRING("LoadingMailMsgForPrint").get(), msg);
+            GetString(MOZ_UTF16("LoadingMailMsgForPrint"), msg);
           }
           if (!msg.IsEmpty()) 
             mPrintProgressParams->SetDocTitle(msg.get());
@@ -420,7 +411,7 @@ nsMsgPrintEngine::StartNextPrintOperation()
 
     // Tell the user we are done...
     nsString msg;
-    GetString(NS_LITERAL_STRING("PrintingComplete").get(), msg);
+    GetString(MOZ_UTF16("PrintingComplete"), msg);
     SetStatusMessage(msg);
     return NS_OK;
   }
@@ -566,7 +557,7 @@ nsMsgPrintEngine::SetStatusMessage(const nsString& aMsgString)
 #define MESSENGER_STRING_URL       "chrome://messenger/locale/messenger.properties"
 
 void
-nsMsgPrintEngine::GetString(const PRUnichar *aStringName, nsString& outStr)
+nsMsgPrintEngine::GetString(const char16_t *aStringName, nsString& outStr)
 {
   outStr.Truncate();
 
@@ -609,7 +600,7 @@ nsMsgPrintEngine::PrintMsgWindow()
       // for mail, it can review the salt.  for addrbook, it's a data:// url, which
       // means nothing to the end user.
       // needs to be " " and not "" or nullptr, otherwise, we'll still print the url
-      mPrintSettings->SetDocURL(NS_LITERAL_STRING(" ").get());
+      mPrintSettings->SetDocURL(MOZ_UTF16(" "));
 
       nsresult rv = NS_ERROR_FAILURE;
       if (mIsDoingPrintPreview) 
@@ -621,15 +612,7 @@ nsMsgPrintEngine::PrintMsgWindow()
       else 
       {
         mPrintSettings->SetPrintSilent(mCurrentlyPrintingURI != 0);
-        nsCOMPtr<nsIContentViewerFile> contentViewerFile(do_QueryInterface(mWebBrowserPrint));
-        if (contentViewerFile && mParentWindow) 
-        {
-          rv = contentViewerFile->PrintWithParent(mParentWindow, mPrintSettings, (nsIWebProgressListener *)this);
-        } 
-        else 
-        {
-          rv = mWebBrowserPrint->Print(mPrintSettings, (nsIWebProgressListener *)this);
-        }
+        rv = mWebBrowserPrint->Print(mPrintSettings, (nsIWebProgressListener *)this);
       }
 
       if (NS_FAILED(rv))
@@ -751,7 +734,7 @@ NS_IMETHODIMP nsMsgPrintEngine::SetMsgType(int32_t aMsgType)
 }
 
 /*=============== nsIObserver Interface ======================*/
-NS_IMETHODIMP nsMsgPrintEngine::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData)
+NS_IMETHODIMP nsMsgPrintEngine::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *aData)
 {
   return FireThatLoadOperation(mLoadURI);
 }
