@@ -23,18 +23,21 @@
 #include "nsIStringBundle.h"
 #include "nsDateTimeFormatCID.h"
 #include "mozilla/Services.h"
-
+#include "mozilla/mailnews/MimeHeaderParser.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 #include "nsMailDirServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIDirectoryEnumerator.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsAbBaseCID.h"
 #include "nsIAbManager.h"
 #include "nsIMsgAccountManager.h"
 #include "nsMsgBaseCID.h"
 
+using namespace mozilla::mailnews;
+
 nsSpamSettings::nsSpamSettings()
 {
   mLevel = 0;
@@ -367,12 +370,12 @@ NS_IMETHODIMP nsSpamSettings::Initialize(nsIMsgIncomingServer *aServer)
     // Loop through all accounts, adding emails from this account, as well as
     // from any accounts that defer to this account.
     mEmails.Clear();
-    nsCOMPtr<nsISupportsArray> accounts;
+    nsCOMPtr<nsIArray> accounts;
     rv = accountManager->GetAccounts(getter_AddRefs(accounts));
     NS_ENSURE_SUCCESS(rv, rv);
     uint32_t accountCount = 0;
     if (account && accounts) // no sense scanning accounts if we've nothing to match
-      accounts->Count(&accountCount);
+      accounts->GetLength(&accountCount);
 
     for (uint32_t i = 0; i < accountCount; i++)
     {
@@ -391,16 +394,16 @@ NS_IMETHODIMP nsSpamSettings::Initialize(nsIMsgIncomingServer *aServer)
       // account itself.
       if (accountKey.Equals(deferredToAccountKey) || accountKey.Equals(loopAccountKey))
       {
-        nsCOMPtr<nsISupportsArray> identities;
+        nsCOMPtr<nsIArray> identities;
         loopAccount->GetIdentities(getter_AddRefs(identities));
         if (!identities)
           continue;
         uint32_t identityCount = 0;
-        identities->Count(&identityCount);
+        identities->GetLength(&identityCount);
         for (uint32_t j = 0; j < identityCount; ++j)
         {
-          nsCOMPtr<nsIMsgIdentity> identity(do_QueryElementAt(identities, j));
-          if (!identity)
+          nsCOMPtr<nsIMsgIdentity> identity(do_QueryElementAt(identities, j, &rv));
+          if (NS_FAILED(rv) || !identity)
             continue;
           nsAutoCString email;
           identity->GetEmail(email);
@@ -678,10 +681,10 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr *aMsgHdr, bool aMoveMessage
     getter_AddRefs(bundle));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  const PRUnichar *junkLogDetectFormatStrings[3] = { authorValue.get(), subjectValue.get(), dateValue.get() };
+  const char16_t *junkLogDetectFormatStrings[3] = { authorValue.get(), subjectValue.get(), dateValue.get() };
   nsString junkLogDetectStr;
   rv = bundle->FormatStringFromName(
-    NS_LITERAL_STRING("junkLogDetectStr").get(),
+    MOZ_UTF16("junkLogDetectStr"),
     junkLogDetectFormatStrings, 3,
     getter_Copies(junkLogDetectStr));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -700,10 +703,10 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr *aMsgHdr, bool aMoveMessage
     NS_ConvertASCIItoUTF16 msgIdValue(msgId);
     NS_ConvertASCIItoUTF16 junkFolderURIValue(junkFolderURI);
 
-    const PRUnichar *logMoveFormatStrings[2] = { msgIdValue.get(), junkFolderURIValue.get() };
+    const char16_t *logMoveFormatStrings[2] = { msgIdValue.get(), junkFolderURIValue.get() };
     nsString logMoveStr;
     rv = bundle->FormatStringFromName(
-      NS_LITERAL_STRING("logMoveStr").get(),
+      MOZ_UTF16("logMoveStr"),
       logMoveFormatStrings, 2,
       getter_Copies(logMoveStr));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -795,14 +798,9 @@ NS_IMETHODIMP nsSpamSettings::CheckWhiteList(nsIMsgDBHdr *aMsgHdr, bool *aResult
 
   nsCString author;
   aMsgHdr->GetAuthor(getter_Copies(author));
-  nsresult rv;
-  nsCOMPtr<nsIMsgHeaderParser> headerParser =
-    do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoCString authorEmailAddress;
-  rv = headerParser->ExtractHeaderAddressMailboxes(author, authorEmailAddress);
-  NS_ENSURE_SUCCESS(rv, rv);
+  ExtractEmail(EncodedHeader(author), authorEmailAddress);
 
   if (authorEmailAddress.IsEmpty())
     return NS_OK;
