@@ -10,6 +10,7 @@
 #include "nsAlgorithm.h"
 #include "nsMsgUtils.h"
 #include "nsIInputStream.h" // used by nsMsgLineStreamBuffer
+#include <algorithm>
 
 nsByteArray::nsByteArray()
 {
@@ -58,7 +59,7 @@ nsresult nsByteArray::AppendBuffer(const char *buffer, uint32_t length)
   nsresult ret = NS_OK;
   if (m_bufferPos + length > m_bufferSize)
     ret = GrowBuffer(m_bufferPos + length, 1024);
-  if (ret == NS_OK)
+  if (NS_SUCCEEDED(ret))
   {
     memcpy(m_buffer + m_bufferPos, buffer, length);
     m_bufferPos += length;
@@ -177,7 +178,7 @@ nsresult nsMsgLineBuffer::BufferInput(const char *net_buffer, int32_t net_buffer
     return NS_OK;
 }
 
-nsresult nsMsgLineBuffer::HandleLine(char *line, uint32_t line_length)
+nsresult nsMsgLineBuffer::HandleLine(const char *line, uint32_t line_length)
 {
   NS_ASSERTION(false, "must override this method if you don't provide a handler");
   return NS_OK;
@@ -274,9 +275,9 @@ nsMsgLineStreamBuffer::~nsMsgLineStreamBuffer()
 
 nsresult nsMsgLineStreamBuffer::GrowBuffer(int32_t desiredSize)
 {
-  m_dataBuffer = (char *) PR_REALLOC(m_dataBuffer, desiredSize);
-  if (!m_dataBuffer)
-    return NS_ERROR_OUT_OF_MEMORY;
+  char* newBuffer = (char *) PR_REALLOC(m_dataBuffer, desiredSize);
+  NS_ENSURE_TRUE(newBuffer, NS_ERROR_OUT_OF_MEMORY);
+  m_dataBuffer = newBuffer;
   m_dataBufferSize = desiredSize;
   return NS_OK;
 }
@@ -366,7 +367,7 @@ char * nsMsgLineStreamBuffer::ReadNextLine(nsIInputStream * aInputStream, uint32
       NS_ASSERTION(m_startPos == 0, "m_startPos should be 0 .....\n");
     }
     
-    uint32_t numBytesToCopy = NS_MIN(numFreeBytesInBuffer - 1 /* leave one for a null terminator */, numBytesInStream);
+    uint32_t numBytesToCopy = std::min(uint64_t(numFreeBytesInBuffer - 1) /* leave one for a null terminator */, numBytesInStream);
     if (numBytesToCopy > 0)
     {
       // read the data into the end of our data buffer
