@@ -12,7 +12,6 @@
 #include "nsIHttpProtocolHandler.h"
 #include "nsMailHeaders.h"
 #include "nsMsgI18N.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsINntpService.h"
 #include "nsMimeTypes.h"
 #include "nsDirectoryServiceDefs.h"
@@ -1362,13 +1361,7 @@ mime_fix_header_1 (const char *string, bool addr_p, bool news_p)
     return 0;
 
   if (addr_p) {
-    nsresult rv = NS_OK;
-    nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      char *n;
-      pHeader->ReformatHeaderAddresses(string, &n);
-      if (n)        return n;
-    }
+    return strdup(string);
   }
 
   old_size = PL_strlen (string);
@@ -1590,7 +1583,7 @@ msg_make_filename_qtext(const char *srcText, bool stripCRLFs)
 /* Rip apart the URL and extract a reasonable value for the `real_name' slot.
  */
 void
-msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *proposedName, const char *charset)
+msg_pick_real_name (nsMsgAttachmentHandler *attachment, const char16_t *proposedName, const char *charset)
 {
   const char *s, *s2;
 
