@@ -130,7 +130,7 @@ nsMsgSendLater::Init()
 
 NS_IMETHODIMP
 nsMsgSendLater::Observe(nsISupports *aSubject, const char* aTopic,
-                        const PRUnichar *aData)
+                        const char16_t *aData)
 {
   if (aSubject == mTimer && !strcmp(aTopic, "timer-callback"))
   {
@@ -435,7 +435,7 @@ SendOperationListener::OnProgress(const char *aMsgID, uint32_t aProgress, uint32
 }
 
 NS_IMETHODIMP
-SendOperationListener::OnStatus(const char *aMsgID, const PRUnichar *aMsg)
+SendOperationListener::OnStatus(const char *aMsgID, const char16_t *aMsg)
 {
 #ifdef NS_DEBUG
   printf("SendOperationListener::OnStatus()\n");
@@ -451,7 +451,7 @@ SendOperationListener::OnSendNotPerformed(const char *aMsgID, nsresult aStatus)
 }
   
 NS_IMETHODIMP
-SendOperationListener::OnStopSending(const char *aMsgID, nsresult aStatus, const PRUnichar *aMsg, 
+SendOperationListener::OnStopSending(const char *aMsgID, nsresult aStatus, const char16_t *aMsg, 
                                      nsIFile *returnFile)
 {
   if (mSendLater && !mSendLater->OnSendStepFinished(aStatus))
@@ -1328,7 +1328,7 @@ nsMsgSendLater::NotifyListenersOnProgress(uint32_t aCurrentMessage,
 void
 nsMsgSendLater::NotifyListenersOnMessageSendError(uint32_t aCurrentMessage,
                                                   nsresult aStatus,
-                                                  const PRUnichar *aMsg)
+                                                  const char16_t *aMsg)
 {
   NOTIFY_LISTENERS(OnMessageSendError, (aCurrentMessage, mMessage,
                                         aStatus, aMsg));
@@ -1339,7 +1339,7 @@ nsMsgSendLater::NotifyListenersOnMessageSendError(uint32_t aCurrentMessage,
  * system and notifies the relevant parties that we have finished.
  */
 void
-nsMsgSendLater::EndSendMessages(nsresult aStatus, const PRUnichar *aMsg,
+nsMsgSendLater::EndSendMessages(nsresult aStatus, const char16_t *aMsg,
                                 uint32_t aTotalTried, uint32_t aSuccessful)
 {
   // Catch-all, we may have had an issue sending, so we may not be calling
@@ -1526,8 +1526,8 @@ nsMsgSendLater::OnItemBoolPropertyChanged(nsIMsgFolder *aItem, nsIAtom *aPropert
 NS_IMETHODIMP
 nsMsgSendLater::OnItemUnicharPropertyChanged(nsIMsgFolder *aItem,
                                              nsIAtom *aProperty,
-                                             const PRUnichar* aOldValue,
-                                             const PRUnichar* aNewValue)
+                                             const char16_t* aOldValue,
+                                             const char16_t* aNewValue)
 {
   return NS_OK;
 }
