@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jean-Francois Ducarroz <ducarroz@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMsgSendReport.h"
 
@@ -42,7 +9,7 @@
 #include "nsIMsgCompose.h"
 #include "nsMsgCompCID.h"
 #include "nsMsgPrompts.h"
-#include "nsNetError.h"
+#include "nsError.h"
 #include "nsComposeStrings.h"
 #include "nsIStringBundle.h"
 #include "nsServiceManagerUtils.h"
@@ -86,13 +53,13 @@ NS_IMETHODIMP nsMsgProcessReport::SetError(nsresult aError)
 }
 
 /* attribute wstring message; */
-NS_IMETHODIMP nsMsgProcessReport::GetMessage(PRUnichar * *aMessage)
+NS_IMETHODIMP nsMsgProcessReport::GetMessage(char16_t * *aMessage)
 {
   NS_ENSURE_ARG_POINTER(aMessage);
   *aMessage = ToNewUnicode(mMessage);
   return NS_OK;
 }
-NS_IMETHODIMP nsMsgProcessReport::SetMessage(const PRUnichar * aMessage)
+NS_IMETHODIMP nsMsgProcessReport::SetMessage(const char16_t * aMessage)
 {
   mMessage = aMessage;
   return NS_OK;
@@ -113,7 +80,7 @@ NS_IMPL_ISUPPORTS1(nsMsgSendReport, nsIMsgSendReport)
 
 nsMsgSendReport::nsMsgSendReport()
 {
-  PRUint32 i;
+  uint32_t i;
   for (i = 0; i <= SEND_LAST_PROCESS; i ++)
     mProcessReport[i] = new nsMsgProcessReport();
 
@@ -122,19 +89,19 @@ nsMsgSendReport::nsMsgSendReport()
 
 nsMsgSendReport::~nsMsgSendReport()
 {
-  PRUint32 i;
+  uint32_t i;
   for (i = 0; i <= SEND_LAST_PROCESS; i ++)
-    mProcessReport[i] = nsnull;
+    mProcessReport[i] = nullptr;
 }
 
 /* attribute long currentProcess; */
-NS_IMETHODIMP nsMsgSendReport::GetCurrentProcess(PRInt32 *aCurrentProcess)
+NS_IMETHODIMP nsMsgSendReport::GetCurrentProcess(int32_t *aCurrentProcess)
 {
   NS_ENSURE_ARG_POINTER(aCurrentProcess);
   *aCurrentProcess = mCurrentProcess;
   return NS_OK;
 }
-NS_IMETHODIMP nsMsgSendReport::SetCurrentProcess(PRInt32 aCurrentProcess)
+NS_IMETHODIMP nsMsgSendReport::SetCurrentProcess(int32_t aCurrentProcess)
 {
   if (aCurrentProcess < 0 || aCurrentProcess > SEND_LAST_PROCESS)
     return NS_ERROR_ILLEGAL_VALUE;
@@ -147,13 +114,13 @@ NS_IMETHODIMP nsMsgSendReport::SetCurrentProcess(PRInt32 aCurrentProcess)
 }
 
 /* attribute long deliveryMode; */
-NS_IMETHODIMP nsMsgSendReport::GetDeliveryMode(PRInt32 *aDeliveryMode)
+NS_IMETHODIMP nsMsgSendReport::GetDeliveryMode(int32_t *aDeliveryMode)
 {
   NS_ENSURE_ARG_POINTER(aDeliveryMode);
   *aDeliveryMode = mDeliveryMode;
   return NS_OK;
 }
-NS_IMETHODIMP nsMsgSendReport::SetDeliveryMode(PRInt32 aDeliveryMode)
+NS_IMETHODIMP nsMsgSendReport::SetDeliveryMode(int32_t aDeliveryMode)
 {
   mDeliveryMode = aDeliveryMode;
   return NS_OK;
@@ -162,7 +129,7 @@ NS_IMETHODIMP nsMsgSendReport::SetDeliveryMode(PRInt32 aDeliveryMode)
 /* void Reset (); */
 NS_IMETHODIMP nsMsgSendReport::Reset()
 {
-  PRUint32 i;
+  uint32_t i;
   for (i = 0; i <= SEND_LAST_PROCESS; i ++)
     if (mProcessReport[i])
       mProcessReport[i]->Reset();
@@ -175,7 +142,7 @@ NS_IMETHODIMP nsMsgSendReport::Reset()
 }
 
 /* void setProceeded (in long process, in boolean proceeded); */
-NS_IMETHODIMP nsMsgSendReport::SetProceeded(PRInt32 process, bool proceeded)
+NS_IMETHODIMP nsMsgSendReport::SetProceeded(int32_t process, bool proceeded)
 {
   if (process < process_Current || process > SEND_LAST_PROCESS)
     return NS_ERROR_ILLEGAL_VALUE;
@@ -190,33 +157,45 @@ NS_IMETHODIMP nsMsgSendReport::SetProceeded(PRInt32 process, bool proceeded)
 }
 
 /* void setError (in long process, in nsresult error, in boolean overwriteError); */
-NS_IMETHODIMP nsMsgSendReport::SetError(PRInt32 process, nsresult newError, bool overwriteError)
+NS_IMETHODIMP nsMsgSendReport::SetError(int32_t process, nsresult newError, bool overwriteError)
 {
   if (process < process_Current || process > SEND_LAST_PROCESS)
     return NS_ERROR_ILLEGAL_VALUE;
 
   if (process == process_Current)
+  {
+    if (mCurrentProcess == process_Current)
+      // We don't know what we're currently trying to do
+      return NS_ERROR_ILLEGAL_VALUE;
+
     process = mCurrentProcess;
+  }
 
   if (!mProcessReport[process])
     return NS_ERROR_NOT_INITIALIZED;
 
   nsresult currError = NS_OK;
   mProcessReport[process]->GetError(&currError);
-  if (overwriteError || currError == NS_OK)
+  if (overwriteError || NS_SUCCEEDED(currError))
     return mProcessReport[process]->SetError(newError);
   else
     return NS_OK;
 }
 
 /* void setMessage (in long process, in wstring message, in boolean overwriteMessage); */
-NS_IMETHODIMP nsMsgSendReport::SetMessage(PRInt32 process, const PRUnichar *message, bool overwriteMessage)
+NS_IMETHODIMP nsMsgSendReport::SetMessage(int32_t process, const char16_t *message, bool overwriteMessage)
 {
   if (process < process_Current || process > SEND_LAST_PROCESS)
     return NS_ERROR_ILLEGAL_VALUE;
 
   if (process == process_Current)
+  {
+    if (mCurrentProcess == process_Current)
+      // We don't know what we're currently trying to do
+      return NS_ERROR_ILLEGAL_VALUE;
+
     process = mCurrentProcess;
+  }
 
   if (!mProcessReport[process])
     return NS_ERROR_NOT_INITIALIZED;
@@ -230,14 +209,20 @@ NS_IMETHODIMP nsMsgSendReport::SetMessage(PRInt32 process, const PRUnichar *mess
 }
 
 /* nsIMsgProcessReport getProcessReport (in long process); */
-NS_IMETHODIMP nsMsgSendReport::GetProcessReport(PRInt32 process, nsIMsgProcessReport **_retval)
+NS_IMETHODIMP nsMsgSendReport::GetProcessReport(int32_t process, nsIMsgProcessReport **_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
   if (process < process_Current || process > SEND_LAST_PROCESS)
     return NS_ERROR_ILLEGAL_VALUE;
 
   if (process == process_Current)
+  {
+    if (mCurrentProcess == process_Current)
+      // We don't know what we're currently trying to do
+      return NS_ERROR_ILLEGAL_VALUE;
+
     process = mCurrentProcess;
+  }
 
   NS_IF_ADDREF(*_retval = mProcessReport[process]);
   return NS_OK;
@@ -248,6 +233,9 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
 {
   NS_ENSURE_ARG_POINTER(_retval);
 
+  NS_ENSURE_TRUE(mCurrentProcess >= 0 && mCurrentProcess <= SEND_LAST_PROCESS,
+                 NS_ERROR_NOT_INITIALIZED);
+
   nsresult currError = NS_OK;
   mProcessReport[mCurrentProcess]->GetError(&currError);
   *_retval = currError;
@@ -315,7 +303,7 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
 
     bundle->GetStringFromID(NS_MSG_SEND_ERROR_TITLE, getter_Copies(dialogTitle));
 
-    PRInt32 preStrId = NS_ERROR_SEND_FAILED;
+    nsresult preStrId = NS_ERROR_SEND_FAILED;
     bool askToGoBackToCompose = false;
     switch (mCurrentProcess)
     {
@@ -345,7 +333,7 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
         askToGoBackToCompose = (mDeliveryMode == nsIMsgCompDeliverMode::Now);
         break;
     }
-    bundle->GetStringFromID(preStrId, getter_Copies(dialogMessage));
+    bundle->GetStringFromID(NS_ERROR_GET_CODE(preStrId), getter_Copies(dialogMessage));
 
     //Do we already have an error message?
     if (!askToGoBackToCompose && currMessage.IsEmpty())
@@ -360,7 +348,7 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
       if (!currMessage.Equals(dialogMessage))
       {
         if (!dialogMessage.IsEmpty())
-          dialogMessage.Append(PRUnichar('\n'));
+          dialogMessage.Append(char16_t('\n'));
         dialogMessage.Append(currMessage);
       }
     }
@@ -382,8 +370,8 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
   }
   else
   {
-    PRInt32 titleID;
-    PRInt32 preStrId;
+    int32_t titleID;
+    nsresult preStrId;
 
     switch (mDeliveryMode)
     {
@@ -411,8 +399,8 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
     }
 
     bundle->GetStringFromID(titleID, getter_Copies(dialogTitle));
-    // preStrId could be a string ID or it could be an error code...yuck.
-    bundle->GetStringFromID(NS_IS_MSG_ERROR(preStrId) ? NS_ERROR_GET_CODE(preStrId) : preStrId, getter_Copies(dialogMessage));
+    bundle->GetStringFromID(NS_ERROR_GET_CODE(preStrId),
+                            getter_Copies(dialogMessage));
 
     //Do we have an error message...
     if (currMessage.IsEmpty())
@@ -424,7 +412,7 @@ NS_IMETHODIMP nsMsgSendReport::DisplayReport(nsIPrompt *prompt, bool showErrorOn
     if (!currMessage.IsEmpty())
     {
       if (!dialogMessage.IsEmpty())
-        dialogMessage.Append(PRUnichar('\n'));
+        dialogMessage.Append(char16_t('\n'));
       dialogMessage.Append(currMessage);
     }
     nsMsgDisplayMessageByString(prompt, dialogMessage.get(), dialogTitle.get());
