@@ -15,7 +15,6 @@
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
 #include "nsISocketTransport.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
@@ -37,21 +36,27 @@
 #include "nsIPipe.h"
 #include "nsNetUtil.h"
 #include "nsIPrefService.h"
-#include "nsISignatureVerifier.h"
 #include "nsISSLSocketControl.h"
 #include "nsComposeStrings.h"
 #include "nsIStringBundle.h"
 #include "nsMsgCompUtils.h"
 #include "nsIMsgWindow.h"
 #include "MailNewsTypes2.h" // for nsMsgSocketType and nsMsgAuthMethod
+#include "nsIIDNService.h"
+#include "mozilla/mailnews/MimeHeaderParser.h"
 #include "mozilla/Services.h"
+#include "nsINetAddr.h"
 
 #ifndef XP_UNIX
 #include <stdarg.h>
 #endif /* !XP_UNIX */
 
+#undef PostMessage // avoid to collision with WinUser.h
+
 static PRLogModuleInfo *SMTPLogModule = nullptr;
 
+using namespace mozilla::mailnews;
+
 /* the output_buffer_size must be larger than the largest possible line
  * 2000 seems good for news
  *
@@ -78,7 +83,7 @@ nsresult nsExplainErrorDetails(nsISmtpUrl * aSmtpUrl, nsresult code, ...)
   aSmtpUrl->GetPrompt(getter_AddRefs(dialog));
   NS_ENSURE_TRUE(dialog, NS_ERROR_FAILURE);
 
-  PRUnichar *  msg;
+  char16_t *  msg;
   nsString eMsg;
   nsCOMPtr<nsIStringBundleService> bundleService =
     mozilla::services::GetStringBundleService();
@@ -91,6 +96,13 @@ nsresult nsExplainErrorDetails(nsISmtpUrl * aSmtpUrl, nsresult code, ...)
 
   switch (code)
   {
+    case NS_ERROR_ILLEGAL_LOCALPART:
+      bundle->GetStringFromName(
+        MOZ_UTF16("errorIllegalLocalPart"),
+        getter_Copies(eMsg));
+      msg = nsTextFormatter::vsmprintf(eMsg.get(), args);
+      break;
+
       case NS_ERROR_SMTP_SERVER_ERROR:
       case NS_ERROR_TCP_READ_ERROR:
       case NS_ERROR_SMTP_TEMP_SIZE_EXCEEDED:
@@ -161,7 +173,7 @@ nsresult nsExplainErrorDetails(nsISmtpUrl * aSmtpUrl, nsresult code, ...)
  */
 /* caller must free the return buffer */
 static char *
-esmtp_value_encode(char *addr)
+esmtp_value_encode(const char *addr)
 {
   char *buffer = (char *) PR_Malloc(512); /* esmtp ORCPT allow up to 500 chars encoded addresses */
   char *bp = buffer, *bpEnd = buffer+500;
@@ -210,7 +222,6 @@ nsSmtpProtocol::nsSmtpProtocol(nsIURI * aURL)
 nsSmtpProtocol::~nsSmtpProtocol()
 {
   // free our local state
-  PR_Free(m_addressCopy);
   PR_Free(m_dataBuf);
   delete m_lineStreamBuffer;
 }
@@ -250,14 +261,8 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     m_previousResponseCode = 0;
     m_continuationResponse = -1;
     m_tlsEnabled = false;
-    m_addressCopy = nullptr;
-    m_addresses = nullptr;
     m_addressesLeft = 0;
 
-#ifdef UNREADY_CODE
-    m_totalAmountWritten = 0;
-#endif /* UNREADY_CODE */
-
     m_sendDone = false;
 
     m_sizelimit = 0;
@@ -335,23 +340,31 @@ void nsSmtpProtocol::AppendHelloArgument(nsACString& aResult)
       }
       else
       {
-          PRNetAddr iaddr; // IP address for this connection
+          nsCOMPtr<nsINetAddr> iaddr; // IP address for this connection
           // our transport is always a nsISocketTransport
           nsCOMPtr<nsISocketTransport> socketTransport = do_QueryInterface(m_transport);
           // should return the interface ip of the SMTP connection
           // minimum case - see bug 68877 and RFC 2821, chapter 4.1.1.1
-          rv = socketTransport->GetSelfAddr(&iaddr);
+          rv = socketTransport->GetScriptableSelfAddr(getter_AddRefs(iaddr));
 
           if (NS_SUCCEEDED(rv))
           {
               // turn it into a string
-              char ipAddressString[64];
-              if (PR_NetAddrToString(&iaddr, ipAddressString, sizeof(ipAddressString)) == PR_SUCCESS)
+              nsCString ipAddressString;
+              rv = iaddr->GetAddress(ipAddressString);
+              if (NS_SUCCEEDED(rv))
               {
-                  NS_ASSERTION(!PR_IsNetAddrType(&iaddr, PR_IpAddrV4Mapped),
+#ifdef DEBUG
+                  bool v4mapped = false;
+                  iaddr->GetIsV4Mapped(&v4mapped);
+                  NS_ASSERTION(!v4mapped,
                                "unexpected IPv4-mapped IPv6 address");
+#endif
 
-                  if (iaddr.raw.family == PR_AF_INET6)   // IPv6 style address?
+                  uint16_t family = nsINetAddr::FAMILY_INET;
+                  iaddr->GetFamily(&family);
+
+                  if (family == nsINetAddr::FAMILY_INET6) // IPv6 style address?
                       aResult.AppendLiteral("[IPv6:");
                   else
                       aResult.AppendLiteral("[");
@@ -373,7 +386,7 @@ void nsSmtpProtocol::AppendHelloArgument(nsACString& aResult)
 NS_IMETHODIMP nsSmtpProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
                                             nsresult aStatus)
 {
-  bool connDroppedDuringAuth = aStatus == NS_OK && !m_sendDone &&
+  bool connDroppedDuringAuth = NS_SUCCEEDED(aStatus) && !m_sendDone &&
       (m_nextStateAfterResponse == SMTP_AUTH_LOGIN_STEP0_RESPONSE ||
        m_nextStateAfterResponse == SMTP_AUTH_LOGIN_RESPONSE);
   // ignore errors handling the QUIT command so fcc can continue.
@@ -383,7 +396,7 @@ NS_IMETHODIMP nsSmtpProtocol::OnStopRequest(nsIRequest *request, nsISupports *ct
      ("SMTP connection error quitting %lx, ignoring ", aStatus));
     aStatus = NS_OK;
   }
-  if (aStatus == NS_OK && !m_sendDone) {
+  if (NS_SUCCEEDED(aStatus) && !m_sendDone) {
     // if we are getting OnStopRequest() with NS_OK,
     // but we haven't finished clean, that's spells trouble.
     // it means that the server has dropped us before we could send the whole mail
@@ -434,7 +447,7 @@ void nsSmtpProtocol::UpdateStatus(int32_t aStatusID)
   }
 }
 
-void nsSmtpProtocol::UpdateStatusWithString(const PRUnichar * aStatusString)
+void nsSmtpProtocol::UpdateStatusWithString(const char16_t * aStatusString)
 {
   if (m_statusFeedback && aStatusString)
     m_statusFeedback->ShowStatusString(nsDependentString(aStatusString));
@@ -586,17 +599,9 @@ nsresult nsSmtpProtocol::SendHeloResponse(nsIInputStream * inputStream, uint32_t
     return(NS_ERROR_COULD_NOT_GET_USERS_MAIL_ADDRESS);
   }
 
-  nsCOMPtr<nsIMsgHeaderParser> parser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
   nsCString fullAddress;
-  if (parser)
-  {
-    // pass nullptr for the name, since we just want the email.
-    //
-    // seems a little weird that we are passing in the emailAddress
-    // when that's the out parameter
-    parser->MakeFullAddressString(nullptr, emailAddress.get(),
-                                  getter_Copies(fullAddress));
-  }
+  // Quote the email address before passing it to the SMTP server.
+  MakeMimeAddress(EmptyCString(), emailAddress, fullAddress);
 
   buffer = "MAIL FROM:<";
   buffer += fullAddress;
@@ -723,34 +728,35 @@ nsresult nsSmtpProtocol::SendEhloResponse(nsIInputStream * inputStream, uint32_t
         }
         else if (StringBeginsWith(responseLine, NS_LITERAL_CSTRING("AUTH"), nsCaseInsensitiveCStringComparator()))
         {
-            SetFlag(SMTP_AUTH);
+          SetFlag(SMTP_AUTH);
 
-            if (responseLine.Find(NS_LITERAL_CSTRING("GSSAPI"), CaseInsensitiveCompare) >= 0)
-                SetFlag(SMTP_AUTH_GSSAPI_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("GSSAPI"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_GSSAPI_ENABLED);
 
-            nsresult rv;
-            nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
-            // this checks if psm is installed...
-            if (NS_SUCCEEDED(rv))
-            {
-                if (responseLine.Find(NS_LITERAL_CSTRING("CRAM-MD5"), CaseInsensitiveCompare) >= 0)
-                    SetFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("CRAM-MD5"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
 
-                if (responseLine.Find(NS_LITERAL_CSTRING("NTLM"), CaseInsensitiveCompare) >= 0)
-                    SetFlag(SMTP_AUTH_NTLM_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("NTLM"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_NTLM_ENABLED);
 
-                if (responseLine.Find(NS_LITERAL_CSTRING("MSN"), CaseInsensitiveCompare) >= 0)
-                    SetFlag(SMTP_AUTH_MSN_ENABLED);
-            }
+          if (responseLine.Find(NS_LITERAL_CSTRING("MSN"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_MSN_ENABLED);
 
-            if (responseLine.Find(NS_LITERAL_CSTRING("PLAIN"), CaseInsensitiveCompare) >= 0)
-                SetFlag(SMTP_AUTH_PLAIN_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("PLAIN"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_PLAIN_ENABLED);
 
-            if (responseLine.Find(NS_LITERAL_CSTRING("LOGIN"), CaseInsensitiveCompare) >= 0)
-                SetFlag(SMTP_AUTH_LOGIN_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("LOGIN"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_LOGIN_ENABLED);
 
-            if (responseLine.Find(NS_LITERAL_CSTRING("EXTERNAL"), CaseInsensitiveCompare) >= 0)
-                SetFlag(SMTP_AUTH_EXTERNAL_ENABLED);
+          if (responseLine.Find(NS_LITERAL_CSTRING("EXTERNAL"),
+                                CaseInsensitiveCompare) >= 0)
+            SetFlag(SMTP_AUTH_EXTERNAL_ENABLED);
         }
         else if (StringBeginsWith(responseLine, NS_LITERAL_CSTRING("SIZE"), nsCaseInsensitiveCStringComparator()))
         {
@@ -1464,15 +1470,16 @@ nsresult nsSmtpProtocol::SendMailResponse()
   bool requestOnNever = false;
   rv = prefBranch->GetBoolPref("mail.dsn.request_never_on", &requestOnNever);
 
+  nsCString &address = m_addresses[m_addressesLeft - 1];
   if (TestFlag(SMTP_EHLO_DSN_ENABLED) && requestDSN && (requestOnSuccess || requestOnFailure || requestOnDelay || requestOnNever))
     {
-      char *encodedAddress = esmtp_value_encode(m_addresses);
+      char *encodedAddress = esmtp_value_encode(address.get());
       nsAutoCString dsnBuffer;
 
       if (encodedAddress)
       {
         buffer = "RCPT TO:<";
-        buffer += m_addresses;
+        buffer += address;
         buffer += "> NOTIFY=";
 
         if (requestOnNever)
@@ -1504,7 +1511,7 @@ nsresult nsSmtpProtocol::SendMailResponse()
     else
     {
       buffer = "RCPT TO:<";
-      buffer += m_addresses;
+      buffer += address;
       buffer += ">";
       buffer += CRLF;
     }
@@ -1534,16 +1541,14 @@ nsresult nsSmtpProtocol::SendRecipientResponse()
       errorcode = NS_ERROR_SENDING_RCPT_COMMAND;
 
     rv = nsExplainErrorDetails(m_runningURL, errorcode,
-                               m_responseText.get(), m_addresses);
+                               m_responseText.get(),
+                               m_addresses[m_addressesLeft - 1].get());
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain SMTP error");
 
     m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
     return(NS_ERROR_SENDING_RCPT_COMMAND);
   }
 
-  /* take the address we sent off the list (move the pointer to just
-     past the terminating null.) */
-  m_addresses += PL_strlen (m_addresses) + 1;
   if (--m_addressesLeft > 0)
   {
     // more senders to RCPT to
@@ -1716,45 +1721,88 @@ nsresult nsSmtpProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer )
 
   if (postMessage)
   {
-    nsCString addrs1;
-    char *addrs2 = 0;
     m_nextState = SMTP_RESPONSE;
     m_nextStateAfterResponse = SMTP_EXTN_LOGIN_RESPONSE;
 
-    // Remove duplicates from the list, to prevent people from getting
-    // more than one copy (the SMTP host may do this too, or it may not.)
-    // This causes the address list to be parsed twice; this probably
-    // doesn't matter.
+    // compile a minimal list of valid target addresses by
+    // - looking only at mailboxes
+    // - dropping addresses with invalid localparts (until we implement RFC 6532)
+    // - using ACE for IDN domainparts
+    // - stripping duplicates
     nsCString addresses;
-    nsCOMPtr<nsIMsgHeaderParser> parser =
-      do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
-
     m_runningURL->GetRecipients(getter_Copies(addresses));
 
-    if (parser)
+    ExtractEmails(EncodedHeader(addresses), UTF16ArrayAdapter<>(m_addresses));
+
+    nsCOMPtr<nsIIDNService> converter = do_GetService(NS_IDNSERVICE_CONTRACTID);
+    addresses.Truncate();
+    uint32_t count = m_addresses.Length();
+    for (uint32_t i = 0; i < count; i++)
     {
-      parser->RemoveDuplicateAddresses(addresses, EmptyCString(), addrs1);
-
-      // Extract just the mailboxes from the full RFC822 address list.
-      // This means that people can post to mailto: URLs which contain
-      // full RFC822 address specs, and we will still send the right
-      // thing in the SMTP RCPT command.
-      if (!addrs1.IsEmpty())
-        parser->ParseHeaderAddresses(addrs1.get(), nullptr, &addrs2,
-                                     &m_addressesLeft);
-
-      // hmm no addresses to send message to...
-      if (m_addressesLeft == 0 || !addrs2)
+      const char *start = m_addresses[i].get();
+      // Location of the @ character
+      const char *lastAt = nullptr;
+      const char *ch = start;
+      for (; *ch; ch++)
       {
-        m_nextState = SMTP_ERROR_DONE;
-        ClearFlag(SMTP_PAUSE_FOR_READ);
-        m_urlErrorState = NS_MSG_NO_RECIPIENTS;
-        return NS_MSG_NO_RECIPIENTS;
+        if (*ch == '@')
+          lastAt = ch;
+        // Check for first illegal character (outside 0x09,0x20-0x7e)
+        else if ((*ch < ' ' || *ch > '~') && (*ch != '\t'))
+        {
+          break;
+        }
       }
+      // validate the just parsed address
+      if (*ch || m_addresses[i].IsEmpty())
+      {
+        // Fortunately, we will always have an @ in each mailbox address.
+        // We try to fix illegal character in the domain part by converting
+        // that to ACE. Illegal characters in the local part are not fixable
+        // (which charset would it be anyway?), hence we error out in that
+        // case as well.
+        nsresult rv = NS_ERROR_FAILURE; // anything but NS_OK
+        if (lastAt)
+        {
+          // Illegal char in the domain part, hence use ACE
+          nsAutoCString domain;
+          domain.Assign(lastAt + 1);
+          rv = converter->ConvertUTF8toACE(domain, domain);
+          if (NS_SUCCEEDED(rv))
+          {
+            m_addresses[i].SetLength(lastAt - start + 1);
+            m_addresses[i] += domain;
+          }
+        }
+        if (NS_FAILED(rv))
+        {
+          // Throw an error, including the broken address
+          m_nextState = SMTP_ERROR_DONE;
+          ClearFlag(SMTP_PAUSE_FOR_READ);
+          // Unfortunately, nsExplainErrorDetails will show the error above
+          // the mailnews main window, because we don't necessarily get
+          // passed down a compose window - we might be sending in the
+          // background!
+          rv = nsExplainErrorDetails(m_runningURL, NS_ERROR_ILLEGAL_LOCALPART,
+            start);
+          NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain illegal localpart");
+          m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
+          return NS_ERROR_BUT_DONT_SHOW_ALERT;
+        }
+      }
+    }
 
-      m_addressCopy = addrs2;
-      m_addresses = m_addressCopy;
-    } // if parser
+    // final cleanup
+    m_addressesLeft = m_addresses.Length();
+
+    // hmm no addresses to send message to...
+    if (m_addressesLeft == 0)
+    {
+      m_nextState = SMTP_ERROR_DONE;
+      ClearFlag(SMTP_PAUSE_FOR_READ);
+      m_urlErrorState = NS_MSG_NO_RECIPIENTS;
+      return NS_MSG_NO_RECIPIENTS;
+    }
   } // if post message
 
   return nsMsgProtocol::LoadUrl(aURL, aConsumer);
@@ -1971,7 +2019,7 @@ nsSmtpProtocol::GetPassword(nsCString &aPassword)
     nsAutoString hostnameUTF16;
     CopyASCIItoUTF16(hostname, hostnameUTF16);
 
-    const PRUnichar *formatStrings[] =
+    const char16_t *formatStrings[] =
     {
       hostnameUTF16.get(),
       usernameUTF16.get()
@@ -1987,7 +2035,7 @@ nsSmtpProtocol::GetPassword(nsCString &aPassword)
  * is the username.
  */
 nsresult
-nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpUrl, const PRUnichar **formatStrings, nsACString &aPassword)
+nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpUrl, const char16_t **formatStrings, nsACString &aPassword)
 {
   nsCOMPtr<nsIStringBundleService> stringService =
     mozilla::services::GetStringBundleService();
@@ -2000,11 +2048,11 @@ nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpU
   nsString passwordPromptString;
   if(formatStrings[1])
     rv = composeStringBundle->FormatStringFromName(
-      NS_LITERAL_STRING("smtpEnterPasswordPromptWithUsername").get(),
+      MOZ_UTF16("smtpEnterPasswordPromptWithUsername"),
       formatStrings, 2, getter_Copies(passwordPromptString));
   else
     rv = composeStringBundle->FormatStringFromName(
-      NS_LITERAL_STRING("smtpEnterPasswordPrompt").get(),
+      MOZ_UTF16("smtpEnterPasswordPrompt"),
       formatStrings, 1, getter_Copies(passwordPromptString));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2014,7 +2062,7 @@ nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpU
 
   nsString passwordTitle;
   rv = composeStringBundle->GetStringFromName(
-    NS_LITERAL_STRING("smtpEnterPasswordPromptTitle").get(),
+    MOZ_UTF16("smtpEnterPasswordPromptTitle"),
     getter_Copies(passwordTitle));
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -2055,7 +2103,7 @@ nsSmtpProtocol::GetUsernamePassword(nsACString &aUsername,
     rv = smtpServer->GetHostname(hostname);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    const PRUnichar *formatStrings[] =
+    const char16_t *formatStrings[] =
     {
       NS_ConvertASCIItoUTF16(hostname).get(),
       nullptr
