@@ -21,7 +21,6 @@
 #include "nsComposeStrings.h"
 #include "nsISmtpServer.h"
 #include "nsIPrompt.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsIMsgCompUtils.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
@@ -31,6 +30,11 @@
 #include "nsNetUtil.h"
 #include "nsIMsgDatabase.h"
 #include "mozilla/Services.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
+#include "mozilla/mailnews/MimeHeaderParser.h"
+
+using namespace mozilla::mailnews;
 
 #define MDN_NOT_IN_TO_CC          ((int) 0x0001)
 #define MDN_OUTSIDE_DOMAIN        ((int) 0x0002)
@@ -94,9 +98,9 @@ nsMsgMdnGenerator::~nsMsgMdnGenerator()
 {
 }
 
-nsresult nsMsgMdnGenerator::FormatStringFromName(const PRUnichar *aName,
-                                                 const PRUnichar *aString,
-                                                 PRUnichar **aResultString)
+nsresult nsMsgMdnGenerator::FormatStringFromName(const char16_t *aName,
+                                                 const char16_t *aString,
+                                                 char16_t **aResultString)
 {
     DEBUG_MDN("nsMsgMdnGenerator::FormatStringFromName");
 
@@ -109,15 +113,15 @@ nsresult nsMsgMdnGenerator::FormatStringFromName(const PRUnichar *aName,
                                               getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv,rv);
 
-    const PRUnichar *formatStrings[1] = { aString };
+    const char16_t *formatStrings[1] = { aString };
     rv = bundle->FormatStringFromName(aName,
                     formatStrings, 1, aResultString);
     NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
 
-nsresult nsMsgMdnGenerator::GetStringFromName(const PRUnichar *aName,
-                                               PRUnichar **aResultString)
+nsresult nsMsgMdnGenerator::GetStringFromName(const char16_t *aName,
+                                               char16_t **aResultString)
 {
     DEBUG_MDN("nsMsgMdnGenerator::GetStringFromName");
 
@@ -324,8 +328,8 @@ bool nsMsgMdnGenerator::NotInToOrCc()
     nsCString cc;
 
     m_identity->GetReplyTo(reply_to);
-    m_headers->ExtractHeader(HEADER_TO, true, getter_Copies(to));
-    m_headers->ExtractHeader(HEADER_CC, true, getter_Copies(cc));
+    m_headers->ExtractHeader(HEADER_TO, true, to);
+    m_headers->ExtractHeader(HEADER_CC, true, cc);
 
   // start with a simple check
   if ((!to.IsEmpty() && PL_strcasestr(to.get(), m_email.get())) ||
@@ -349,8 +353,7 @@ bool nsMsgMdnGenerator::ValidateReturnPath()
         return m_reallySendMdn;
 
     nsCString returnPath;
-    m_headers->ExtractHeader(HEADER_RETURN_PATH, false,
-                             getter_Copies(returnPath));
+    m_headers->ExtractHeader(HEADER_RETURN_PATH, false, returnPath);
     if (returnPath.IsEmpty())
     {
       m_autoSend = false;
@@ -460,21 +463,15 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
     m_identity->GetFullName(fullName);
 
     nsCString fullAddress;
-    nsCOMPtr<nsIMsgHeaderParser> parser (do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID));
-    if (parser)
-    {
-        // convert fullName to UTF8 before passing it to MakeFullAddressString
-        parser->MakeFullAddressString(NS_ConvertUTF16toUTF8(fullName).get(),
-                                      m_email.get(), getter_Copies(fullAddress));
-    }
+    // convert fullName to UTF8 before passing it to MakeMimeAddress
+    MakeMimeAddress(NS_ConvertUTF16toUTF8(fullName), m_email, fullAddress);
 
-    convbuf = nsMsgI18NEncodeMimePartIIStr(
-        (!fullAddress.IsEmpty()) ? fullAddress.get(): m_email.get(),
+    convbuf = nsMsgI18NEncodeMimePartIIStr(fullAddress.get(),
         true, m_charset.get(), 0, conformToStandard);
 
     parm = PR_smprintf("From: %s" CRLF, convbuf ? convbuf : m_email.get());
 
-    rv = FormatStringFromName(NS_LITERAL_STRING("MsgMdnMsgSentTo").get(), NS_ConvertASCIItoUTF16(m_email).get(),
+    rv = FormatStringFromName(MOZ_UTF16("MsgMdnMsgSentTo"), NS_ConvertASCIItoUTF16(m_email).get(),
                             getter_Copies(firstPart1));
     if (NS_FAILED(rv))
         return rv;
@@ -496,32 +493,32 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
     {
     case nsIMsgMdnGenerator::eDisplayed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnDisplayedReceipt").get(),
+            MOZ_UTF16("MdnDisplayedReceipt"),
             getter_Copies(receipt_string));
         break;
     case nsIMsgMdnGenerator::eDispatched:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnDispatchedReceipt").get(),
+            MOZ_UTF16("MdnDispatchedReceipt"),
             getter_Copies(receipt_string));
         break;
     case nsIMsgMdnGenerator::eProcessed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnProcessedReceipt").get(),
+            MOZ_UTF16("MdnProcessedReceipt"),
             getter_Copies(receipt_string));
         break;
     case nsIMsgMdnGenerator::eDeleted:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnDeletedReceipt").get(),
+            MOZ_UTF16("MdnDeletedReceipt"),
             getter_Copies(receipt_string));
         break;
     case nsIMsgMdnGenerator::eDenied:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnDeniedReceipt").get(),
+            MOZ_UTF16("MdnDeniedReceipt"),
             getter_Copies(receipt_string));
         break;
     case nsIMsgMdnGenerator::eFailed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MdnFailedReceipt").get(),
+            MOZ_UTF16("MdnFailedReceipt"),
             getter_Copies(receipt_string));
         break;
     default:
@@ -538,7 +535,7 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
                                                                "UTF-8", 0, conformToStandard);
 
     nsCString subject;
-    m_headers->ExtractHeader(HEADER_SUBJECT, false, getter_Copies(subject));
+    m_headers->ExtractHeader(HEADER_SUBJECT, false, subject);
     convbuf = nsMsgI18NEncodeMimePartIIStr(subject.Length() ? subject.get() : "[no subject]",
                                            false, m_charset.get(), 0, conformToStandard);
     tmpBuffer = PR_smprintf("Subject: %s%s" CRLF,
@@ -559,8 +556,7 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
 
   // *** This is not in the spec. I am adding this so we could do
   // threading
-    m_headers->ExtractHeader(HEADER_MESSAGE_ID, false,
-                             getter_Copies(m_messageId));
+    m_headers->ExtractHeader(HEADER_MESSAGE_ID, false, m_messageId);
 
     if (!m_messageId.IsEmpty())
     {
@@ -598,32 +594,32 @@ report-type=disposition-notification;\r\n\tboundary=\"%s\"" CRLF CRLF,
     {
     case nsIMsgMdnGenerator::eDisplayed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnDisplayed").get(),
+            MOZ_UTF16("MsgMdnDisplayed"),
             getter_Copies(firstPart2));
         break;
     case nsIMsgMdnGenerator::eDispatched:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnDispatched").get(),
+            MOZ_UTF16("MsgMdnDispatched"),
             getter_Copies(firstPart2));
         break;
     case nsIMsgMdnGenerator::eProcessed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnProcessed").get(),
+            MOZ_UTF16("MsgMdnProcessed"),
             getter_Copies(firstPart2));
         break;
     case nsIMsgMdnGenerator::eDeleted:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnDeleted").get(),
+            MOZ_UTF16("MsgMdnDeleted"),
             getter_Copies(firstPart2));
         break;
     case nsIMsgMdnGenerator::eDenied:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnDenied").get(),
+            MOZ_UTF16("MsgMdnDenied"),
             getter_Copies(firstPart2));
         break;
     case nsIMsgMdnGenerator::eFailed:
         rv = GetStringFromName(
-            NS_LITERAL_STRING("MsgMdnFailed").get(),
+            MOZ_UTF16("MsgMdnFailed"),
             getter_Copies(firstPart2));
         break;
     default:
@@ -693,7 +689,7 @@ nsresult nsMsgMdnGenerator::CreateSecondPart()
 
     nsCString originalRecipient;
     m_headers->ExtractHeader(HEADER_ORIGINAL_RECIPIENT, false,
-                             getter_Copies(originalRecipient));
+                             originalRecipient);
 
     if (!originalRecipient.IsEmpty())
     {
@@ -773,7 +769,7 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders()
     int32_t all_headers_size = 0;
     nsresult rv = NS_OK;
 
-    rv = m_headers->GetAllHeaders(getter_Copies(all_headers));
+    rv = m_headers->GetAllHeaders(all_headers);
     if (NS_FAILED(rv))
         return rv;
     all_headers_size = all_headers.Length();
@@ -886,7 +882,7 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool *needToAskUser)
           // in which case we find the originating account's identity.
           nsCString accountKey;
           m_headers->ExtractHeader(HEADER_X_MOZILLA_ACCOUNT_KEY, false,
-                               getter_Copies(accountKey));
+                                   accountKey);
           nsCOMPtr <nsIMsgAccount> account;
           if (!accountKey.IsEmpty())
             accountManager->GetAccount(accountKey, getter_AddRefs(account));
@@ -898,20 +894,20 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool *needToAskUser)
             // Find the correct identity based on the "To:" and "Cc:" header
             nsCString mailTo;
             nsCString mailCC;
-            m_headers->ExtractHeader(HEADER_TO, true, getter_Copies(mailTo));
-            m_headers->ExtractHeader(HEADER_CC, true, getter_Copies(mailCC));
-            nsCOMPtr<nsISupportsArray> servIdentities;
+            m_headers->ExtractHeader(HEADER_TO, true, mailTo);
+            m_headers->ExtractHeader(HEADER_CC, true, mailCC);
+            nsCOMPtr<nsIArray> servIdentities;
             accountManager->GetIdentitiesForServer(m_server, getter_AddRefs(servIdentities));
             if (servIdentities)
             {
               nsCOMPtr<nsIMsgIdentity> ident;
               nsCString identEmail;
               uint32_t count = 0;
-              servIdentities->Count(&count);
+              servIdentities->GetLength(&count);
               // First check in the "To:" header
               for (uint32_t i = 0; i < count; i++)
               {
-                rv = servIdentities->QueryElementAt(i, NS_GET_IID(nsIMsgIdentity),getter_AddRefs(ident));
+                ident = do_QueryElementAt(servIdentities, i, &rv);
                 if (NS_FAILED(rv))
                   continue;
                 ident->GetEmail(identEmail);
@@ -990,10 +986,10 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool *needToAskUser)
     if (m_mdnEnabled)
     {
         m_headers->ExtractHeader(HEADER_DISPOSITION_NOTIFICATION_TO, false,
-                                 getter_Copies(m_dntRrt));
+                                 m_dntRrt);
         if (m_dntRrt.IsEmpty())
             m_headers->ExtractHeader(HEADER_RETURN_RECEIPT_TO, false,
-                                     getter_Copies(m_dntRrt));
+                                     m_dntRrt);
         if (!m_dntRrt.IsEmpty() && ProcessSendMode() && ValidateReturnPath())
         {
             if (!m_autoSend)
@@ -1112,13 +1108,13 @@ NS_IMETHODIMP nsMsgMdnGenerator::OnStopRunningUrl(nsIURI *url,
     // Get the smtp hostname and format the string.
     nsCString smtpHostName;
     nsCOMPtr<nsISmtpServer> smtpServer;
-    rv = smtpService->GetSmtpServerByIdentity(m_identity, getter_AddRefs(smtpServer));
+    rv = smtpService->GetServerByIdentity(m_identity, getter_AddRefs(smtpServer));
     if (NS_SUCCEEDED(rv)) 
       smtpServer->GetHostname(smtpHostName);
      
     nsAutoString hostStr;
     CopyASCIItoUTF16(smtpHostName, hostStr);
-    const PRUnichar *params[] = { hostStr.get() };
+    const char16_t *params[] = { hostStr.get() };
 
     nsCOMPtr<nsIStringBundle> bundle;
     nsCOMPtr<nsIStringBundleService> bundleService =
