@@ -8,7 +8,6 @@
 
 #include "msgCore.h"
 #include "nsIMsgCompFields.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsIMsgIdentity.h"
 #include "nsISMimeCert.h"
 #include "nsIX509CertDB.h"
@@ -21,10 +20,11 @@
 #include "nsAlgorithm.h"
 #include "mozilla/Services.h"
 #include "mozilla/mailnews/MimeEncoder.h"
+#include "mozilla/mailnews/MimeHeaderParser.h"
 #include "nsIMimeConverter.h"
 #include <algorithm>
 
-using mozilla::mailnews::MimeEncoder;
+using namespace mozilla::mailnews;
 
 #define MK_MIME_ERROR_WRITING_FILE -1
 
@@ -170,8 +170,8 @@ NS_IMETHODIMP nsMsgComposeSecure::RequiresCryptoEncapsulation(nsIMsgIdentity * a
 }
 
 
-nsresult nsMsgComposeSecure::GetSMIMEBundleString(const PRUnichar *name,
-                                                  PRUnichar **outString)
+nsresult nsMsgComposeSecure::GetSMIMEBundleString(const char16_t *name,
+                                                  char16_t **outString)
 {
   *outString = nullptr;
 
@@ -184,10 +184,10 @@ nsresult nsMsgComposeSecure::GetSMIMEBundleString(const PRUnichar *name,
 
 nsresult
 nsMsgComposeSecure::
-SMIMEBundleFormatStringFromName(const PRUnichar *name,
-                                const PRUnichar **params,
+SMIMEBundleFormatStringFromName(const char16_t *name,
+                                const char16_t **params,
                                 uint32_t numParams,
-                                PRUnichar **outString)
+                                char16_t **outString)
 {
   NS_ENSURE_ARG_POINTER(name);
 
@@ -212,7 +212,7 @@ bool nsMsgComposeSecure::InitializeSMIMEBundle()
   return true;
 }
 
-void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string)
+void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const char16_t *bundle_string)
 {
   if (!sendReport || !bundle_string)
     return;
@@ -236,7 +236,7 @@ void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const PRUnichar
   }
 }
 
-void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string, const char *param)
+void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const char16_t *bundle_string, const char *param)
 {
   if (!sendReport || !bundle_string || !param)
     return;
@@ -248,7 +248,7 @@ void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const P
   
   nsString errorString;
   nsresult res;
-  const PRUnichar *params[1];
+  const char16_t *params[1];
 
   NS_ConvertASCIItoUTF16 ucs2(param);
   params[0]= ucs2.get();
@@ -464,8 +464,8 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
 
   if (!sMIMEBundle)
     return NS_ERROR_FAILURE;
- 
-  sMIMEBundle->GetStringFromName(NS_LITERAL_STRING("mime_smimeEncryptedContentDesc").get(),
+
+  sMIMEBundle->GetStringFromName(MOZ_UTF16("mime_smimeEncryptedContentDesc"),
                                  getter_Copies(mime_smime_enc_content_desc));
   NS_ConvertUTF16toUTF8 enc_content_desc_utf8(mime_smime_enc_content_desc);
 
@@ -525,7 +525,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
   if (NS_FAILED(rv)) return rv;
   rv = mEncryptionCinfo->CreateEncrypted(mCerts);
   if (NS_FAILED(rv)) {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorEncryptMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorEncryptMail"));
     goto FAIL;
   }
 
@@ -542,7 +542,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(bool aSign, nsIMsgSendReport *se
 
   rv = mEncryptionContext->Start(mEncryptionCinfo, mime_crypto_write_base64, mCryptoEncoder);
   if (NS_FAILED(rv)) {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorEncryptMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorEncryptMail"));
     goto FAIL;
   }
 
@@ -580,21 +580,12 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
 
   if (!sMIMEBundle)
     return NS_ERROR_FAILURE;
-  
-  sMIMEBundle->GetStringFromName(NS_LITERAL_STRING("mime_smimeSignatureContentDesc").get(),
+
+  sMIMEBundle->GetStringFromName(MOZ_UTF16("mime_smimeSignatureContentDesc"),
                                  getter_Copies(mime_smime_sig_content_desc));
 
   NS_ConvertUTF16toUTF8 sig_content_desc_utf8(mime_smime_sig_content_desc);
 
-  nsCOMPtr<nsIMimeConverter> mimeConverter =
-     do_GetService(NS_MIME_CONVERTER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsCString encodedContentDescription;
-  mimeConverter->EncodeMimePartIIStr_UTF8(sig_content_desc_utf8, false, "UTF-8",
-      sizeof("Content-Description: "),
-      nsIMimeConverter::MIME_ENCODED_WORD_SIZE,
-      getter_Copies(encodedContentDescription));
-
   /* Compute the hash...
    */
 
@@ -620,8 +611,8 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
           "Content-Description: %s" CRLF
           CRLF,
           mMultipartSignedBoundary,
-          encodedContentDescription.get());
-
+          sig_content_desc_utf8.get());
+          
   if (!header) {
     rv = NS_ERROR_OUT_OF_MEMORY;
     goto FAIL;
@@ -653,7 +644,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
 
   rv = cinfo->CreateSigned(mSelfSigningCert, mSelfEncryptionCert, (unsigned char*)hashString.get(), hashString.Length());
   if (NS_FAILED(rv))  {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSignMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorCanNotSignMail"));
     goto FAIL;
   }
 
@@ -667,14 +658,14 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
   PR_SetError(0,0);
   rv = encoder->Start(cinfo, mime_crypto_write_base64, mSigEncoder);
   if (NS_FAILED(rv)) {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSignMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorCanNotSignMail"));
     goto FAIL;
   }
 
   // We're not passing in any data, so no update needed.
   rv = encoder->Finish();
   if (NS_FAILED(rv)) {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSignMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorCanNotSignMail"));
     goto FAIL;
   }
 
@@ -745,10 +736,10 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (bool aSign, nsIMsgSendReport
       goto FAIL;
     }
   }
-  
+
   rv = mEncryptionContext->Finish();
   if (NS_FAILED(rv)) {
-    SetError(sendReport, NS_LITERAL_STRING("ErrorEncryptMail").get());
+    SetError(sendReport, MOZ_UTF16("ErrorEncryptMail"));
     goto FAIL;
   }
 
@@ -782,14 +773,8 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
                                                  bool aEncrypt,
                                                  bool aSign)
 {
-  char *mailbox_list = 0;
-  nsCString all_mailboxes, mailboxes;
-  const char *mailbox = 0;
-  uint32_t count = 0;
   nsCOMPtr<nsIX509CertDB> certdb = do_GetService(NS_X509CERTDB_CONTRACTID);
   nsresult res;
-  nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &res);
-  NS_ENSURE_SUCCESS(res,res);
 
   mCerts = do_CreateInstance(NS_ARRAY_CONTRACTID, &res);
   if (NS_FAILED(res)) {
@@ -802,25 +787,15 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
   // must have both the signing and encryption certs to sign
   if ((mSelfSigningCert == nullptr) && aSign) {
-    SetError(sendReport, NS_LITERAL_STRING("NoSenderSigningCert").get());
-    res = NS_ERROR_FAILURE;
-    goto FAIL;
+    SetError(sendReport, MOZ_UTF16("NoSenderSigningCert"));
+    return NS_ERROR_FAILURE;
   }
 
   if ((mSelfEncryptionCert == nullptr) && aEncrypt) {
-    SetError(sendReport, NS_LITERAL_STRING("NoSenderEncryptionCert").get());
-    res = NS_ERROR_FAILURE;
-    goto FAIL;
+    SetError(sendReport, MOZ_UTF16("NoSenderEncryptionCert"));
+    return NS_ERROR_FAILURE;
   }
 
-  pHeader->ExtractHeaderAddressMailboxes(nsDependentCString(aRecipients),
-                                         all_mailboxes);
-  pHeader->RemoveDuplicateAddresses(all_mailboxes, EmptyCString(), mailboxes);
-
-  pHeader->ParseHeaderAddresses(mailboxes.get(), 0, &mailbox_list, &count);
-
-  // XXX This is not a valid use of nsresult
-  if (count < 0) return static_cast<nsresult>(count);
 
   if (aEncrypt && mSelfEncryptionCert) {
     // Make sure self's configured cert is prepared for being used
@@ -834,13 +809,16 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
   /* If the message is to be encrypted, then get the recipient certs */
   if (aEncrypt) {
-    mailbox = mailbox_list;
+    nsTArray<nsCString> mailboxes;
+    ExtractEmails(EncodedHeader(nsDependentCString(aRecipients)),
+      UTF16ArrayAdapter<>(mailboxes));
+    uint32_t count = mailboxes.Length();
 
     bool already_added_self_cert = false;
 
-    for (; count > 0; count--) {
+    for (uint32_t i = 0; i < count; i++) {
       nsCString mailbox_lowercase;
-      ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
+      ToLowerCase(mailboxes[i], mailbox_lowercase);
       nsCOMPtr<nsIX509Cert> cert;
       res = certdb->FindCertByEmailAddress(nullptr, mailbox_lowercase.get(),
                                            getter_AddRefs(cert));
@@ -848,10 +826,10 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
         // Failure to find a valid encryption cert is fatal.
         // Here I assume that mailbox is ascii rather than utf8.
         SetErrorWithParam(sendReport,
-                          NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(),
-                          mailbox);
+                          MOZ_UTF16("MissingRecipientEncryptionCert"),
+                          mailboxes[i].get());
 
-        goto FAIL;
+        return res;
       }
 
     /* #### see if recipient requests `signedData'.
@@ -868,19 +846,12 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
       }
 
       mCerts->AppendElement(cert, false);
-      // To understand this loop, especially the "+= strlen +1", look at the documentation
-      // of ParseHeaderAddresses. Basically, it returns a list of zero terminated strings.
-      mailbox += strlen(mailbox) + 1;
     }
     
     if (!already_added_self_cert) {
       mCerts->AppendElement(mSelfEncryptionCert, false);
     }
   }
-FAIL:
-  if (mailbox_list) {
-    nsMemory::Free(mailbox_list);
-  }
   return res;
 }
 
