@@ -28,23 +28,23 @@ bool MsgStrategyComparatorAdaptor::Equals(const nsMsgKey& a, const nsMsgKey& b)
 {
   nsCOMPtr<nsIMsgDBHdr> hdrA;
   nsCOMPtr<nsIMsgDBHdr> hdrB;
-  
+
   mDatabase->GetMsgHdrForKey(a, getter_AddRefs(hdrA));
   mDatabase->GetMsgHdrForKey(b, getter_AddRefs(hdrB));
-  
+
   if (hdrA && hdrB)
   {
-    nsresult rv;
+    nsresult rv = NS_OK;
     nsAutoSyncStrategyDecisionType decision = nsAutoSyncStrategyDecisions::Same;
-    
+
     nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(mFolder);
     if (mStrategy)
       rv = mStrategy->Sort(folder, hdrA, hdrB, &decision);
-    
+
     if (NS_SUCCEEDED(rv))
       return (decision == nsAutoSyncStrategyDecisions::Same);
   }
-  
+
   return false;
 }
 
@@ -53,13 +53,13 @@ bool MsgStrategyComparatorAdaptor::LessThan(const nsMsgKey& a, const nsMsgKey& b
 {
   nsCOMPtr<nsIMsgDBHdr> hdrA;
   nsCOMPtr<nsIMsgDBHdr> hdrB;
-  
+
   mDatabase->GetMsgHdrForKey(a, getter_AddRefs(hdrA));
   mDatabase->GetMsgHdrForKey(b, getter_AddRefs(hdrB));
 
   if (hdrA && hdrB)
   {
-    nsresult rv;
+    nsresult rv = NS_OK;
     nsAutoSyncStrategyDecisionType decision = nsAutoSyncStrategyDecisions::Same;
 
     nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(mFolder);
@@ -69,7 +69,7 @@ bool MsgStrategyComparatorAdaptor::LessThan(const nsMsgKey& a, const nsMsgKey& b
     if (NS_SUCCEEDED(rv))
       return (decision == nsAutoSyncStrategyDecisions::Lower);
   }
-  
+
   return false;
 }
 
@@ -266,9 +266,9 @@ NS_IMETHODIMP nsAutoSyncState::GetNextGroupOfMessages(uint32_t aSuggestedGroupSi
         // ensure that we don't have this message body offline already,
         // possible if the user explicitly selects this message prior
         // to auto-sync kicks in
-        uint32_t msgFlags = 0;
-        qhdr->GetFlags(&msgFlags);
-        if (msgFlags & nsMsgMessageFlags::Offline)
+        bool hasMessageOffline;
+        folder->HasMsgOffline(mDownloadQ[idx], &hasMessageOffline);
+        if (hasMessageOffline)
           continue;
 
         // this check point allows msg strategy function
@@ -350,16 +350,10 @@ NS_IMETHODIMP nsAutoSyncState::ProcessExistingHeaders(uint32_t aNumOfHdrsToProce
   uint32_t keyCount = mExistingHeadersQ.Length();
   for (; mProcessPointer < (lastIdx + aNumOfHdrsToProcess) && mProcessPointer < keyCount; mProcessPointer++)
   {
-    nsCOMPtr<nsIMsgDBHdr> hdr;
-    rv = database->GetMsgHdrForKey(mExistingHeadersQ[mProcessPointer], getter_AddRefs(hdr));
-    if (hdr)
-    {
-      uint32_t msgFlags = 0;
-      hdr->GetFlags(&msgFlags);
-
-      if (!(msgFlags & nsMsgMessageFlags::Offline))
-        msgKeys.AppendElement(mExistingHeadersQ[mProcessPointer]);
-    }
+    bool hasMessageOffline;
+    folder->HasMsgOffline(mExistingHeadersQ[mProcessPointer], &hasMessageOffline);
+    if (!hasMessageOffline)
+      msgKeys.AppendElement(mExistingHeadersQ[mProcessPointer]);
   }
   if (!msgKeys.IsEmpty())
   {
