@@ -24,7 +24,8 @@
 #include "nsIMsgfilter.h"
 #include "nsEudoraFilters.h"
 #include "nsEudoraStringBundle.h"
-
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 #include "nsNetUtil.h"
 #include "nsILineInputStream.h"
 #include "EudoraDebugLog.h"
@@ -58,7 +59,7 @@ nsEudoraFilters::~nsEudoraFilters()
 
 NS_IMPL_ISUPPORTS1(nsEudoraFilters, nsIImportFilters)
 
-NS_IMETHODIMP nsEudoraFilters::AutoLocate(PRUnichar **aDescription, nsIFile **aLocation, bool *_retval)
+NS_IMETHODIMP nsEudoraFilters::AutoLocate(char16_t **aDescription, nsIFile **aLocation, bool *_retval)
 {
   NS_ENSURE_ARG_POINTER(aDescription);
   NS_ENSURE_ARG_POINTER(aLocation);
@@ -91,7 +92,7 @@ NS_IMETHODIMP nsEudoraFilters::SetLocation(nsIFile *aLocation)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsEudoraFilters::Import(PRUnichar **aError, bool *_retval)
+NS_IMETHODIMP nsEudoraFilters::Import(char16_t **aError, bool *_retval)
 {
   NS_ENSURE_ARG_POINTER(aError);
   NS_ENSURE_ARG_POINTER(_retval);
@@ -368,8 +369,8 @@ nsresult nsEudoraFilters::Init()
   if (NS_SUCCEEDED(rv))
   {
     nsAutoString Eudora(NS_LITERAL_STRING("Eudora"));
-    const PRUnichar *moduleName[] = { Eudora.get() };
-    rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImportModuleFolderName").get(),
+    const char16_t *moduleName[] = { Eudora.get() };
+    rv = bundle->FormatStringFromName(MOZ_UTF16("ImportModuleFolderName"),
                                       moduleName, 1, getter_Copies(folderName));
   }
   localRootFolder->GetChildNamed(folderName, getter_AddRefs(m_pMailboxesRoot));
@@ -390,22 +391,22 @@ nsresult nsEudoraFilters::LoadServers()
   if (m_pServerArray)
     rv = m_pServerArray->Clear();
   else
-    rv = NS_NewISupportsArray(getter_AddRefs(m_pServerArray));
+    m_pServerArray = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!m_pFilterArray)
-    rv = NS_NewISupportsArray(getter_AddRefs(m_pFilterArray));
+    m_pFilterArray = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgAccountManager> accountMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISupportsArray> allServers;
+  nsCOMPtr<nsIArray> allServers;
   rv = accountMgr->GetAllServers(getter_AddRefs(allServers));
   NS_ENSURE_SUCCESS(rv, rv);
 
   uint32_t numServers;
-  rv = allServers->Count(&numServers);
+  rv = allServers->GetLength(&numServers);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t serverIndex = 0; serverIndex < numServers; serverIndex++)
   {
@@ -430,7 +431,7 @@ nsresult nsEudoraFilters::LoadServers()
             rv = server->GetFilterList(nullptr, getter_AddRefs(filterList));
             NS_ENSURE_SUCCESS(rv, rv);
 
-            m_pServerArray->AppendElement(server);
+            m_pServerArray->AppendElement(server, false);
           }
         }
       }
@@ -445,7 +446,7 @@ nsresult nsEudoraFilters::SaveFilters()
   nsresult rv;
 
   uint32_t numServers;
-  rv = m_pServerArray->Count(&numServers);
+  rv = m_pServerArray->GetLength(&numServers);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t serverIndex = 0; serverIndex < numServers; serverIndex++)
   {
@@ -473,7 +474,7 @@ nsresult nsEudoraFilters::CreateNewFilter(const char* pName)
   nsAutoString unicodeName;
   NS_CopyNativeToUnicode(nsCString(pName), unicodeName);
   uint32_t numServers;
-  rv = m_pServerArray->Count(&numServers);
+  rv = m_pServerArray->GetLength(&numServers);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t serverIndex = 0; serverIndex < numServers; serverIndex++)
   {
@@ -498,7 +499,7 @@ nsresult nsEudoraFilters::CreateNewFilter(const char* pName)
     rv = filterList->InsertFilterAt(count, newFilter);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    m_pFilterArray->AppendElement(newFilter);
+    m_pFilterArray->AppendElement(newFilter, false);
   }
 
   m_isAnd = false;
@@ -538,7 +539,7 @@ nsresult nsEudoraFilters::EnableFilter(bool enable)
   nsresult rv;
 
   uint32_t numFilters;
-  rv = m_pFilterArray->Count(&numFilters);
+  rv = m_pFilterArray->GetLength(&numFilters);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t filterIndex = 0; filterIndex < numFilters; filterIndex++)
   {
@@ -743,7 +744,7 @@ nsresult nsEudoraFilters::AddTerm(const char* pHeader, const char* pVerb, const
   }
 
   uint32_t numFilters;
-  rv = m_pFilterArray->Count(&numFilters);
+  rv = m_pFilterArray->GetLength(&numFilters);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t filterIndex = 0; filterIndex < numFilters; filterIndex++)
   {
@@ -792,7 +793,7 @@ nsresult nsEudoraFilters::AddAction(nsMsgRuleActionType actionType, int32_t junk
   nsresult rv;
 
   uint32_t numFilters;
-  rv = m_pFilterArray->Count(&numFilters);
+  rv = m_pFilterArray->GetLength(&numFilters);
   NS_ENSURE_SUCCESS(rv, rv);
   for (uint32_t filterIndex = 0; filterIndex < numFilters; filterIndex++)
   {
