@@ -25,6 +25,7 @@
 #include "nsIMutableArray.h"
 #include "nsMsgUtils.h"
 #include "nsComponentManagerUtils.h"
+#include "mozilla/Likely.h"
 
 static NS_DEFINE_CID(kCPop3ServiceCID, NS_POP3SERVICE_CID);
 
@@ -36,13 +37,14 @@ public:
 
   nsPop3GetMailChainer();
   ~nsPop3GetMailChainer();
-  nsresult GetNewMailForServers(nsISupportsArray *servers, nsIMsgWindow *msgWindow,
+  nsresult GetNewMailForServers(nsIPop3IncomingServer** servers, uint32_t count,
+                                nsIMsgWindow *msgWindow,
                                 nsIMsgFolder *folderToDownloadTo, nsIUrlListener *listener);
   nsresult RunNextGetNewMail();
 protected:
   nsCOMPtr <nsIMsgFolder> m_folderToDownloadTo;
   nsCOMPtr <nsIMsgWindow> m_downloadingMsgWindow;
-  nsCOMPtr <nsISupportsArray> m_serversToGetNewMailFor;
+  nsCOMArray<nsIPop3IncomingServer> m_serversToGetNewMailFor;
   nsCOMPtr <nsIUrlListener> m_listener;
 };
 
@@ -167,7 +169,7 @@ NS_IMETHODIMP nsPop3IncomingServer::GetDeferredToAccount(nsACString& aRetVal)
                   rv = subFolderDB->ListAllKeys(keys);
                   nsCOMPtr<nsIMutableArray> hdrsToCopy(do_CreateInstance(NS_ARRAY_CONTRACTID));
                   MsgGetHeadersFromKeys(subFolderDB, keys->m_keys, hdrsToCopy);
-                  PRUint32 numHdrs = 0;
+                  uint32_t numHdrs = 0;
                   if (hdrsToCopy)
                     hdrsToCopy->GetLength(&numHdrs);
                   if (numHdrs)
@@ -180,7 +182,7 @@ NS_IMETHODIMP nsPop3IncomingServer::GetDeferredToAccount(nsACString& aRetVal)
                                                     getter_AddRefs(dest));
                     if (dest)
                       dest->CopyMessages(subFolder, hdrsToCopy, false,
-                                         nsnull, nsnull, false,false);
+                                         nullptr, nullptr, false,false);
                     // Should we copy the folder if the dest doesn't exist?
                   }
                 }
@@ -206,7 +208,7 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(const nsACString& aAcco
 {
   nsCString deferredToAccount;
   GetDeferredToAccount(deferredToAccount);
-  m_rootMsgFolder = nsnull; // clear this so we'll recalculate it on demand.
+  m_rootMsgFolder = nullptr; // clear this so we'll recalculate it on demand.
   //Notify listeners who listen to every folder
 
   nsresult rv =  SetCharValue("deferred_to_account", aAccountKey);
@@ -260,7 +262,7 @@ NS_IMETHODIMP nsPop3IncomingServer::SetDeferredToAccount(const nsACString& aAcco
                   rv = server->GetRootFolder(getter_AddRefs(rootFolder));
                   NS_ENSURE_SUCCESS(rv, rv);
                   // this will fail if it already exists, which is fine.
-                  rootFolder->CreateSubfolder(NS_LITERAL_STRING("Inbox"), nsnull);
+                  rootFolder->CreateSubfolder(NS_LITERAL_STRING("Inbox"), nullptr);
                 }
               }
             }
@@ -288,14 +290,14 @@ NS_IMETHODIMP nsPop3IncomingServer::SetAuthenticated(bool aAuthenticated)
 }
 
 nsresult
-nsPop3IncomingServer::GetPop3CapabilityFlags(PRUint32 *flags)
+nsPop3IncomingServer::GetPop3CapabilityFlags(uint32_t *flags)
 {
   *flags = m_capabilityFlags;
   return NS_OK;
 }
 
 nsresult
-nsPop3IncomingServer::SetPop3CapabilityFlags(PRUint32 flags)
+nsPop3IncomingServer::SetPop3CapabilityFlags(uint32_t flags)
 {
   m_capabilityFlags = flags;
   return NS_OK;
@@ -357,10 +359,10 @@ nsresult nsPop3IncomingServer::GetInbox(nsIMsgWindow *msgWindow, nsIMsgFolder **
     rv = (*inbox)->GetMsgDatabase(getter_AddRefs(db));
     if (NS_FAILED(rv))
     {
-      (*inbox)->SetMsgDatabase(nsnull);
+      (*inbox)->SetMsgDatabase(nullptr);
       (void) localInbox->SetCheckForNewMessagesAfterParsing(true);
       // this will cause a reparse of the mail folder.
-      localInbox->GetDatabaseWithReparse(nsnull, msgWindow, getter_AddRefs(db));
+      localInbox->GetDatabaseWithReparse(nullptr, msgWindow, getter_AddRefs(db));
       rv = NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
     }
   }
@@ -404,7 +406,7 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow *aMsgWindow)
       if (NS_SUCCEEDED(rv) && db)
         rv = db->GetSummaryValid(&valid);
       if (NS_SUCCEEDED(rv) && valid)
-        rv = pop3Service->GetNewMail(aMsgWindow, urlListener, inbox, this, nsnull);
+        rv = pop3Service->GetNewMail(aMsgWindow, urlListener, inbox, this, nullptr);
       else
       {
         bool isLocked;
@@ -417,7 +419,7 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow *aMsgWindow)
     }
   }
   else
-    rv = pop3Service->CheckForNewMail(aMsgWindow, urlListener, inbox, this, nsnull);
+    rv = pop3Service->CheckForNewMail(aMsgWindow, urlListener, inbox, this, nullptr);
   return NS_OK;
 }
 
@@ -432,21 +434,17 @@ nsPop3IncomingServer::SetFlagsOnDefaultMailboxes()
   NS_ENSURE_SUCCESS(rv, rv);
 
   // pop3 gets an inbox, but no queue (unsent messages)
-  localFolder->SetFlagsOnDefaultMailboxes(nsMsgFolderFlags::Inbox |
-                                          nsMsgFolderFlags::Archive |
-                                          nsMsgFolderFlags::SentMail |
-                                          nsMsgFolderFlags::Drafts |
-                                          nsMsgFolderFlags::Templates |
-                                          nsMsgFolderFlags::Trash |
-                                          nsMsgFolderFlags::Junk);
+  localFolder->SetFlagsOnDefaultMailboxes(nsMsgFolderFlags::SpecialUse &
+                                         ~nsMsgFolderFlags::Queue);
   return NS_OK;
 }
 
 
-NS_IMETHODIMP nsPop3IncomingServer::CreateDefaultMailboxes(nsIFile *aPath)
+NS_IMETHODIMP nsPop3IncomingServer::CreateDefaultMailboxes()
 {
   nsresult rv = CreateLocalFolder(NS_LITERAL_STRING("Inbox"));
   NS_ENSURE_SUCCESS(rv, rv);
+
   return CreateLocalFolder(NS_LITERAL_STRING("Trash"));
 }
 
@@ -486,7 +484,8 @@ nsPop3IncomingServer::VerifyLogon(nsIUrlListener *aUrlListener,
   return pop3Service->VerifyLogon(this, aUrlListener, aMsgWindow, aURL);
 }
 
-NS_IMETHODIMP nsPop3IncomingServer::DownloadMailFromServers(nsISupportsArray *aServers,
+NS_IMETHODIMP nsPop3IncomingServer::DownloadMailFromServers(nsIPop3IncomingServer** aServers,
+                              uint32_t aCount,
                               nsIMsgWindow *aMsgWindow,
                               nsIMsgFolder *aFolder,
                               nsIUrlListener *aUrlListener)
@@ -494,7 +493,7 @@ NS_IMETHODIMP nsPop3IncomingServer::DownloadMailFromServers(nsISupportsArray *aS
   nsPop3GetMailChainer *getMailChainer = new nsPop3GetMailChainer;
   NS_ENSURE_TRUE(getMailChainer, NS_ERROR_OUT_OF_MEMORY);
   getMailChainer->AddRef(); // this object owns itself and releases when done.
-  return getMailChainer->GetNewMailForServers(aServers, aMsgWindow, aFolder, aUrlListener);
+  return getMailChainer->GetNewMailForServers(aServers, aCount, aMsgWindow, aFolder, aUrlListener);
 }
 
 NS_IMETHODIMP nsPop3IncomingServer::GetNewMail(nsIMsgWindow *aMsgWindow,
@@ -525,26 +524,23 @@ nsPop3IncomingServer::GetNewMessages(nsIMsgFolder *aFolder, nsIMsgWindow *aMsgWi
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr <nsIURI> url;
   nsCOMPtr <nsIMsgIncomingServer> server;
-  nsCOMPtr <nsISupportsArray> deferredServers;
+  nsCOMArray<nsIPop3IncomingServer> deferredServers;
   nsCString deferredToAccount;
   GetDeferredToAccount(deferredToAccount);
 
   if (deferredToAccount.IsEmpty())
   {
     aFolder->GetServer(getter_AddRefs(server));
-    GetDeferredServers(server, getter_AddRefs(deferredServers));
+    GetDeferredServers(server, deferredServers);
   }
-  PRUint32 numDeferredServers;
-  if (deferredToAccount.IsEmpty() && deferredServers && NS_SUCCEEDED(deferredServers->Count(&numDeferredServers))
-    && numDeferredServers > 0)
+  if (deferredToAccount.IsEmpty() && !deferredServers.IsEmpty())
   {
     nsPop3GetMailChainer *getMailChainer = new nsPop3GetMailChainer;
     NS_ENSURE_TRUE(getMailChainer, NS_ERROR_OUT_OF_MEMORY);
     getMailChainer->AddRef(); // this object owns itself and releases when done.
-    nsCOMPtr <nsISupports> supports;
-    this->QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(supports));
-    deferredServers->InsertElementAt(supports, 0);
-    return getMailChainer->GetNewMailForServers(deferredServers, aMsgWindow, inbox, aUrlListener);
+    deferredServers.InsertElementAt(0, this);
+    return getMailChainer->GetNewMailForServers(deferredServers.Elements(),
+          deferredServers.Length(), aMsgWindow, inbox, aUrlListener);
   }
   if (m_runningProtocol)
     return NS_MSG_FOLDER_BUSY;
@@ -587,7 +583,7 @@ nsPop3IncomingServer::CloseCachedConnections()
 }
 
 NS_IMETHODIMP
-nsPop3IncomingServer::GetOfflineSupportLevel(PRInt32 *aSupportLevel)
+nsPop3IncomingServer::GetOfflineSupportLevel(int32_t *aSupportLevel)
 {
   NS_ENSURE_ARG_POINTER(aSupportLevel);
 
@@ -615,7 +611,7 @@ NS_IMETHODIMP nsPop3IncomingServer::GetRunningProtocol(nsIPop3Protocol **aProtoc
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char *aUidl, PRInt32 aMark)
+NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char *aUidl, int32_t aMark)
 {
   NS_ENSURE_ARG_POINTER(aUidl);
 
@@ -623,7 +619,7 @@ NS_IMETHODIMP nsPop3IncomingServer::AddUidlToMark(const char *aUidl, PRInt32 aMa
   NS_ENSURE_TRUE(uidlEntry, NS_ERROR_OUT_OF_MEMORY);
 
   uidlEntry->uidl = strdup(aUidl);
-  if (NS_UNLIKELY(!uidlEntry->uidl)) {
+  if (MOZ_UNLIKELY(!uidlEntry->uidl)) {
     PR_Free(uidlEntry);
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -652,8 +648,8 @@ NS_IMETHODIMP nsPop3IncomingServer::MarkMessages()
     // do it all in one fell swoop
     rv = nsPop3Protocol::MarkMsgForHost(hostName.get(), userName.get(), localPath, m_uidlsToMark);
   }
-  PRUint32 count = m_uidlsToMark.Count();
-  for (PRUint32 i = 0; i < count; i++)
+  uint32_t count = m_uidlsToMark.Count();
+  for (uint32_t i = 0; i < count; i++)
   {
     Pop3UidlEntry *ue = static_cast<Pop3UidlEntry*>(m_uidlsToMark[i]);
     PR_Free(ue->uidl);
@@ -672,12 +668,13 @@ nsPop3GetMailChainer::~nsPop3GetMailChainer()
 {
 }
 
-nsresult nsPop3GetMailChainer::GetNewMailForServers(nsISupportsArray *servers, nsIMsgWindow *msgWindow,
+nsresult nsPop3GetMailChainer::GetNewMailForServers(nsIPop3IncomingServer** servers,
+                                uint32_t count, nsIMsgWindow *msgWindow,
                                 nsIMsgFolder *folderToDownloadTo, nsIUrlListener *listener)
 {
   NS_ENSURE_ARG_POINTER(folderToDownloadTo);
 
-  m_serversToGetNewMailFor = servers;
+  m_serversToGetNewMailFor.AppendElements(servers, count);
   m_folderToDownloadTo = folderToDownloadTo;
   m_downloadingMsgWindow = msgWindow;
   m_listener = listener;
@@ -711,13 +708,12 @@ nsPop3GetMailChainer::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
 nsresult nsPop3GetMailChainer::RunNextGetNewMail()
 {
   nsresult rv;
-  PRUint32 numServersLeft;
-  m_serversToGetNewMailFor->Count(&numServersLeft);
+  uint32_t numServersLeft = m_serversToGetNewMailFor.Count();
 
   for (; numServersLeft > 0;)
   {
-    nsCOMPtr <nsIPop3IncomingServer> popServer (do_QueryElementAt(m_serversToGetNewMailFor, 0));
-    m_serversToGetNewMailFor->RemoveElementAt(0);
+    nsCOMPtr<nsIPop3IncomingServer> popServer(m_serversToGetNewMailFor[0]);
+    m_serversToGetNewMailFor.RemoveObjectAt(0);
     numServersLeft--;
     if (popServer)
     {
@@ -742,7 +738,7 @@ nsresult nsPop3GetMailChainer::RunNextGetNewMail()
       }
     }
   }
-  rv = m_listener ? m_listener->OnStopRunningUrl(nsnull, NS_OK) : NS_OK;
+  rv = m_listener ? m_listener->OnStopRunningUrl(nullptr, NS_OK) : NS_OK;
   Release(); // release ref to ourself.
   return rv;
 }
