@@ -21,6 +21,7 @@
 #include "nsIAccessibleRelation.h"
 #include "nsIAutoCompleteInput.h"
 #include "nsIAutoCompletePopup.h"
+#include "nsIBoxObject.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDOMXULMenuListElement.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
@@ -40,7 +41,8 @@ using namespace mozilla::a11y;
 XULTreeAccessible::
   XULTreeAccessible(nsIContent* aContent, DocAccessible* aDoc,
                     nsTreeBodyFrame* aTreeFrame) :
-  AccessibleWrap(aContent, aDoc)
+  AccessibleWrap(aContent, aDoc),
+  mAccessibleCache(kDefaultTreeCacheSize)
 {
   mType = eXULTreeType;
   mGenericTypes |= eSelect;
@@ -58,8 +60,6 @@ XULTreeAccessible::
     if (autoCompletePopupElm)
       mGenericTypes |= eAutoCompletePopup;
   }
-
-  mAccessibleCache.Init(kDefaultTreeCacheSize);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -134,7 +134,7 @@ XULTreeAccessible::Value(nsString& aValue)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// XULTreeAccessible: nsAccessNode implementation
+// XULTreeAccessible: Accessible implementation
 
 void
 XULTreeAccessible::Shutdown()
@@ -151,9 +151,6 @@ XULTreeAccessible::Shutdown()
   AccessibleWrap::Shutdown();
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// XULTreeAccessible: Accessible implementation (put methods here)
-
 role
 XULTreeAccessible::NativeRole()
 {
@@ -275,9 +272,7 @@ XULTreeAccessible::SelectedItems()
     }
   }
 
-  nsIMutableArray* items = nullptr;
-  selectedItems.forget(&items);
-  return items;
+  return selectedItems.forget();
 }
 
 uint32_t
@@ -419,7 +414,7 @@ XULTreeAccessible::SelectAll()
 // XULTreeAccessible: Accessible implementation
 
 Accessible*
-XULTreeAccessible::GetChildAt(uint32_t aIndex)
+XULTreeAccessible::GetChildAt(uint32_t aIndex) const
 {
   uint32_t childCount = Accessible::ChildCount();
   if (aIndex < childCount)
@@ -444,9 +439,9 @@ XULTreeAccessible::ChildCount() const
 }
 
 Relation
-XULTreeAccessible::RelationByType(uint32_t aType)
+XULTreeAccessible::RelationByType(RelationType aType)
 {
-  if (aType == nsIAccessibleRelation::RELATION_NODE_PARENT_OF) {
+  if (aType == RelationType::NODE_PARENT_OF) {
     if (mTreeView)
       return Relation(new XULTreeItemIterator(this, mTreeView, -1));
 
@@ -527,7 +522,7 @@ XULTreeAccessible::ContainerWidget() const
 // XULTreeAccessible: public implementation
 
 Accessible*
-XULTreeAccessible::GetTreeItemAccessible(int32_t aRow)
+XULTreeAccessible::GetTreeItemAccessible(int32_t aRow) const
 {
   if (aRow < 0 || IsDefunct() || !mTreeView)
     return nullptr;
@@ -545,10 +540,8 @@ XULTreeAccessible::GetTreeItemAccessible(int32_t aRow)
   nsRefPtr<Accessible> treeItem = CreateTreeItemAccessible(aRow);
   if (treeItem) {
     mAccessibleCache.Put(key, treeItem);
-    if (Document()->BindToDocument(treeItem, nullptr))
-      return treeItem;
-
-    mAccessibleCache.Remove(key);
+    Document()->BindToDocument(treeItem, nullptr);
+    return treeItem;
   }
 
   return nullptr;
@@ -686,10 +679,11 @@ XULTreeAccessible::TreeViewChanged(nsITreeView* aView)
 // XULTreeAccessible: protected implementation
 
 already_AddRefed<Accessible>
-XULTreeAccessible::CreateTreeItemAccessible(int32_t aRow)
+XULTreeAccessible::CreateTreeItemAccessible(int32_t aRow) const
 {
   nsRefPtr<Accessible> accessible =
-    new XULTreeItemAccessible(mContent, mDoc, this, mTree, mTreeView, aRow);
+    new XULTreeItemAccessible(mContent, mDoc, const_cast<XULTreeAccessible*>(this),
+                              mTree, mTreeView, aRow);
 
   return accessible.forget();
 }
@@ -781,7 +775,7 @@ XULTreeItemAccessibleBase::GetBounds(int32_t* aX, int32_t* aY,
 NS_IMETHODIMP
 XULTreeItemAccessibleBase::SetSelected(bool aSelect)
 {
-  if (IsDefunct() || !mTreeView)
+  if (IsDefunct())
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsITreeSelection> selection;
@@ -799,7 +793,7 @@ XULTreeItemAccessibleBase::SetSelected(bool aSelect)
 NS_IMETHODIMP
 XULTreeItemAccessibleBase::TakeFocus()
 {
-  if (IsDefunct() || !mTreeView)
+  if (IsDefunct())
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsITreeSelection> selection;
@@ -812,13 +806,11 @@ XULTreeItemAccessibleBase::TakeFocus()
 }
 
 Relation
-XULTreeItemAccessibleBase::RelationByType(uint32_t aType)
+XULTreeItemAccessibleBase::RelationByType(RelationType aType)
 {
-  if (!mTreeView)
-    return Relation();
 
   switch (aType) {
-    case nsIAccessibleRelation::RELATION_NODE_CHILD_OF: {
+    case RelationType::NODE_CHILD_OF: {
       int32_t parentIndex = -1;
       if (!NS_SUCCEEDED(mTreeView->GetParentIndex(mRow, &parentIndex)))
         return Relation();
@@ -830,7 +822,7 @@ XULTreeItemAccessibleBase::RelationByType(uint32_t aType)
       return Relation(treeAcc->GetTreeItemAccessible(parentIndex));
     }
 
-    case nsIAccessibleRelation::RELATION_NODE_PARENT_OF: {
+    case RelationType::NODE_PARENT_OF: {
       bool isTrue = false;
       if (NS_FAILED(mTreeView->IsContainerEmpty(mRow, &isTrue)) || isTrue)
         return Relation();
@@ -895,7 +887,7 @@ XULTreeItemAccessibleBase::DoAction(uint8_t aIndex)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// XULTreeItemAccessibleBase: nsAccessNode implementation
+// XULTreeItemAccessibleBase: Accessible implementation
 
 void
 XULTreeItemAccessibleBase::Shutdown()
@@ -907,10 +899,6 @@ XULTreeItemAccessibleBase::Shutdown()
   AccessibleWrap::Shutdown();
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// XULTreeItemAccessibleBase: Accessible public methods
-
-// nsIAccessible::groupPosition
 GroupPos
 XULTreeItemAccessibleBase::GroupPosition()
 {
@@ -958,8 +946,6 @@ XULTreeItemAccessibleBase::GroupPosition()
 uint64_t
 XULTreeItemAccessibleBase::NativeState()
 {
-  if (!mTreeView)
-    return states::DEFUNCT;
 
   // focusable and selectable states
   uint64_t state = NativeInteractiveState();
@@ -1064,8 +1050,6 @@ XULTreeItemAccessibleBase::GetSiblingAtOffset(int32_t aOffset,
 bool
 XULTreeItemAccessibleBase::IsExpandable()
 {
-  if (!mTreeView)
-    return false;
 
   bool isContainer = false;
   mTreeView->IsContainer(mRow, &isContainer);
@@ -1091,8 +1075,6 @@ XULTreeItemAccessibleBase::IsExpandable()
 void
 XULTreeItemAccessibleBase::GetCellName(nsITreeColumn* aColumn, nsAString& aName)
 {
-  if (!mTreeView)
-    return;
 
   mTreeView->GetCellText(mRow, aColumn, aName);
 
@@ -1146,7 +1128,7 @@ XULTreeItemAccessible::Name(nsString& aName)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// XULTreeItemAccessible: nsAccessNode implementation
+// XULTreeItemAccessible: Accessible implementation
 
 void
 XULTreeItemAccessible::Shutdown()
@@ -1155,9 +1137,6 @@ XULTreeItemAccessible::Shutdown()
   XULTreeItemAccessibleBase::Shutdown();
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// XULTreeItemAccessible: Accessible implementation
-
 role
 XULTreeItemAccessible::NativeRole()
 {
