@@ -20,6 +20,15 @@
 namespace mozilla {
 namespace dom {
 
+NS_IMPL_CYCLE_COLLECTION_1(URL, mSearchParams)
+
+NS_IMPL_CYCLE_COLLECTING_ADDREF(URL)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(URL)
+
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(URL)
+  NS_INTERFACE_MAP_ENTRY(nsISupports)
+NS_INTERFACE_MAP_END
+
 URL::URL(nsIURI* aURI)
   : mURI(aURI)
 {
@@ -211,7 +220,11 @@ URL::SetHref(const nsAString& aHref, ErrorResult& aRv)
     return;
   }
 
-  aRv = mURI->SetSpec(href);
+  mURI = uri;
+
+  if (mSearchParams) {
+    mSearchParams->Invalidate();
+  }
 }
 
 void
@@ -229,7 +242,7 @@ URL::GetProtocol(nsString& aProtocol) const
   }
 
   CopyASCIItoUTF16(protocol, aProtocol);
-  aProtocol.Append(PRUnichar(':'));
+  aProtocol.Append(char16_t(':'));
 }
 
 void
@@ -288,6 +301,33 @@ URL::SetHost(const nsAString& aHost)
   mURI->SetHostPort(NS_ConvertUTF16toUTF8(aHost));
 }
 
+void
+URL::URLSearchParamsUpdated()
+{
+  MOZ_ASSERT(mSearchParams && mSearchParams->IsValid());
+
+  nsAutoString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
+void
+URL::URLSearchParamsNeedsUpdates()
+{
+  MOZ_ASSERT(mSearchParams);
+
+  nsAutoCString search;
+  nsCOMPtr<nsIURL> url(do_QueryInterface(mURI));
+  if (url) {
+    nsresult rv = url->GetQuery(search);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Failed to get the query from a nsIURL.");
+    }
+  }
+
+  mSearchParams->ParseInput(search);
+}
+
 void
 URL::GetHostname(nsString& aHostname) const
 {
@@ -319,9 +359,14 @@ URL::SetPort(const nsAString& aPort)
 {
   nsresult rv;
   nsAutoString portStr(aPort);
-  int32_t port = portStr.ToInteger(&rv);
-  if (NS_FAILED(rv)) {
-    return;
+  int32_t port = -1;
+
+  // nsIURI uses -1 as default value.
+  if (!portStr.IsEmpty()) {
+    port = portStr.ToInteger(&rv);
+    if (NS_FAILED(rv)) {
+      return;
+    }
   }
 
   mURI->SetPort(port);
@@ -379,6 +424,16 @@ URL::GetSearch(nsString& aSearch) const
 
 void
 URL::SetSearch(const nsAString& aSearch)
+{
+  SetSearchInternal(aSearch);
+
+  if (mSearchParams) {
+    mSearchParams->Invalidate();
+  }
+}
+
+void
+URL::SetSearchInternal(const nsAString& aSearch)
 {
   nsCOMPtr<nsIURL> url(do_QueryInterface(mURI));
   if (!url) {
@@ -389,6 +444,35 @@ URL::SetSearch(const nsAString& aSearch)
   url->SetQuery(NS_ConvertUTF16toUTF8(aSearch));
 }
 
+URLSearchParams*
+URL::GetSearchParams()
+{
+  CreateSearchParamsIfNeeded();
+  return mSearchParams;
+}
+
+void
+URL::SetSearchParams(URLSearchParams* aSearchParams)
+{
+  if (!aSearchParams) {
+    return;
+  }
+
+  if (!aSearchParams->HasURLAssociated()) {
+    MOZ_ASSERT(aSearchParams->IsValid());
+
+    mSearchParams = aSearchParams;
+    mSearchParams->SetObserver(this);
+  } else {
+    CreateSearchParamsIfNeeded();
+    mSearchParams->CopyFromURLSearchParams(*aSearchParams);
+  }
+
+  nsAutoString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
 void
 URL::GetHash(nsString& aHash) const
 {
@@ -398,7 +482,7 @@ URL::GetHash(nsString& aHash) const
   nsresult rv = mURI->GetRef(ref);
   if (NS_SUCCEEDED(rv) && !ref.IsEmpty()) {
     NS_UnescapeURL(ref); // XXX may result in random non-ASCII bytes!
-    aHash.Assign(PRUnichar('#'));
+    aHash.Assign(char16_t('#'));
     AppendUTF8toUTF16(ref, aHash);
   }
 }
@@ -409,5 +493,23 @@ URL::SetHash(const nsAString& aHash)
   mURI->SetRef(NS_ConvertUTF16toUTF8(aHash));
 }
 
+bool IsChromeURI(nsIURI* aURI)
+{
+  bool isChrome = false;
+  if (NS_SUCCEEDED(aURI->SchemeIs("chrome", &isChrome)))
+      return isChrome;
+  return false;
+}
+
+void
+URL::CreateSearchParamsIfNeeded()
+{
+  if (!mSearchParams) {
+    mSearchParams = new URLSearchParams();
+    mSearchParams->SetObserver(this);
+    mSearchParams->Invalidate();
+  }
+}
+
 }
 }
