@@ -60,8 +60,8 @@ enum UIStateChangeType
 };
 
 #define NS_PIDOMWINDOW_IID \
-{ 0x4f4eadf9, 0xe795, 0x48e5, \
-  { 0x89, 0x4b, 0x04, 0x40, 0xb2, 0x5d, 0xa6, 0xfa } }
+{ 0xf26953de, 0xa799, 0x4a92, \
+  { 0x87, 0x49, 0x7c, 0x37, 0xe5, 0x90, 0x3f, 0x37 } }
 
 class nsPIDOMWindow : public nsIDOMWindowInternal
 {
@@ -70,39 +70,37 @@ public:
 
   virtual nsPIDOMWindow* GetPrivateRoot() = 0;
 
+  // Outer windows only.
   virtual void ActivateOrDeactivate(bool aActivate) = 0;
 
   // this is called GetTopWindowRoot to avoid conflicts with nsIDOMWindow::GetWindowRoot
   virtual already_AddRefed<nsPIWindowRoot> GetTopWindowRoot() = 0;
 
-  virtual void SetActive(bool aActive)
-  {
-    NS_PRECONDITION(IsOuterWindow(),
-                    "active state is only maintained on outer windows");
-    mIsActive = aActive;
-  }
-
+  // Inner windows only.
   virtual nsresult RegisterIdleObserver(nsIIdleObserver* aIdleObserver) = 0;
   virtual nsresult UnregisterIdleObserver(nsIIdleObserver* aIdleObserver) = 0;
 
+  // Outer windows only.
+  virtual void SetActive(bool aActive)
+  {
+    MOZ_ASSERT(IsOuterWindow());
+    mIsActive = aActive;
+  }
   bool IsActive()
   {
-    NS_PRECONDITION(IsOuterWindow(),
-                    "active state is only maintained on outer windows");
+    MOZ_ASSERT(IsOuterWindow());
     return mIsActive;
   }
 
+  // Outer windows only.
   virtual void SetIsBackground(bool aIsBackground)
   {
-    NS_PRECONDITION(IsOuterWindow(),
-                    "background state is only maintained on outer windows");
+    MOZ_ASSERT(IsOuterWindow());
     mIsBackground = aIsBackground;
   }
-
   bool IsBackground()
   {
-    NS_PRECONDITION(IsOuterWindow(),
-                    "background state is only maintained on outer windows");
+    MOZ_ASSERT(IsOuterWindow());
     return mIsBackground;
   }
 
@@ -111,6 +109,7 @@ public:
     return mChromeEventHandler;
   }
 
+  // Outer windows only.
   virtual void SetChromeEventHandler(mozilla::dom::EventTarget* aChromeEventHandler) = 0;
 
   mozilla::dom::EventTarget* GetParentTarget()
@@ -189,6 +188,8 @@ public:
     return mDoc;
   }
 
+  virtual NS_HIDDEN_(bool) IsRunningTimeout() = 0;
+
 protected:
   // Lazily instantiate an about:blank document if necessary, and if
   // we have what it takes to do so.
@@ -325,13 +326,24 @@ public:
     return mIsInnerWindow;
   }
 
+  // Returns true if this object has an outer window and it is the current inner
+  // window of that outer. Only call this on inner windows.
+  bool IsCurrentInnerWindow() const
+  {
+    MOZ_ASSERT(IsInnerWindow(),
+               "It doesn't make sense to call this on outer windows.");
+    return mOuterWindow && mOuterWindow->GetCurrentInnerWindow() == this;
+  }
 
+  // Returns true if the document of this window is the active document.  This
+  // is not identical to IsCurrentInnerWindow() because document.open() will
+  // keep the same document active but create a new window.
   bool HasActiveDocument()
   {
-    return GetOuterWindow() &&
-      (GetOuterWindow()->GetCurrentInnerWindow() == this ||
-       (GetOuterWindow()->GetCurrentInnerWindow() &&
-        GetOuterWindow()->GetCurrentInnerWindow()->GetDoc() == mDoc));
+    return IsCurrentInnerWindow() ||
+      (GetOuterWindow() &&
+       GetOuterWindow()->GetCurrentInnerWindow() &&
+       GetOuterWindow()->GetCurrentInnerWindow()->GetDoc() == mDoc);
   }
 
   bool IsOuterWindow() const
@@ -553,12 +565,18 @@ public:
   virtual nsresult DispatchSyncPopState() = 0;
 
   /**
-   * Tell this window that it should listen for sensor changes of the given type.
+   * Tell this window that it should listen for sensor changes of the given
+   * type.
+   *
+   * Inner windows only.
    */
   virtual void EnableDeviceSensor(uint32_t aType) = 0;
 
   /**
-   * Tell this window that it should remove itself from sensor change notifications.
+   * Tell this window that it should remove itself from sensor change
+   * notifications.
+   *
+   * Inner windows only.
    */
   virtual void DisableDeviceSensor(uint32_t aType) = 0;
 
@@ -569,12 +587,16 @@ public:
   /**
    * Tell the window that it should start to listen to the network event of the
    * given aType.
+   *
+   * Inner windows only.
    */
   virtual void EnableNetworkEvent(uint32_t aType) = 0;
 
   /**
    * Tell the window that it should stop to listen to the network event of the
    * given aType.
+   *
+   * Inner windows only.
    */
   virtual void DisableNetworkEvent(uint32_t aType) = 0;
 #endif // MOZ_B2G
@@ -651,6 +673,15 @@ public:
   // WebIDL-ish APIs
   nsPerformance* GetPerformance();
 
+  void MarkUncollectableForCCGeneration(uint32_t aGeneration)
+  {
+    mMarkedCCGeneration = aGeneration;
+  }
+
+  uint32_t GetMarkedCCGeneration()
+  {
+    return mMarkedCCGeneration;
+  }
 protected:
   // The nsPIDOMWindow constructor. The aOuterWindow argument should
   // be null if and only if the created window itself is an outer
@@ -734,6 +765,8 @@ protected:
   // This is only used by the inner window. Set to true once we've sent
   // the (chrome|content)-document-global-created notification.
   bool mHasNotifiedGlobalCreated;
+
+  uint32_t mMarkedCCGeneration;
 };
 
 
