@@ -3,9 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
+#include "mozilla/MemoryReporting.h"
 
-#if defined(MOZ_WIDGET_GTK2)
+#if (MOZ_WIDGET_GTK == 2)
 #include "gfxPlatformGtk.h"
 #define gfxToolkitPlatform gfxPlatformGtk
 #elif defined(MOZ_WIDGET_QT)
@@ -50,7 +51,10 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsISimpleEnumerator.h"
+#include "nsIMemory.h"
+#include "gfxFontConstants.h"
 
+#include "mozilla/Preferences.h"
 #include "mozilla/scache/StartupCache.h"
 #include <sys/stat.h>
 
@@ -62,12 +66,21 @@
 using namespace mozilla;
 
 #ifdef PR_LOGGING
-static PRLogModuleInfo *gFontInfoLog = PR_NewLogModule("fontInfoLog");
+static PRLogModuleInfo *
+GetFontInfoLog()
+{
+    static PRLogModuleInfo *sLog;
+    if (!sLog)
+        sLog = PR_NewLogModule("fontInfoLog");
+    return sLog;
+}
 #endif /* PR_LOGGING */
 
 #undef LOG
-#define LOG(args) PR_LOG(gFontInfoLog, PR_LOG_DEBUG, args)
-#define LOG_ENABLED() PR_LOG_TEST(gFontInfoLog, PR_LOG_DEBUG)
+#define LOG(args) PR_LOG(GetFontInfoLog(), PR_LOG_DEBUG, args)
+#define LOG_ENABLED() PR_LOG_TEST(GetFontInfoLog(), PR_LOG_DEBUG)
+
+static cairo_user_data_key_t sFTUserFontDataKey;
 
 static __inline void
 BuildKeyNameFromFontName(nsAString &aName)
@@ -79,6 +92,96 @@ BuildKeyNameFromFontName(nsAString &aName)
     ToLowerCase(aName);
 }
 
+// Helper to access the FT_Face for a given FT2FontEntry,
+// creating a temporary face if the entry does not have one yet.
+// This allows us to read font names, tables, etc if necessary
+// without permanently instantiating a freetype face and consuming
+// memory long-term.
+// This may fail (resulting in a null FT_Face), e.g. if it fails to
+// allocate memory to uncompress a font from omnijar.
+class AutoFTFace {
+public:
+    AutoFTFace(FT2FontEntry* aFontEntry)
+        : mFace(nullptr), mFontDataBuf(nullptr), mOwnsFace(false)
+    {
+        if (aFontEntry->mFTFace) {
+            mFace = aFontEntry->mFTFace;
+            return;
+        }
+
+        NS_ASSERTION(!aFontEntry->mFilename.IsEmpty(),
+                     "can't use AutoFTFace for fonts without a filename");
+        FT_Library ft = gfxToolkitPlatform::GetPlatform()->GetFTLibrary();
+
+        // A relative path (no initial "/") means this is a resource in
+        // omnijar, not an installed font on the device.
+        // The NS_ASSERTIONs here should never fail, as the resource must have
+        // been read successfully during font-list initialization or we'd never
+        // have created the font entry. The only legitimate runtime failure
+        // here would be memory allocation, in which case mFace remains null.
+        if (aFontEntry->mFilename[0] != '/') {
+            nsRefPtr<nsZipArchive> reader =
+                Omnijar::GetReader(Omnijar::Type::GRE);
+            nsZipItem *item = reader->GetItem(aFontEntry->mFilename.get());
+            NS_ASSERTION(item, "failed to find zip entry");
+
+            uint32_t bufSize = item->RealSize();
+            mFontDataBuf = static_cast<uint8_t*>(moz_malloc(bufSize));
+            if (mFontDataBuf) {
+                nsZipCursor cursor(item, reader, mFontDataBuf, bufSize);
+                cursor.Copy(&bufSize);
+                NS_ASSERTION(bufSize == item->RealSize(),
+                             "error reading bundled font");
+
+                if (FT_Err_Ok != FT_New_Memory_Face(ft, mFontDataBuf, bufSize,
+                                                    aFontEntry->mFTFontIndex,
+                                                    &mFace)) {
+                    NS_WARNING("failed to create freetype face");
+                }
+            }
+        } else {
+            if (FT_Err_Ok != FT_New_Face(ft, aFontEntry->mFilename.get(),
+                                         aFontEntry->mFTFontIndex, &mFace)) {
+                NS_WARNING("failed to create freetype face");
+            }
+        }
+        if (FT_Err_Ok != FT_Select_Charmap(mFace, FT_ENCODING_UNICODE)) {
+            NS_WARNING("failed to select Unicode charmap");
+        }
+        mOwnsFace = true;
+    }
+
+    ~AutoFTFace() {
+        if (mFace && mOwnsFace) {
+            FT_Done_Face(mFace);
+            if (mFontDataBuf) {
+                moz_free(mFontDataBuf);
+            }
+        }
+    }
+
+    operator FT_Face() { return mFace; }
+
+    // If we 'forget' the FT_Face (used when ownership is handed over to Cairo),
+    // we do -not- free the mFontDataBuf (if used); that also becomes the
+    // responsibility of the new owner of the face.
+    FT_Face forget() {
+        NS_ASSERTION(mOwnsFace, "can't forget() when we didn't own the face");
+        mOwnsFace = false;
+        return mFace;
+    }
+
+    const uint8_t* FontData() const { return mFontDataBuf; }
+
+private:
+    FT_Face  mFace;
+    uint8_t* mFontDataBuf; // Uncompressed data (for fonts stored in a JAR),
+                           // or null for fonts instantiated from a file.
+                           // If non-null, this must survive as long as the
+                           // FT_Face.
+    bool     mOwnsFace;
+};
+
 /*
  * FT2FontEntry
  * gfxFontEntry subclass corresponding to a specific face that can be
@@ -92,7 +195,12 @@ BuildKeyNameFromFontName(nsAString &aName)
 cairo_scaled_font_t *
 FT2FontEntry::CreateScaledFont(const gfxFontStyle *aStyle)
 {
-    cairo_scaled_font_t *scaledFont = NULL;
+    cairo_font_face_t *cairoFace = CairoFontFace();
+    if (!cairoFace) {
+        return nullptr;
+    }
+
+    cairo_scaled_font_t *scaledFont = nullptr;
 
     cairo_matrix_t sizeMatrix;
     cairo_matrix_t identityMatrix;
@@ -121,11 +229,11 @@ FT2FontEntry::CreateScaledFont(const gfxFontStyle *aStyle)
 
     cairo_font_options_t *fontOptions = cairo_font_options_create();
 
-    if (!gfxPlatform::GetPlatform()->FontHintingEnabled()) {
+    if (gfxPlatform::GetPlatform()->RequiresLinearZoom()) {
         cairo_font_options_set_hint_metrics(fontOptions, CAIRO_HINT_METRICS_OFF);
     }
 
-    scaledFont = cairo_scaled_font_create(CairoFontFace(),
+    scaledFont = cairo_scaled_font_create(cairoFace,
                                           &sizeMatrix,
                                           &identityMatrix, fontOptions);
     cairo_font_options_destroy(fontOptions);
@@ -153,6 +261,9 @@ gfxFont*
 FT2FontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, bool aNeedsBold)
 {
     cairo_scaled_font_t *scaledFont = CreateScaledFont(aFontStyle);
+    if (!scaledFont) {
+        return nullptr;
+    }
     gfxFont *font = new gfxFT2Font(scaledFont, this, aFontStyle, aNeedsBold);
     cairo_scaled_font_destroy(scaledFont);
     return font;
@@ -189,6 +300,7 @@ FT2FontEntry::CreateFontEntry(const gfxProxyFontEntry &aProxyEntry,
         fe->mItalic = aProxyEntry.mItalic;
         fe->mWeight = aProxyEntry.mWeight;
         fe->mStretch = aProxyEntry.mStretch;
+        fe->mIsUserFont = true;
     }
     return fe;
 }
@@ -208,6 +320,8 @@ public:
         }
     }
 
+    const uint8_t *FontData() const { return mFontData; }
+
 private:
     FT_Face        mFace;
     const uint8_t *mFontData;
@@ -233,28 +347,16 @@ FT2FontEntry::CreateFontEntry(const FontListEntry& aFLE)
     return fe;
 }
 
-/* static */
-FT2FontEntry*
-FT2FontEntry::CreateFontEntry(FT_Face aFace,
-                              const char* aFilename, uint8_t aIndex,
-                              const nsAString& aName,
-                              const uint8_t *aFontData)
+// Helpers to extract font entry properties from an FT_Face
+static bool
+FTFaceIsItalic(FT_Face aFace)
 {
-    static cairo_user_data_key_t key;
-
-    FT2FontEntry *fe = new FT2FontEntry(aName);
-    fe->mItalic = aFace->style_flags & FT_STYLE_FLAG_ITALIC;
-    fe->mFTFace = aFace;
-    int flags = gfxPlatform::GetPlatform()->FontHintingEnabled() ?
-                FT_LOAD_DEFAULT :
-                (FT_LOAD_NO_AUTOHINT | FT_LOAD_NO_HINTING);
-    fe->mFontFace = cairo_ft_font_face_create_for_ft_face(aFace, flags);
-    fe->mFilename = aFilename;
-    fe->mFTFontIndex = aIndex;
-    FTUserFontData *userFontData = new FTUserFontData(aFace, aFontData);
-    cairo_font_face_set_user_data(fe->mFontFace, &key,
-                                  userFontData, FTFontDestroyFunc);
+    return !!(aFace->style_flags & FT_STYLE_FLAG_ITALIC);
+}
 
+static uint16_t
+FTFaceGetWeight(FT_Face aFace)
+{
     TT_OS2 *os2 = static_cast<TT_OS2*>(FT_Get_Sfnt_Table(aFace, ft_sfnt_os2));
     uint16_t os2weight = 0;
     if (os2 && os2->version != 0xffff) {
@@ -262,20 +364,57 @@ FT2FontEntry::CreateFontEntry(FT_Face aFace,
         // have this set wrong -- e.g. "Microsoft Logo Bold Italic" has
         // it set to 6 instead of 600.  We try to be nice and handle that
         // as well.
-        if (os2->usWeightClass >= 100 && os2->usWeightClass <= 900)
+        if (os2->usWeightClass >= 100 && os2->usWeightClass <= 900) {
             os2weight = os2->usWeightClass;
-        else if (os2->usWeightClass >= 1 && os2->usWeightClass <= 9)
+        } else if (os2->usWeightClass >= 1 && os2->usWeightClass <= 9) {
             os2weight = os2->usWeightClass * 100;
+        }
     }
 
-    if (os2weight != 0)
-        fe->mWeight = os2weight;
-    else if (aFace->style_flags & FT_STYLE_FLAG_BOLD)
-        fe->mWeight = 700;
-    else
-        fe->mWeight = 400;
+    uint16_t result;
+    if (os2weight != 0) {
+        result = os2weight;
+    } else if (aFace->style_flags & FT_STYLE_FLAG_BOLD) {
+        result = 700;
+    } else {
+        result = 400;
+    }
 
-    NS_ASSERTION(fe->mWeight >= 100 && fe->mWeight <= 900, "Invalid final weight in font!");
+    NS_ASSERTION(result >= 100 && result <= 900, "Invalid weight in font!");
+
+    return result;
+}
+
+// Used to create the font entry for installed faces on the device,
+// when iterating over the fonts directories.
+// We use the FT_Face to retrieve the details needed for the font entry,
+// but unless we have been passed font data (i.e. for a user font),
+// we do *not* save a reference to it, nor create a cairo face,
+// as we don't want to keep a freetype face for every installed font
+// permanently in memory.
+/* static */
+FT2FontEntry*
+FT2FontEntry::CreateFontEntry(FT_Face aFace,
+                              const char* aFilename, uint8_t aIndex,
+                              const nsAString& aName,
+                              const uint8_t *aFontData)
+{
+    FT2FontEntry *fe = new FT2FontEntry(aName);
+    fe->mItalic = FTFaceIsItalic(aFace);
+    fe->mWeight = FTFaceGetWeight(aFace);
+    fe->mFilename = aFilename;
+    fe->mFTFontIndex = aIndex;
+
+    if (aFontData) {
+        fe->mFTFace = aFace;
+        int flags = gfxPlatform::GetPlatform()->FontHintingEnabled() ?
+                    FT_LOAD_DEFAULT :
+                    (FT_LOAD_NO_AUTOHINT | FT_LOAD_NO_HINTING);
+        fe->mFontFace = cairo_ft_font_face_create_for_ft_face(aFace, flags);
+        FTUserFontData *userFontData = new FTUserFontData(aFace, aFontData);
+        cairo_font_face_set_user_data(fe->mFontFace, &sFTUserFontDataKey,
+                                      userFontData, FTFontDestroyFunc);
+    }
 
     return fe;
 }
@@ -306,32 +445,25 @@ gfxFT2Font::GetFontEntry()
 cairo_font_face_t *
 FT2FontEntry::CairoFontFace()
 {
-    static cairo_user_data_key_t key;
-
     if (!mFontFace) {
-        FT_Face face;
-        if (FT_Err_Ok != FT_New_Face(gfxToolkitPlatform::GetPlatform()->GetFTLibrary(),
-                                     mFilename.get(), mFTFontIndex, &face)) {
-            NS_WARNING("failed to create freetype face");
+        AutoFTFace face(this);
+        if (!face) {
             return nullptr;
         }
-        if (FT_Err_Ok != FT_Select_Charmap(face, FT_ENCODING_UNICODE)) {
-            NS_WARNING("failed to select Unicode charmap, text may be garbled");
-        }
-        mFTFace = face;
+        mFTFace = face.forget();
         int flags = gfxPlatform::GetPlatform()->FontHintingEnabled() ?
                     FT_LOAD_DEFAULT :
                     (FT_LOAD_NO_AUTOHINT | FT_LOAD_NO_HINTING);
         mFontFace = cairo_ft_font_face_create_for_ft_face(face, flags);
-        FTUserFontData *userFontData = new FTUserFontData(face, nullptr);
-        cairo_font_face_set_user_data(mFontFace, &key,
+        FTUserFontData *userFontData = new FTUserFontData(face, face.FontData());
+        cairo_font_face_set_user_data(mFontFace, &sFTUserFontDataKey,
                                       userFontData, FTFontDestroyFunc);
     }
     return mFontFace;
 }
 
 nsresult
-FT2FontEntry::ReadCMAP()
+FT2FontEntry::ReadCMAP(FontInfoData *aFontInfoData)
 {
     if (mCharacterMap) {
         return NS_OK;
@@ -340,7 +472,7 @@ FT2FontEntry::ReadCMAP()
     nsRefPtr<gfxCharacterMap> charmap = new gfxCharacterMap();
 
     AutoFallibleTArray<uint8_t,16384> buffer;
-    nsresult rv = GetFontTable(TTAG_cmap, buffer);
+    nsresult rv = CopyFontTable(TTAG_cmap, buffer);
     
     if (NS_SUCCEEDED(rv)) {
         bool unicodeFont;
@@ -362,16 +494,17 @@ FT2FontEntry::ReadCMAP()
 }
 
 nsresult
-FT2FontEntry::GetFontTable(uint32_t aTableTag,
-                           FallibleTArray<uint8_t>& aBuffer)
+FT2FontEntry::CopyFontTable(uint32_t aTableTag,
+                            FallibleTArray<uint8_t>& aBuffer)
 {
-    // Ensure existence of mFTFace
-    CairoFontFace();
-    NS_ENSURE_TRUE(mFTFace, NS_ERROR_FAILURE);
+    AutoFTFace face(this);
+    if (!face) {
+        return NS_ERROR_FAILURE;
+    }
 
     FT_Error status;
     FT_ULong len = 0;
-    status = FT_Load_Sfnt_Table(mFTFace, aTableTag, 0, nullptr, &len);
+    status = FT_Load_Sfnt_Table(face, aTableTag, 0, nullptr, &len);
     if (status != FT_Err_Ok || len == 0) {
         return NS_ERROR_FAILURE;
     }
@@ -380,27 +513,45 @@ FT2FontEntry::GetFontTable(uint32_t aTableTag,
         return NS_ERROR_OUT_OF_MEMORY;
     }
     uint8_t *buf = aBuffer.Elements();
-    status = FT_Load_Sfnt_Table(mFTFace, aTableTag, 0, buf, &len);
+    status = FT_Load_Sfnt_Table(face, aTableTag, 0, buf, &len);
     NS_ENSURE_TRUE(status == FT_Err_Ok, NS_ERROR_FAILURE);
 
     return NS_OK;
 }
 
-void
-FT2FontEntry::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf,
-                                  FontListSizes*    aSizes) const
+hb_blob_t*
+FT2FontEntry::GetFontTable(uint32_t aTableTag)
 {
-    gfxFontEntry::SizeOfExcludingThis(aMallocSizeOf, aSizes);
+    if (mFontFace) {
+        // if there's a cairo font face, we may be able to return a blob
+        // that just wraps a range of the attached user font data
+        FTUserFontData *userFontData = static_cast<FTUserFontData*>(
+            cairo_font_face_get_user_data(mFontFace, &sFTUserFontDataKey));
+        if (userFontData && userFontData->FontData()) {
+            return GetTableFromFontData(userFontData->FontData(), aTableTag);
+        }
+    }
+
+    // otherwise, use the default method (which in turn will call our
+    // implementation of CopyFontTable)
+    return gfxFontEntry::GetFontTable(aTableTag);
+}
+
+void
+FT2FontEntry::AddSizeOfExcludingThis(MallocSizeOf aMallocSizeOf,
+                                     FontListSizes* aSizes) const
+{
+    gfxFontEntry::AddSizeOfExcludingThis(aMallocSizeOf, aSizes);
     aSizes->mFontListSize +=
         mFilename.SizeOfExcludingThisIfUnshared(aMallocSizeOf);
 }
 
 void
-FT2FontEntry::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf,
-                                  FontListSizes*    aSizes) const
+FT2FontEntry::AddSizeOfIncludingThis(MallocSizeOf aMallocSizeOf,
+                                     FontListSizes* aSizes) const
 {
     aSizes->mFontListSize += aMallocSizeOf(this);
-    SizeOfExcludingThis(aMallocSizeOf, aSizes);
+    AddSizeOfExcludingThis(aMallocSizeOf, aSizes);
 }
 
 /*
@@ -451,7 +602,7 @@ public:
             MoveEntry,
             PL_DHashClearEntryStub,
             PL_DHashFinalizeStub,
-            NULL
+            nullptr
         };
 
         if (!PL_DHashTableInit(&mMap, &mOps, nullptr,
@@ -510,12 +661,12 @@ public:
             if (!(end = strchr(beginning, ';'))) {
                 break;
             }
-            uint32_t timestamp = strtoul(beginning, NULL, 10);
+            uint32_t timestamp = strtoul(beginning, nullptr, 10);
             beginning = end + 1;
             if (!(end = strchr(beginning, ';'))) {
                 break;
             }
-            uint32_t filesize = strtoul(beginning, NULL, 10);
+            uint32_t filesize = strtoul(beginning, nullptr, 10);
 
             FNCMapEntry* mapEntry =
                 static_cast<FNCMapEntry*>
@@ -539,7 +690,7 @@ public:
     }
 
     virtual void
-    GetInfoForFile(nsCString& aFileName, nsCString& aFaceList,
+    GetInfoForFile(const nsCString& aFileName, nsCString& aFaceList,
                    uint32_t *aTimestamp, uint32_t *aFilesize)
     {
         if (!mMap.ops) {
@@ -551,7 +702,7 @@ public:
             return;
         }
         FNCMapEntry* entry = static_cast<FNCMapEntry*>(hdr);
-        if (entry && entry->mTimestamp && entry->mFilesize) {
+        if (entry && entry->mFilesize) {
             *aTimestamp = entry->mTimestamp;
             *aFilesize = entry->mFilesize;
             aFaceList.Assign(entry->mFaces);
@@ -563,7 +714,7 @@ public:
     }
 
     virtual void
-    CacheFileInfo(nsCString& aFileName, nsCString& aFaceList,
+    CacheFileInfo(const nsCString& aFileName, const nsCString& aFaceList,
                   uint32_t aTimestamp, uint32_t aFilesize)
     {
         if (!mMap.ops) {
@@ -660,9 +811,9 @@ gfxFT2FontList::gfxFT2FontList()
 }
 
 void
-gfxFT2FontList::AppendFacesFromCachedFaceList(nsCString& aFileName,
+gfxFT2FontList::AppendFacesFromCachedFaceList(const nsCString& aFileName,
                                               bool aStdFile,
-                                              nsCString& aFaceList)
+                                              const nsCString& aFaceList)
 {
     const char *beginning = aFaceList.get();
     const char *end = strchr(beginning, ',');
@@ -680,7 +831,7 @@ gfxFT2FontList::AppendFacesFromCachedFaceList(nsCString& aFileName,
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        uint32_t index = strtoul(beginning, NULL, 10);
+        uint32_t index = strtoul(beginning, nullptr, 10);
         beginning = end + 1;
         if (!(end = strchr(beginning, ','))) {
             break;
@@ -690,12 +841,12 @@ gfxFT2FontList::AppendFacesFromCachedFaceList(nsCString& aFileName,
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        uint32_t weight = strtoul(beginning, NULL, 10);
+        uint32_t weight = strtoul(beginning, nullptr, 10);
         beginning = end + 1;
         if (!(end = strchr(beginning, ','))) {
             break;
         }
-        int32_t stretch = strtol(beginning, NULL, 10);
+        int32_t stretch = strtol(beginning, nullptr, 10);
 
         FontListEntry fle(familyName, faceName, aFileName,
                           weight, stretch, italic, index);
@@ -725,36 +876,37 @@ AppendToFaceList(nsCString& aFaceList,
 }
 
 void
-FT2FontEntry::CheckForBrokenFont()
+FT2FontEntry::CheckForBrokenFont(gfxFontFamily *aFamily)
 {
-    NS_ASSERTION(mFamily != nullptr, "font entry must belong to a family");
-
     // note if the family is in the "bad underline" blacklist
-    if (mFamily->IsBadUnderlineFamily()) {
+    if (aFamily->IsBadUnderlineFamily()) {
         mIsBadUnderlineFont = true;
     }
 
     // bug 721719 - set the IgnoreGSUB flag on entries for Roboto
     // because of unwanted on-by-default "ae" ligature.
     // (See also AppendFaceFromFontListEntry.)
-    if (mFamily->Name().EqualsLiteral("roboto")) {
+    if (aFamily->Name().EqualsLiteral("roboto")) {
         mIgnoreGSUB = true;
     }
 
     // bug 706888 - set the IgnoreGSUB flag on the broken version of
     // Droid Sans Arabic from certain phones, as identified by the
     // font checksum in the 'head' table
-    else if (mFamily->Name().EqualsLiteral("droid sans arabic")) {
-        const TT_Header *head = static_cast<const TT_Header*>
-            (FT_Get_Sfnt_Table(mFTFace, ft_sfnt_head));
-        if (head && head->CheckSum_Adjust == 0xe445242) {
-            mIgnoreGSUB = true;
+    else if (aFamily->Name().EqualsLiteral("droid sans arabic")) {
+        AutoFTFace face(this);
+        if (face) {
+            const TT_Header *head = static_cast<const TT_Header*>
+                (FT_Get_Sfnt_Table(face, ft_sfnt_head));
+            if (head && head->CheckSum_Adjust == 0xe445242) {
+                mIgnoreGSUB = true;
+            }
         }
     }
 }
 
 void
-gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
+gfxFT2FontList::AppendFacesFromFontFile(const nsCString& aFileName,
                                         bool aStdFile,
                                         FontNameCache *aCache)
 {
@@ -790,41 +942,8 @@ gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
             if (FT_Err_Ok != FT_New_Face(ftLibrary, aFileName.get(), i, &face)) {
                 continue;
             }
-            if (FT_Err_Ok != FT_Select_Charmap(face, FT_ENCODING_UNICODE)) {
-                FT_Done_Face(face);
-                continue;
-            }
-            FT2FontEntry* fe =
-                CreateNamedFontEntry(face, aFileName.get(), i);
-            if (fe) {
-                NS_ConvertUTF8toUTF16 name(face->family_name);
-                BuildKeyNameFromFontName(name);       
-                gfxFontFamily *family = mFontFamilies.GetWeak(name);
-                if (!family) {
-                    family = new FT2FontFamily(name);
-                    mFontFamilies.Put(name, family);
-                    if (mBadUnderlineFamilyNames.Contains(name)) {
-                        family->SetBadUnderlineFamily();
-                    }
-                }
-                fe->mStandardFace = aStdFile;
-                family->AddFontEntry(fe);
-
-                // this depends on the entry having been added to its family
-                fe->CheckForBrokenFont();
-
-                AppendToFaceList(faceList, name, fe);
-#ifdef PR_LOGGING
-                if (LOG_ENABLED()) {
-                    LOG(("(fontinit) added (%s) to family (%s)"
-                         " with style: %s weight: %d stretch: %d",
-                         NS_ConvertUTF16toUTF8(fe->Name()).get(), 
-                         NS_ConvertUTF16toUTF8(family->Name()).get(), 
-                         fe->IsItalic() ? "italic" : "normal",
-                         fe->Weight(), fe->Stretch()));
-                }
-#endif
-            }
+            AddFaceToList(aFileName, i, aStdFile, face, faceList);
+            FT_Done_Face(face);
         }
         FT_Done_Face(dummy);
         if (aCache && 0 == statRetval && !faceList.IsEmpty()) {
@@ -833,6 +952,160 @@ gfxFT2FontList::AppendFacesFromFontFile(nsCString& aFileName,
     }
 }
 
+#define JAR_LAST_MODIFED_TIME "jar-last-modified-time"
+
+void
+gfxFT2FontList::FindFontsInOmnijar(FontNameCache *aCache)
+{
+    bool jarChanged = false;
+
+    mozilla::scache::StartupCache* cache =
+        mozilla::scache::StartupCache::GetSingleton();
+    char *cachedModifiedTimeBuf;
+    uint32_t longSize;
+    int64_t jarModifiedTime;
+    if (cache &&
+        NS_SUCCEEDED(cache->GetBuffer(JAR_LAST_MODIFED_TIME,
+                                      &cachedModifiedTimeBuf,
+                                      &longSize)) &&
+        longSize == sizeof(int64_t))
+    {
+        nsCOMPtr<nsIFile> jarFile = Omnijar::GetPath(Omnijar::Type::GRE);
+        jarFile->GetLastModifiedTime(&jarModifiedTime);
+        if (jarModifiedTime > *(int64_t*)cachedModifiedTimeBuf) {
+            jarChanged = true;
+        }
+    }
+
+    static const char* sJarSearchPaths[] = {
+        "res/fonts/*.ttf$",
+    };
+    nsRefPtr<nsZipArchive> reader = Omnijar::GetReader(Omnijar::Type::GRE);
+    for (unsigned i = 0; i < ArrayLength(sJarSearchPaths); i++) {
+        nsZipFind* find;
+        if (NS_SUCCEEDED(reader->FindInit(sJarSearchPaths[i], &find))) {
+            const char* path;
+            uint16_t len;
+            while (NS_SUCCEEDED(find->FindNext(&path, &len))) {
+                nsCString entryName(path, len);
+                AppendFacesFromOmnijarEntry(reader, entryName, aCache,
+                                            jarChanged);
+            }
+            delete find;
+        }
+    }
+
+    if (cache) {
+        cache->PutBuffer(JAR_LAST_MODIFED_TIME, (char*)&jarModifiedTime,
+                         sizeof(jarModifiedTime));
+    }
+}
+
+// Given the freetype face corresponding to an entryName and face index,
+// add the face to the available font list and to the faceList string
+void
+gfxFT2FontList::AddFaceToList(const nsCString& aEntryName, uint32_t aIndex,
+                              bool aStdFile, FT_Face aFace,
+                              nsCString& aFaceList)
+{
+    if (FT_Err_Ok != FT_Select_Charmap(aFace, FT_ENCODING_UNICODE)) {
+        // ignore faces that don't support a Unicode charmap
+        return;
+    }
+
+    // build the font entry name and create an FT2FontEntry,
+    // but do -not- keep a reference to the FT_Face
+    FT2FontEntry* fe =
+        CreateNamedFontEntry(aFace, aEntryName.get(), aIndex);
+
+    if (fe) {
+        NS_ConvertUTF8toUTF16 name(aFace->family_name);
+        BuildKeyNameFromFontName(name);
+        gfxFontFamily *family = mFontFamilies.GetWeak(name);
+        if (!family) {
+            family = new FT2FontFamily(name);
+            mFontFamilies.Put(name, family);
+            if (mBadUnderlineFamilyNames.Contains(name)) {
+                family->SetBadUnderlineFamily();
+            }
+        }
+        fe->mStandardFace = aStdFile;
+        family->AddFontEntry(fe);
+
+        fe->CheckForBrokenFont(family);
+
+        AppendToFaceList(aFaceList, name, fe);
+#ifdef PR_LOGGING
+        if (LOG_ENABLED()) {
+            LOG(("(fontinit) added (%s) to family (%s)"
+                 " with style: %s weight: %d stretch: %d",
+                 NS_ConvertUTF16toUTF8(fe->Name()).get(),
+                 NS_ConvertUTF16toUTF8(family->Name()).get(),
+                 fe->IsItalic() ? "italic" : "normal",
+                 fe->Weight(), fe->Stretch()));
+        }
+#endif
+    }
+}
+
+void
+gfxFT2FontList::AppendFacesFromOmnijarEntry(nsZipArchive* aArchive,
+                                            const nsCString& aEntryName,
+                                            FontNameCache *aCache,
+                                            bool aJarChanged)
+{
+    nsCString faceList;
+    if (aCache && !aJarChanged) {
+        uint32_t filesize, timestamp;
+        aCache->GetInfoForFile(aEntryName, faceList, &timestamp, &filesize);
+        if (faceList.Length() > 0) {
+            AppendFacesFromCachedFaceList(aEntryName, true, faceList);
+            return;
+        }
+    }
+
+    nsZipItem *item = aArchive->GetItem(aEntryName.get());
+    NS_ASSERTION(item, "failed to find zip entry");
+
+    uint32_t bufSize = item->RealSize();
+    // We use fallible allocation here; if there's not enough RAM, we'll simply
+    // ignore the bundled fonts and fall back to the device's installed fonts.
+    nsAutoPtr<uint8_t> buf(static_cast<uint8_t*>(moz_malloc(bufSize)));
+    if (!buf) {
+        return;
+    }
+
+    nsZipCursor cursor(item, aArchive, buf, bufSize);
+    uint8_t* data = cursor.Copy(&bufSize);
+    NS_ASSERTION(data && bufSize == item->RealSize(),
+                 "error reading bundled font");
+    if (!data) {
+        return;
+    }
+
+    FT_Library ftLibrary = gfxAndroidPlatform::GetPlatform()->GetFTLibrary();
+
+    FT_Face dummy;
+    if (FT_Err_Ok != FT_New_Memory_Face(ftLibrary, buf, bufSize, 0, &dummy)) {
+        return;
+    }
+
+    for (FT_Long i = 0; i < dummy->num_faces; i++) {
+        FT_Face face;
+        if (FT_Err_Ok != FT_New_Memory_Face(ftLibrary, buf, bufSize, i, &face)) {
+            continue;
+        }
+        AddFaceToList(aEntryName, i, true, face, faceList);
+        FT_Done_Face(face);
+    }
+
+    FT_Done_Face(dummy);
+
+    if (aCache && !faceList.IsEmpty()) {
+        aCache->CacheFileInfo(aEntryName, faceList, 0, bufSize);
+    }
+}
+
 // Called on each family after all fonts are added to the list;
 // this will sort faces to give priority to "standard" font files
 // if aUserArg is non-null (i.e. we're using it as a boolean flag)
@@ -854,131 +1127,6 @@ FinalizeFamilyMemberList(nsStringHashKey::KeyType aKey,
     return PL_DHASH_NEXT;
 }
 
-#ifdef ANDROID
-
-#define JAR_READ_BUFFER_SIZE 1024
-
-nsresult
-CopyFromUriToFile(nsCString aSpec, nsIFile* aLocalFile)
-{
-    nsCOMPtr<nsIURI> uri;
-    nsCOMPtr<nsIInputStream> inputStream;
-    nsresult rv = NS_NewURI(getter_AddRefs(uri), aSpec);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    rv = NS_OpenURI(getter_AddRefs(inputStream), uri);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsIOutputStream> outputStream;
-    rv = NS_NewLocalFileOutputStream(getter_AddRefs(outputStream), aLocalFile);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    char buf[JAR_READ_BUFFER_SIZE];
-    while (true) {
-        uint32_t read;
-        uint32_t written;
-
-        rv = inputStream->Read(buf, JAR_READ_BUFFER_SIZE, &read);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        rv = outputStream->Write(buf, read, &written);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        if (written != read) {
-            return NS_ERROR_FAILURE;
-        }
-
-        if (read != JAR_READ_BUFFER_SIZE) {
-            break;
-        }
-    }
-    return NS_OK;
-}
-
-#define JAR_LAST_MODIFED_TIME "jar-last-modified-time"
-
-void ExtractFontsFromJar(nsIFile* aLocalDir)
-{
-    bool exists;
-    bool allFontsExtracted = true;
-    nsCString jarPath;
-    int64_t jarModifiedTime;
-    uint32_t longSize;
-    char* cachedModifiedTimeBuf;
-    nsZipFind* find;
-
-    nsRefPtr<nsZipArchive> reader = Omnijar::GetReader(Omnijar::Type::GRE);
-    nsCOMPtr<nsIFile> jarFile = Omnijar::GetPath(Omnijar::Type::GRE);
-
-    Omnijar::GetURIString(Omnijar::Type::GRE, jarPath);
-    jarFile->GetLastModifiedTime(&jarModifiedTime);
-
-    mozilla::scache::StartupCache* cache = mozilla::scache::StartupCache::GetSingleton();
-    if (cache && NS_SUCCEEDED(cache->GetBuffer(JAR_LAST_MODIFED_TIME, &cachedModifiedTimeBuf, &longSize))
-        && longSize == sizeof(int64_t)) {
-        if (jarModifiedTime < *((int64_t*) cachedModifiedTimeBuf)) {
-            return;
-        }
-    }
-
-    aLocalDir->Exists(&exists);
-    if (!exists) {
-        aLocalDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
-    }
-
-    static const char* sJarSearchPaths[] = {
-        "res/fonts/*.ttf$",
-    };
-
-    for (int i = 0; i < ArrayLength(sJarSearchPaths); i++) {
-        reader->FindInit(sJarSearchPaths[i], &find);
-        while (true) {
-            const char* tmpPath;
-            uint16_t len;
-            find->FindNext(&tmpPath, &len);
-            if (!tmpPath) {
-                break;
-            }
-
-            nsCString path(tmpPath, len);
-            nsCOMPtr<nsIFile> localFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-            if (NS_FAILED(localFile->InitWithFile(aLocalDir))) {
-                allFontsExtracted = false;
-                continue;
-            }
-
-            int32_t lastSlash = path.RFindChar('/');
-            nsCString fileName;
-            if (lastSlash == kNotFound) {
-                fileName = path;
-            } else {
-                fileName = Substring(path, lastSlash + 1);
-            }
-            if (NS_FAILED(localFile->AppendNative(fileName))) {
-                allFontsExtracted = false;
-                continue;
-            }
-            int64_t lastModifiedTime;
-            localFile->Exists(&exists);
-            localFile->GetLastModifiedTime(&lastModifiedTime);
-            if (!exists || lastModifiedTime < jarModifiedTime) {
-                nsCString spec;
-                spec.Append(jarPath);
-                spec.Append(path);
-                if (NS_FAILED(CopyFromUriToFile(spec, localFile))) {
-                    localFile->Remove(true);
-                    allFontsExtracted = false;
-                }
-            }
-        }
-    }
-    if (allFontsExtracted && cache) {
-        cache->PutBuffer(JAR_LAST_MODIFED_TIME, (char*)&jarModifiedTime, sizeof(int64_t));
-    }
-}
-
-#endif
-
 void
 gfxFT2FontList::FindFonts()
 {
@@ -1011,7 +1159,7 @@ gfxFT2FontList::FindFonts()
                                              FindExInfoStandard,
                                              &results,
                                              FindExSearchNameMatch,
-                                             NULL,
+                                             nullptr,
                                              0);
             bool moreFiles = handle != INVALID_HANDLE_VALUE;
             while (moreFiles) {
@@ -1071,24 +1219,25 @@ gfxFT2FontList::FindFonts()
         // if we can't find/read the font directory, we are doomed!
         NS_RUNTIMEABORT("Could not read the system fonts directory");
     }
+#endif // XP_WIN && ANDROID
 
-    // look for fonts shipped with the product
-    nsCOMPtr<nsIFile> localDir;
-    nsresult rv = NS_GetSpecialDirectory(NS_APP_RES_DIR,
-                                         getter_AddRefs(localDir));
-    if (NS_SUCCEEDED(rv) && NS_SUCCEEDED(localDir->Append(NS_LITERAL_STRING("fonts")))) {
-        ExtractFontsFromJar(localDir);
-        nsCString localPath;
-        rv = localDir->GetNativePath(localPath);
-        if (NS_SUCCEEDED(rv)) {
-            FindFontsInDir(localPath, &fnc);
-        }
+    // Look for fonts stored in omnijar, unless we're on a low-memory
+    // device where we don't want to spend the RAM to decompress them.
+    // (Prefs may disable this, or force-enable it even with low memory.)
+    bool lowmem;
+    nsCOMPtr<nsIMemory> mem = nsMemory::GetGlobalMemoryService();
+    if ((NS_SUCCEEDED(mem->IsLowMemoryPlatform(&lowmem)) && !lowmem &&
+         Preferences::GetBool("gfx.bundled_fonts.enabled")) ||
+        Preferences::GetBool("gfx.bundled_fonts.force-enabled")) {
+        FindFontsInOmnijar(&fnc);
     }
 
-    // look for locally-added fonts in the profile
-    rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_LOCAL_50_DIR,
-                                getter_AddRefs(localDir));
-    if (NS_SUCCEEDED(rv)) {
+    // look for locally-added fonts in a "fonts" subdir of the profile
+    nsCOMPtr<nsIFile> localDir;
+    nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_LOCAL_50_DIR,
+                                         getter_AddRefs(localDir));
+    if (NS_SUCCEEDED(rv) &&
+        NS_SUCCEEDED(localDir->Append(NS_LITERAL_STRING("fonts")))) {
         nsCString localPath;
         rv = localDir->GetNativePath(localPath);
         if (NS_SUCCEEDED(rv)) {
@@ -1100,7 +1249,6 @@ gfxFT2FontList::FindFonts()
     // and marking "simple" families.
     // Passing non-null userData here says that we want faces to be sorted.
     mFontFamilies.Enumerate(FinalizeFamilyMemberList, this);
-#endif // XP_WIN && ANDROID
 }
 
 #ifdef ANDROID
@@ -1120,6 +1268,7 @@ gfxFT2FontList::FindFontsInDir(const nsCString& aDir, FontNameCache *aFNC)
         "DroidSansThai.ttf",
         "MTLmr3m.ttf",
         "MTLc3m.ttf",
+        "NanumGothic.ttf",
         "DroidSansJapanese.ttf",
         "DroidSansFallback.ttf"
     };
@@ -1129,8 +1278,8 @@ gfxFT2FontList::FindFontsInDir(const nsCString& aDir, FontNameCache *aFNC)
         return;
     }
 
-    struct dirent *ent = NULL;
-    while ((ent = readdir(d)) != NULL) {
+    struct dirent *ent = nullptr;
+    while ((ent = readdir(d)) != nullptr) {
         int namelen = strlen(ent->d_name);
         if (namelen <= 4) {
             // cannot be a usable font filename
@@ -1180,8 +1329,7 @@ gfxFT2FontList::AppendFaceFromFontListEntry(const FontListEntry& aFLE,
         }
         family->AddFontEntry(fe);
 
-        // this depends on the entry having been added to its family
-        fe->CheckForBrokenFont();
+        fe->CheckForBrokenFont(family);
     }
 }
 
@@ -1267,8 +1415,8 @@ gfxFT2FontList::LookupLocalFont(const gfxProxyFontEntry *aProxyEntry,
     return data.mFontEntry;
 }
 
-gfxFontEntry*
-gfxFT2FontList::GetDefaultFont(const gfxFontStyle* aStyle, bool& aNeedsBold)
+gfxFontFamily*
+gfxFT2FontList::GetDefaultFont(const gfxFontStyle* aStyle)
 {
 #ifdef XP_WIN
     HGDIOBJ hGDI = ::GetStockObject(SYSTEM_FONT);
@@ -1276,14 +1424,19 @@ gfxFT2FontList::GetDefaultFont(const gfxFontStyle* aStyle, bool& aNeedsBold)
     if (hGDI && ::GetObjectW(hGDI, sizeof(logFont), &logFont)) {
         nsAutoString resolvedName;
         if (ResolveFontName(nsDependentString(logFont.lfFaceName), resolvedName)) {
-            return FindFontForFamily(resolvedName, aStyle, aNeedsBold);
+            return FindFamily(resolvedName);
         }
     }
-#elif defined(ANDROID)
+#elif defined(MOZ_WIDGET_GONK)
+    nsAutoString resolvedName;
+    if (ResolveFontName(NS_LITERAL_STRING("Fira Sans OT"), resolvedName)) {
+        return FindFamily(resolvedName);
+    }
+#elif defined(MOZ_WIDGET_ANDROID)
     nsAutoString resolvedName;
     if (ResolveFontName(NS_LITERAL_STRING("Roboto"), resolvedName) ||
         ResolveFontName(NS_LITERAL_STRING("Droid Sans"), resolvedName)) {
-        return FindFontForFamily(resolvedName, aStyle, aNeedsBold);
+        return FindFamily(resolvedName);
     }
 #endif
     /* TODO: what about Qt or other platforms that may use this? */
