@@ -323,10 +323,15 @@ pref("offline.startup_state",            0);
 pref("offline.send.unsent_messages",            0);
 pref("offline.download.download_messages",  0);
 
+// allow offline web apps to store data but ask for permission by default
+pref("offline-apps.allow_by_default", false);
 pref("browser.offline-apps.notify", true);
 
 pref("browser.formfill.expire_days",        180);
 
+// bug 1005566 - Disable seer until properly supported
+pref("network.seer.enabled", false);
+
 pref("mail.biff.show_new_alert",     true);
 
 pref("mailnews.ui.deleteMarksRead", true);
@@ -870,6 +875,7 @@ pref("services.sync.prefs.sync.mailnews.display.prefer_plaintext", true);
 pref("services.sync.prefs.sync.mailnews.display.show_all_body_parts_menu", true);
 pref("services.sync.prefs.sync.mailnews.emptyTrash.dontAskAgain", true);
 pref("services.sync.prefs.sync.mailnews.filters.confirm_delete", true);
+pref("services.sync.prefs.sync.mailnews.forward_header_originalmessage", true);
 pref("services.sync.prefs.sync.mailnews.headers.extraExpandedHeaders", true);
 pref("services.sync.prefs.sync.mailnews.headers.showMessageId", true);
 pref("services.sync.prefs.sync.mailnews.headers.showOrganization", true);
