@@ -553,6 +553,8 @@ function Startup()
 
   // hook up UI through progress listener
   getBrowser().addProgressListener(window.XULBrowserWindow);
+  // setup the search service DOMLinkAdded listener
+  getBrowser().addEventListener("DOMLinkAdded", BrowserSearch, false);
   // hook up drag'n'drop
   getBrowser().droppedLinkHandler = handleDroppedLink;
 
@@ -748,6 +750,8 @@ function Shutdown()
   // shut down browser access support
   window.browserDOMWindow = null;
 
+  getBrowser().removeEventListener("DOMLinkAdded", BrowserSearch, false);
+
   try {
     getBrowser().removeProgressListener(window.XULBrowserWindow);
   } catch (ex) {
@@ -1044,6 +1048,18 @@ function BrowserHome(aEvent)
 }
 
 const BrowserSearch = {
+  handleEvent: function (event) { // "DOMLinkAdded" event
+    var link = event.originalTarget;
+
+    var isSearch = /(?:^|\s)search(?:\s|$)/i.test(link.rel) && link.title &&
+                   /^(https?|ftp):/i.test(link.href) &&
+                   /(?:^|\s)application\/opensearchdescription\+xml(?:;?.*)$/i.test(link.type);
+
+    if (isSearch) {
+      this.addEngine(link, link.ownerDocument);
+    }
+  },
+
   addEngine: function(engine, targetDoc) {
     if (!this.searchBar)
       return;
@@ -1953,27 +1969,29 @@ function URLBarSetURI(aURI, aValid) {
 function losslessDecodeURI(aURI) {
   var value = aURI.spec;
   // Try to decode as UTF-8 if there's no encoding sequence that we would break.
-  if (!/%25(?:3B|2F|3F|3A|40|26|3D|2B|24|2C|23)/i.test(value))
-    try {
-      value = decodeURI(value)
-                // decodeURI decodes %25 to %, which creates unintended
-                // encoding sequences. Re-encode it, unless it's part of
-                // a sequence that survived decodeURI, i.e. one for:
-                // ';', '/', '?', ':', '@', '&', '=', '+', '$', ',', '#'
-                // (RFC 3987 section 3.2)
-                .replace(/%(?!3B|2F|3F|3A|40|26|3D|2B|24|2C|23)/ig,
-                         encodeURIComponent);
-    } catch (e) {}
+  if (!/%25(?:3B|2F|3F|3A|40|26|3D|2B|24|2C|23)/i.test(value)) {
+    const textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
+                                   .getService(Components.interfaces.nsITextToSubURI);
+    var unescaped = textToSubURI.unEscapeURIForUI(aURI.originCharset, value);
+    // unEscapeURIForUI decodes %25 to %, which creates unintended
+    // encoding sequences. Re-encode it, unless it's part of
+    // a sequence that survived unEscapeURIForUI, i.e. one for:
+    // ';', '/', '?', ':', '@', '&', '=', '+', '$', ',', '#'
+    // (RFC 3987 section 3.2)
+    if (unescaped != value)
+      value = unescaped.replace(/%(?!3B|2F|3F|3A|40|26|3D|2B|24|2C|23)/ig,
+                                encodeURIComponent);
+  }
 
   // Encode invisible characters (soft hyphen, zero-width space, BOM,
   // line and paragraph separator, word joiner, invisible times,
-  // invisible separator, object replacement character) (bug 452979)
+  // invisible separator, object replacement character,
+  // C0/C1 controls). (bug 452979, bug 909264)
   // Encode bidirectional formatting characters.
   // (RFC 3987 sections 3.2 and 4.1 paragraph 6)
   // Re-encode whitespace so that it doesn't get eaten away
   // by the location bar (bug 410726).
-  return value.replace(/[\x09-\x0d\x1c-\x1f\u00ad\u200b\u200e\u200f\u2028-\u202e\u2060\u2062\u2063\ufeff\ufffc]/g,
-                        encodeURIComponent);
+  return value.replace(/[\u0000-\u001f\u007f-\u00a0\u00ad\u034f\u061c\u115f\u1160\u17b4\u17b5\u180b-\u180d\u200b\u200e\u200f\u2028-\u202e\u2060-\u206f\u3164\udb40-\udb43\udc00-\udfff\ufe00-\ufe0f\ufeff\uffa0\ufff0-\ufff8\ufffc]|i\ud834[\udd73-\udd7a]/g, encodeURIComponent);
 }
 
 /**
