@@ -114,7 +114,8 @@ function Recipients2CompFields(msgCompFields)
           case "addr_bcc"   :
           case "addr_reply" :
             try {
-              recipient = gMimeHeaderParser.reformatUnquotedAddresses(fieldValue);
+              let headerParser = MailServices.headerParser;
+              recipient = [headerParser.makeMimeAddress(fullValue.name, fullValue.email) for (fullValue of headerParser.makeFromDisplayAddress(fieldValue, {}))].join(", ");
             } catch (ex) {recipient = fieldValue;}
             break;
         }
@@ -152,8 +153,6 @@ function Recipients2CompFields(msgCompFields)
 function CompFields2Recipients(msgCompFields)
 {
   if (msgCompFields) {
-    gMimeHeaderParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
-
     var listbox = document.getElementById('addressingWidget');
     var newListBoxNode = listbox.cloneNode(false);
     var listBoxColsClone = listbox.firstChild.cloneNode(true);
@@ -207,8 +206,6 @@ function CompFields2Recipients(msgCompFields)
     // CompFields2Recipients is called whenever a user replies or edits an existing message.
     // We want to add all of the recipients for this message to the ignore list for spell check 
     addRecipientsToIgnoreList((gCurrentIdentity ? gCurrentIdentity.identityName + ', ' : '') + msgTo + ', ' + msgCC + ', ' + msgBCC);
-
-    gMimeHeaderParser = null; //Release the mime parser
   }
 }
 
@@ -265,19 +262,8 @@ function awSetInputAndPopupFromArray(inputArray, popupValue, parentNode, templat
 {
   if (popupValue)
   {
-    var recipient;
-    for (var index = 0; index < inputArray.length; index++)
-    {
-      recipient = null;
-      if (gMimeHeaderParser)
-        try {
-          recipient =
-            gMimeHeaderParser.unquotePhraseOrAddrWString(inputArray[index], true);
-        } catch (ex) {};
-      if (!recipient)
-        recipient = inputArray[index];
+    for (let recipient of inputArray)
       _awSetInputAndPopup(recipient, popupValue, parentNode, templateNode);
-    }
   }
 }
 
